@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtConcurrent module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 /*!
     \namespace QtConcurrent
@@ -144,6 +109,7 @@
 /*!
     \page qtconcurrentmap.html
     \title Concurrent Map and Map-Reduce
+    \brief Transforming values from a sequence and combining them, all in parallel.
     \ingroup thread
 
     The QtConcurrent::map(), QtConcurrent::mapped() and
@@ -203,6 +169,18 @@
     return any results via QFuture. However, you can still use QFuture and
     QFutureWatcher to monitor the status of the map.
 
+    \section2 Concurrent Mapped and Continuations
+
+    The result of QtConcurrent::mapped() call is a QFuture that contains
+    multiple results. When attaching a \c {.then()} continuation to such
+    QFuture, make sure to use a continuation that takes QFuture as a parameter,
+    otherwise only the first result will be processed:
+
+    \snippet code/src_concurrent_qtconcurrentmap.cpp 18
+
+    In this example \c {badFuture} will only print a single result, while
+    \c {goodFuture} will print all results.
+
     \section1 Concurrent Map-Reduce
 
     QtConcurrent::mappedReduced() is similar to QtConcurrent::mapped(), but
@@ -277,9 +255,7 @@
 
     \snippet code/src_concurrent_qtconcurrentmap.cpp 14
 
-    For the reduce function, function objects are not directly
-    supported. Function objects can, however, be used
-    when the type of the reduction result is explicitly specified:
+    Function objects are also supported for the reduce function:
 
     \snippet code/src_concurrent_qtconcurrentmap.cpp 11
 
@@ -297,9 +273,7 @@
 
     \snippet code/src_concurrent_qtconcurrentmap.cpp 16
 
-    For the reduce function, lambda expressions are not directly supported.
-    Lambda expressions can, however, be used when the type of the reduction
-    result is explicitly specified:
+    You can also pass a lambda as a reduce object:
 
     \snippet code/src_concurrent_qtconcurrentmap.cpp 17
 
