@@ -1,6 +1,7 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:provides-trusted-directory-paths
 
 #include "qstandardpaths.h"
 
@@ -594,7 +595,7 @@ QString QStandardPaths::displayName(StandardLocation type)
     case GenericStateLocation:
         return QCoreApplication::translate("QStandardPaths", "Shared State");
     case DownloadLocation:
-        return QCoreApplication::translate("QStandardPaths", "Download");
+        return QCoreApplication::translate("QStandardPaths", "Downloads");
     case AppDataLocation:
     case AppConfigLocation:
         return QCoreApplication::translate("QStandardPaths", "Application Configuration");
