@@ -1,49 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qeventdispatcher_win_p.h"
 
 #include "qcoreapplication.h"
 #include <private/qsystemlibrary_p.h>
 #include "qoperatingsystemversion.h"
-#include "qpair.h"
 #include "qset.h"
 #include "qsocketnotifier.h"
 #include "qvarlengtharray.h"
@@ -52,9 +15,9 @@
 #include "qcoreapplication_p.h"
 #include <private/qthread_p.h>
 
-QT_BEGIN_NAMESPACE
+#include "q26numeric.h"
 
-extern uint qGlobalPostedEventsCount();
+QT_BEGIN_NAMESPACE
 
 #ifndef TIME_KILL_SYNCHRONOUS
 #  define TIME_KILL_SYNCHRONOUS 0x0100
@@ -94,6 +57,13 @@ class QEventDispatcherWin32Private;
 
 LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPARAM lp);
 
+static quint64 qt_msectime()
+{
+    using namespace std::chrono;
+    auto t = duration_cast<milliseconds>(steady_clock::now().time_since_epoch());
+    return t.count();
+}
+
 QEventDispatcherWin32Private::QEventDispatcherWin32Private()
     : interrupt(false), internalHwnd(0),
       sendPostedEventsTimerId(0), wakeUps(0),
@@ -137,11 +107,7 @@ LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPA
     if (dispatcher->filterNativeEvent(QByteArrayLiteral("windows_dispatcher_MSG"), &msg, &result))
         return result;
 
-#ifdef GWLP_USERDATA
     auto q = reinterpret_cast<QEventDispatcherWin32 *>(GetWindowLongPtr(hwnd, GWLP_USERDATA));
-#else
-    auto q = reinterpret_cast<QEventDispatcherWin32 *>(GetWindowLong(hwnd, GWL_USERDATA));
-#endif
     QEventDispatcherWin32Private *d = nullptr;
     if (q != nullptr)
         d = q->d_func();
@@ -171,7 +137,7 @@ LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPA
             QSNDict *sn_vec[4] = { &d->sn_read, &d->sn_write, &d->sn_except, &d->sn_read };
             QSNDict *dict = sn_vec[type];
 
-            QSockNot *sn = dict ? dict->value(wp) : 0;
+            QSockNot *sn = dict ? dict->value(qintptr(wp)) : 0;
             if (sn == nullptr) {
                 d->postActivateSocketNotifiers();
             } else {
@@ -327,24 +293,52 @@ static HWND qt_create_internal_window(const QEventDispatcherWin32 *eventDispatch
         return 0;
     }
 
-#ifdef GWLP_USERDATA
     SetWindowLongPtr(wnd, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(eventDispatcher));
-#else
-    SetWindowLong(wnd, GWL_USERDATA, reinterpret_cast<LONG>(eventDispatcher));
-#endif
 
     return wnd;
 }
 
-static void calculateNextTimeout(WinTimerInfo *t, quint64 currentTime)
+static ULONG calculateNextTimeout(WinTimerInfo *t, quint64 currentTime)
 {
-    uint interval = t->interval;
-    if ((interval >= 20000u && t->timerType != Qt::PreciseTimer) || t->timerType == Qt::VeryCoarseTimer) {
-        // round the interval, VeryCoarseTimers only have full second accuracy
-        interval = ((interval + 500)) / 1000 * 1000;
+    qint64 interval = t->interval;
+    ULONG tolerance = TIMERV_DEFAULT_COALESCING;
+    switch (t->timerType) {
+    case Qt::PreciseTimer:
+        // high precision timer is based on millisecond precision
+        // so no adjustment is necessary
+        break;
+
+    case Qt::CoarseTimer:
+        // this timer has up to 5% coarseness
+        // so our boundaries are 20 ms and 20 s
+        // below 20 ms, 5% inaccuracy is below 1 ms, so we convert to high precision
+        // above 20 s, 5% inaccuracy is above 1 s, so we convert to VeryCoarseTimer
+        if (interval >= 20000) {
+            t->timerType = Qt::VeryCoarseTimer;
+        } else if (interval <= 20) {
+            // no adjustment necessary
+            t->timerType = Qt::PreciseTimer;
+            break;
+        } else {
+            tolerance = interval / 20;
+            break;
+        }
+        Q_FALLTHROUGH();
+    case Qt::VeryCoarseTimer:
+        // the very coarse timer is based on full second precision,
+        // so we round to closest second (but never to zero)
+        tolerance = 1000;
+        if (interval < 1000)
+            interval = 1000;
+        else
+            interval = (interval + 500) / 1000 * 1000;
+        currentTime = currentTime / 1000 * 1000;
+        break;
     }
+
     t->interval = interval;
     t->timeout = currentTime + interval;
+    return tolerance;
 }
 
 void QEventDispatcherWin32Private::registerTimer(WinTimerInfo *t)
@@ -354,13 +348,19 @@ void QEventDispatcherWin32Private::registerTimer(WinTimerInfo *t)
     Q_Q(QEventDispatcherWin32);
 
     bool ok = false;
-    calculateNextTimeout(t, qt_msectime());
-    uint interval = t->interval;
+    ULONG tolerance = calculateNextTimeout(t, qt_msectime());
+    uint interval = q26::saturate_cast<uint>(t->interval);
+    if (interval != t->interval) {
+        // Now we'll need to post the timer event at each second timeout.
+        interval = q26::saturate_cast<uint>(t->interval / 2 + 1);
+        t->usesExtendedInterval = true;
+        t->isSecondTimeout = false;
+    }
     if (interval == 0u) {
         // optimization for single-shot-zero-timer
         QCoreApplication::postEvent(q, new QZeroTimerEvent(t->timerId));
         ok = true;
-    } else if (interval < 20u || t->timerType == Qt::PreciseTimer) {
+    } else if (tolerance == TIMERV_DEFAULT_COALESCING) {
         // 3/2016: Although MSDN states timeSetEvent() is deprecated, the function
         // is still deemed to be the most reliable precision timer.
         t->fastTimerId = timeSetEvent(interval, 1, qt_fast_timer_proc, DWORD_PTR(t),
@@ -370,8 +370,10 @@ void QEventDispatcherWin32Private::registerTimer(WinTimerInfo *t)
 
     if (!ok) {
         // user normal timers for (Very)CoarseTimers, or if no more multimedia timers available
-        ok = SetTimer(internalHwnd, t->timerId, interval, 0);
+        ok = SetCoalescableTimer(internalHwnd, t->timerId, interval, nullptr, tolerance);
     }
+    if (!ok)
+        ok = SetTimer(internalHwnd, t->timerId, interval, nullptr);
 
     if (!ok)
         qErrnoWarning("QEventDispatcherWin32::registerTimer: Failed to create a timer");
@@ -396,6 +398,12 @@ void QEventDispatcherWin32Private::sendTimerEvent(int timerId)
 {
     WinTimerInfo *t = timerDict.value(timerId);
     if (t && !t->inTimerEvent) {
+        if (t->usesExtendedInterval && !t->isSecondTimeout) {
+            // That's the case when the interval is larger than uint
+            t->isSecondTimeout = true;
+            return;
+        }
+
         // send event, but don't allow it to recurse
         t->inTimerEvent = true;
 
@@ -409,12 +417,13 @@ void QEventDispatcherWin32Private::sendTimerEvent(int timerId)
         if (t->timerId == -1) {
             delete t;
         } else {
+            t->isSecondTimeout = false;
             t->inTimerEvent = false;
         }
     }
 }
 
-void QEventDispatcherWin32Private::doWsaAsyncSelect(int socket, long event)
+void QEventDispatcherWin32Private::doWsaAsyncSelect(qintptr socket, long event)
 {
     Q_ASSERT(internalHwnd);
     // BoundsChecker may emit a warning for WSAAsyncSelect when event == 0
@@ -434,7 +443,7 @@ QEventDispatcherWin32::QEventDispatcherWin32(QObject *parent)
 }
 
 QEventDispatcherWin32::QEventDispatcherWin32(QEventDispatcherWin32Private &dd, QObject *parent)
-    : QAbstractEventDispatcher(dd, parent)
+    : QAbstractEventDispatcherV2(dd, parent)
 {
     Q_D(QEventDispatcherWin32);
 
@@ -563,7 +572,7 @@ bool QEventDispatcherWin32::processEvents(QEventLoop::ProcessEventsFlags flags)
 void QEventDispatcherWin32::registerSocketNotifier(QSocketNotifier *notifier)
 {
     Q_ASSERT(notifier);
-    int sockfd = notifier->socket();
+    qintptr sockfd = notifier->socket();
     int type = notifier->type();
 #ifndef QT_NO_DEBUG
     if (sockfd < 0) {
@@ -587,7 +596,7 @@ void QEventDispatcherWin32::registerSocketNotifier(QSocketNotifier *notifier)
         const char *t[] = { "Read", "Write", "Exception" };
     /* Variable "socket" below is a function pointer. */
         qWarning("QSocketNotifier: Multiple socket notifiers for "
-                 "same socket %d and type %s", sockfd, t[type]);
+                 "same socket %" PRIdQINTPTR " and type %s", sockfd, t[type]);
     }
 
     QSockNot *sn = new QSockNot;
@@ -631,7 +640,7 @@ void QEventDispatcherWin32::unregisterSocketNotifier(QSocketNotifier *notifier)
 {
     Q_ASSERT(notifier);
 #ifndef QT_NO_DEBUG
-    int sockfd = notifier->socket();
+    qintptr sockfd = notifier->socket();
     if (sockfd < 0) {
         qWarning("QEventDispatcherWin32::unregisterSocketNotifier: invalid socket identifier");
         return;
@@ -648,7 +657,7 @@ void QEventDispatcherWin32::doUnregisterSocketNotifier(QSocketNotifier *notifier
 {
     Q_D(QEventDispatcherWin32);
     int type = notifier->type();
-    int sockfd = notifier->socket();
+    qintptr sockfd = notifier->socket();
     Q_ASSERT(sockfd >= 0);
 
     QSFDict::iterator it = d->active_fd.find(sockfd);
@@ -676,10 +685,11 @@ void QEventDispatcherWin32::doUnregisterSocketNotifier(QSocketNotifier *notifier
     delete sn;
 }
 
-void QEventDispatcherWin32::registerTimer(int timerId, qint64 interval, Qt::TimerType timerType, QObject *object)
+void QEventDispatcherWin32::registerTimer(Qt::TimerId timerId, Duration interval,
+                                          Qt::TimerType timerType, QObject *object)
 {
 #ifndef QT_NO_DEBUG
-    if (timerId < 1 || interval < 0 || !object) {
+    if (qToUnderlying(timerId) < 1 || interval.count() < 0 || !object) {
         qWarning("QEventDispatcherWin32::registerTimer: invalid arguments");
         return;
     }
@@ -696,24 +706,29 @@ void QEventDispatcherWin32::registerTimer(int timerId, qint64 interval, Qt::Time
     if (d->closingDown)
         return;
 
+    using namespace std::chrono;
+
     WinTimerInfo *t = new WinTimerInfo;
     t->dispatcher = this;
-    t->timerId  = timerId;
-    t->interval = interval;
+    t->timerId  = qToUnderlying(timerId);
+    // Windows does not have a ns-resolution timer
+    t->interval = ceil<milliseconds>(interval).count();
     t->timerType = timerType;
     t->obj  = object;
     t->inTimerEvent = false;
     t->fastTimerId = 0;
+    t->usesExtendedInterval = false;
+    t->isSecondTimeout = false;
 
     d->registerTimer(t);
 
     d->timerDict.insert(t->timerId, t);          // store timers in dict
 }
 
-bool QEventDispatcherWin32::unregisterTimer(int timerId)
+bool QEventDispatcherWin32::unregisterTimer(Qt::TimerId timerId)
 {
 #ifndef QT_NO_DEBUG
-    if (timerId < 1) {
+    if (qToUnderlying(timerId) < 1) {
         qWarning("QEventDispatcherWin32::unregisterTimer: invalid argument");
         return false;
     }
@@ -725,7 +740,7 @@ bool QEventDispatcherWin32::unregisterTimer(int timerId)
 
     Q_D(QEventDispatcherWin32);
 
-    WinTimerInfo *t = d->timerDict.take(timerId);
+    WinTimerInfo *t = d->timerDict.take(qToUnderlying(timerId));
     if (!t)
         return false;
 
@@ -764,50 +779,55 @@ bool QEventDispatcherWin32::unregisterTimers(QObject *object)
     return true;
 }
 
-QList<QEventDispatcherWin32::TimerInfo>
-QEventDispatcherWin32::registeredTimers(QObject *object) const
+QList<QEventDispatcherWin32::TimerInfoV2>
+QEventDispatcherWin32::timersForObject(QObject *object) const
 {
 #ifndef QT_NO_DEBUG
     if (!object) {
         qWarning("QEventDispatcherWin32:registeredTimers: invalid argument");
-        return QList<TimerInfo>();
+        return QList<TimerInfoV2>();
     }
 #endif
 
     Q_D(const QEventDispatcherWin32);
-    QList<TimerInfo> list;
-    for (WinTimerInfo *t : qAsConst(d->timerDict)) {
+    QList<TimerInfoV2> list;
+    for (WinTimerInfo *t : std::as_const(d->timerDict)) {
         Q_ASSERT(t);
-        if (t->obj == object)
-            list << TimerInfo(t->timerId, t->interval, t->timerType);
+        if (t->obj == object) {
+            list << TimerInfoV2{std::chrono::milliseconds{t->interval},
+                                Qt::TimerId{t->timerId},
+                                t->timerType};
+        }
     }
     return list;
 }
 
-int QEventDispatcherWin32::remainingTime(int timerId)
+QEventDispatcherWin32::Duration QEventDispatcherWin32::remainingTime(Qt::TimerId timerId) const
 {
 #ifndef QT_NO_DEBUG
-    if (timerId < 1) {
+    if (qToUnderlying(timerId) < 1) {
         qWarning("QEventDispatcherWin32::remainingTime: invalid argument");
-        return -1;
+        return Duration::min();
     }
 #endif
 
-    Q_D(QEventDispatcherWin32);
+    Q_D(const QEventDispatcherWin32);
 
-    quint64 currentTime = qt_msectime();
-
-    WinTimerInfo *t = d->timerDict.value(timerId);
+    WinTimerInfo *t = d->timerDict.value(qToUnderlying(timerId));
     if (t) {
         // timer found, return time to wait
-        return t->timeout > currentTime ? t->timeout - currentTime : 0;
+        using namespace std::chrono;
+        using namespace std::chrono_literals;
+        const auto currentTimeMs = duration_cast<milliseconds>(steady_clock::now().time_since_epoch());
+        const auto timeoutMs = t->timeout * 1ms;
+        return timeoutMs > currentTimeMs ? milliseconds{timeoutMs - currentTimeMs} : 0ms;
     }
 
 #ifndef QT_NO_DEBUG
-    qWarning("QEventDispatcherWin32::remainingTime: timer id %d not found", timerId);
+    qWarning("QEventDispatcherWin32::remainingTime: timer id %d not found", qToUnderlying(timerId));
 #endif
 
-    return -1;
+    return Duration::min();
 }
 
 void QEventDispatcherWin32::wakeUp()
@@ -844,7 +864,7 @@ void QEventDispatcherWin32::closingDown()
     Q_ASSERT(d->active_fd.isEmpty());
 
     // clean up any timers
-    for (WinTimerInfo *t : qAsConst(d->timerDict))
+    for (WinTimerInfo *t : std::as_const(d->timerDict))
         d->unregisterTimer(t);
     d->timerDict.clear();
 
@@ -911,3 +931,5 @@ HWND QEventDispatcherWin32::internalHwnd()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qeventdispatcher_win_p.cpp"
