@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qmetaobjectbuilder_p.h"
 
@@ -77,19 +41,16 @@ QT_BEGIN_NAMESPACE
     \value AllPrimaryMembers Add everything except the class name, super class, and static metacall function.
 */
 
-// copied from moc's generator.cpp
-namespace QtPrivate {
-Q_CORE_EXPORT bool isBuiltinType(const QByteArray &type)
+static bool isBuiltinType(QByteArrayView type)
 {
     int id = QMetaType::fromName(type).id();
     if (!id && !type.isEmpty() && type != "void")
         return false;
     return (id < QMetaType::User);
 }
-} // namespace QtPrivate
 
 // copied from qmetaobject.cpp
-[[maybe_unused]] static inline const QMetaObjectPrivate *priv(const uint* data)
+[[maybe_unused]] static inline const QMetaObjectPrivate *qmobPriv(const uint* data)
 { return reinterpret_cast<const QMetaObjectPrivate*>(data); }
 
 class QMetaMethodBuilderPrivate
@@ -189,16 +150,24 @@ class QMetaEnumBuilderPrivate
 {
 public:
     QMetaEnumBuilderPrivate(const QByteArray &_name)
-        : name(_name), enumName(_name), isFlag(false), isScoped(false)
+        : name(_name), enumName(_name)
     {
     }
 
     QByteArray name;
     QByteArray enumName;
-    bool isFlag;
-    bool isScoped;
+    QMetaType metaType;
     QList<QByteArray> keys;
-    QList<int> values;
+    QList<quint64> values;
+    QFlags<EnumFlags> flags = {};
+
+    int addKey(const QByteArray &name, quint64 value)
+    {
+        int index = keys.size();
+        keys += name;
+        values += value;
+        return index;
+    }
 };
 Q_DECLARE_TYPEINFO(QMetaEnumBuilderPrivate, Q_RELOCATABLE_TYPE);
 
@@ -212,8 +181,6 @@ public:
         staticMetacallFunction = nullptr;
     }
 
-    bool hasRevisionedMethods() const;
-
     QByteArray className;
     const QMetaObject *superClass;
     QMetaObjectBuilder::StaticMetacallFunction staticMetacallFunction;
@@ -224,18 +191,9 @@ public:
     QList<QByteArray> classInfoValues;
     std::vector<QMetaEnumBuilderPrivate> enumerators;
     QList<const QMetaObject *> relatedMetaObjects;
-    int flags;
+    MetaObjectFlags flags;
 };
 
-bool QMetaObjectBuilderPrivate::hasRevisionedMethods() const
-{
-    for (const auto &method : methods) {
-        if (method.revision)
-            return true;
-    }
-    return false;
-}
-
 /*!
     Constructs a new QMetaObjectBuilder.
 */
@@ -325,7 +283,7 @@ void QMetaObjectBuilder::setSuperClass(const QMetaObject *meta)
 */
 MetaObjectFlags QMetaObjectBuilder::flags() const
 {
-    return MetaObjectFlags(d->flags);
+    return d->flags;
 }
 
 /*!
@@ -472,6 +430,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QMetaMethod &prototype)
     method.setAccess(prototype.access());
     method.setAttributes(prototype.attributes());
     method.setRevision(prototype.revision());
+    method.setConst(prototype.isConst());
     return method;
 }
 
@@ -558,7 +517,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QMetaMethod &prototy
 QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QByteArray &name, const QByteArray &type,
                                                      int notifierId)
 {
-    return addProperty(name, type, QMetaType::fromName(name), notifierId);
+    return addProperty(name, type, QMetaType::fromName(type), notifierId);
 }
 
 /*!
@@ -633,11 +592,14 @@ QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QMetaEnum &prototype)
 {
     QMetaEnumBuilder en = addEnumerator(prototype.name());
     en.setEnumName(prototype.enumName());
+    en.setMetaType(prototype.metaType());
     en.setIsFlag(prototype.isFlag());
     en.setIsScoped(prototype.isScoped());
     int count = prototype.keyCount();
     for (int index = 0; index < count; ++index)
-        en.addKey(prototype.key(index), prototype.value(index));
+        en.addKey(prototype.key(index), prototype.value64(index).value_or(0));
+    // reset the is64Bit() flag if necessary
+    en.setIs64Bit(prototype.is64Bit());
     return en;
 }
 
@@ -740,7 +702,7 @@ void QMetaObjectBuilder::addMetaObject(const QMetaObject *prototype,
     }
 
     if ((members & RelatedMetaObjects) != 0) {
-        Q_ASSERT(priv(prototype->d.data)->revision >= 2);
+        Q_ASSERT(qmobPriv(prototype->d.data)->revision >= 2);
         const auto *objects = prototype->d.relatedMetaObjects;
         if (objects) {
             while (*objects != nullptr) {
@@ -751,7 +713,7 @@ void QMetaObjectBuilder::addMetaObject(const QMetaObject *prototype,
     }
 
     if ((members & StaticMetacall) != 0) {
-        Q_ASSERT(priv(prototype->d.data)->revision >= 6);
+        Q_ASSERT(qmobPriv(prototype->d.data)->revision >= 6);
         if (prototype->d.static_metacall)
             setStaticMetacallFunction(prototype->d.static_metacall);
     }
@@ -1062,6 +1024,9 @@ int QMetaObjectBuilder::indexOfClassInfo(const QByteArray &name)
 }
 
 // Align on a specific type boundary.
+#ifdef ALIGN
+#  undef ALIGN
+#endif
 #define ALIGN(size,type)    \
     (size) = ((size) + sizeof(type) - 1) & ~(sizeof(type) - 1)
 
@@ -1119,7 +1084,7 @@ static void writeString(char *out, int i, const QByteArray &str,
     memcpy(out + 2 * i * sizeof(uint), &offsetLen, 2 * sizeof(uint));
 
     memcpy(out + offset, str.constData(), size);
-    out[offsetOfStringdataMember + stringdataOffset + size] = '\0';
+    out[offset + size] = '\0';
 
     stringdataOffset += size + 1;
 }
@@ -1149,14 +1114,24 @@ void QMetaStringTable::writeBlob(char *out) const
     }
 }
 
-// Returns the sum of all parameters (including return type) for the given
-// \a methods. This is needed for calculating the size of the methods'
-// parameter type/name meta-data.
+// Returns the number of integers needed to store these methods' parameter type
+// infos or type names, parameter names and, if present, the method revision.
+// This is needed for calculating the size of the methods' parameter type/name
+// meta-data.
 static int aggregateParameterCount(const std::vector<QMetaMethodBuilderPrivate> &methods)
 {
     int sum = 0;
-    for (const auto &method : methods)
-        sum += method.parameterCount() + 1; // +1 for return type
+    for (const auto &method : methods) {
+        if (method.revision)
+            ++sum;
+
+        // type infos or type names; +1 for return type (constructors don't
+        // have one, so this stores a link to an empty string)
+        sum += method.parameterCount() + 1;
+
+        // parameter names (return type doesn't get one)
+        sum += method.parameterCount();
+    }
     return sum;
 }
 
@@ -1178,7 +1153,6 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     int paramsIndex;
     int enumIndex;
     int index;
-    bool hasRevisionedMethods = d->hasRevisionedMethods();
 
     // Create the main QMetaObject structure at the start of the buffer.
     QMetaObject *meta = reinterpret_cast<QMetaObject *>(buf);
@@ -1193,19 +1167,16 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     }
 
     // Populate the QMetaObjectPrivate structure.
-    QMetaObjectPrivate *pmeta
-        = reinterpret_cast<QMetaObjectPrivate *>(buf + size);
+    QMetaObjectPrivate *pmeta = buf ? reinterpret_cast<QMetaObjectPrivate *>(buf + size)
+                                    : nullptr;
     //int pmetaSize = size;
     dataIndex = MetaObjectPrivateFieldCount;
-    int methodParametersDataSize =
-            ((aggregateParameterCount(d->methods)
-             + aggregateParameterCount(d->constructors)) * 2) // types and parameter names
-            - int(d->methods.size())       // return "parameters" don't have names
-            - int(d->constructors.size()); // "this" parameters don't have names
+    int methodParametersDataSize = aggregateParameterCount(d->methods)
+             + aggregateParameterCount(d->constructors);
     if constexpr (mode == Construct) {
-        static_assert(QMetaObjectPrivate::OutputRevision == 9, "QMetaObjectBuilder should generate the same version as moc");
+        static_assert(QMetaObjectPrivate::OutputRevision == 13, "QMetaObjectBuilder should generate the same version as moc");
         pmeta->revision = QMetaObjectPrivate::OutputRevision;
-        pmeta->flags = d->flags;
+        pmeta->flags = d->flags.toInt() | AllocatedMetaObject;
         pmeta->className = 0;   // Class name is always the first string.
         //pmeta->signalCount is handled in the "output method loop" as an optimization.
 
@@ -1216,8 +1187,6 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         pmeta->methodCount = int(d->methods.size());
         pmeta->methodData = dataIndex;
         dataIndex += QMetaObjectPrivate::IntsPerMethod * int(d->methods.size());
-        if (hasRevisionedMethods)
-            dataIndex += int(d->methods.size());
         paramsIndex = dataIndex;
         dataIndex += methodParametersDataSize;
 
@@ -1235,8 +1204,6 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     } else {
         dataIndex += 2 * int(d->classInfoNames.size());
         dataIndex += QMetaObjectPrivate::IntsPerMethod * int(d->methods.size());
-        if (hasRevisionedMethods)
-            dataIndex += int(d->methods.size());
         paramsIndex = dataIndex;
         dataIndex += methodParametersDataSize;
         dataIndex += QMetaObjectPrivate::IntsPerProperty * int(d->properties.size());
@@ -1246,8 +1213,11 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
     // Allocate space for the enumerator key names and values.
     enumIndex = dataIndex;
-    for (const auto &enumerator : d->enumerators)
+    for (const auto &enumerator : d->enumerators) {
         dataIndex += 2 * enumerator.keys.size();
+        if (enumerator.flags & EnumIs64Bit)
+            dataIndex += enumerator.keys.size();
+    }
 
     // Zero terminator at the end of the data offset table.
     ++dataIndex;
@@ -1281,12 +1251,15 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
     // Output the methods in the class.
     Q_ASSERT(!buf || dataIndex == pmeta->methodData);
-    int parameterMetaTypesIndex = int(d->properties.size());
+    // property count + enum count  + 1 for metatype of this metaobject
+    int parameterMetaTypesIndex = int(d->properties.size()) + int(d->enumerators.size()) + 1;
     for (const auto &method : d->methods) {
         [[maybe_unused]] int name = strings.enter(method.name());
         int argc = method.parameterCount();
         [[maybe_unused]] int tag = strings.enter(method.tag);
         [[maybe_unused]] int attrs = method.attributes;
+        if (method.revision)
+            ++paramsIndex;
         if constexpr (mode == Construct) {
             data[dataIndex]     = name;
             data[dataIndex + 1] = argc;
@@ -1301,26 +1274,24 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         paramsIndex += 1 + argc * 2;
         parameterMetaTypesIndex += 1 + argc;
     }
-    if (hasRevisionedMethods) {
-        for (const auto &method : d->methods) {
-            if constexpr (mode == Construct)
-                data[dataIndex] = method.revision;
-            ++dataIndex;
-        }
-    }
 
     // Output the method parameters in the class.
-    Q_ASSERT(!buf || dataIndex == pmeta->methodData + int(d->methods.size()) * QMetaObjectPrivate::IntsPerMethod
-             + (hasRevisionedMethods ? int(d->methods.size()) : 0));
+    Q_ASSERT(!buf || dataIndex == pmeta->methodData + int(d->methods.size()) * QMetaObjectPrivate::IntsPerMethod);
     for (int x = 0; x < 2; ++x) {
         const std::vector<QMetaMethodBuilderPrivate> &methods = (x == 0) ? d->methods : d->constructors;
         for (const auto &method : methods) {
+            if (method.revision) {
+                if constexpr (mode == Construct)
+                    data[dataIndex] = method.revision;
+                ++dataIndex;
+            }
+
             const QList<QByteArray> paramTypeNames = method.parameterTypes();
             int paramCount = paramTypeNames.size();
             for (int i = -1; i < paramCount; ++i) {
                 const QByteArray &typeName = (i < 0) ? method.returnType : paramTypeNames.at(i);
                 [[maybe_unused]] int typeInfo;
-                if (QtPrivate::isBuiltinType(typeName))
+                if (isBuiltinType(typeName))
                     typeInfo = QMetaType::fromName(typeName).id();
                 else
                     typeInfo = IsUnresolvedType | strings.enter(typeName);
@@ -1355,7 +1326,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
         [[maybe_unused]] int flags = prop.flags;
 
-        if (!QtPrivate::isBuiltinType(prop.type))
+        if (isBuiltinType(prop.type))
             flags |= EnumOrFlag;
 
         if constexpr (mode == Construct) {
@@ -1373,26 +1344,30 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     for (const auto &enumerator : d->enumerators) {
         [[maybe_unused]] int name = strings.enter(enumerator.name);
         [[maybe_unused]] int enumName = strings.enter(enumerator.enumName);
-        [[maybe_unused]] int isFlag = enumerator.isFlag ? EnumIsFlag : 0;
-        [[maybe_unused]] int isScoped = enumerator.isScoped ? EnumIsScoped : 0;
         int count = enumerator.keys.size();
-        int enumOffset = enumIndex;
         if constexpr (mode == Construct) {
             data[dataIndex]     = name;
             data[dataIndex + 1] = enumName;
-            data[dataIndex + 2] = isFlag | isScoped;
+            data[dataIndex + 2] = enumerator.flags.toInt();
             data[dataIndex + 3] = count;
-            data[dataIndex + 4] = enumOffset;
+            data[dataIndex + 4] = enumIndex;
         }
         for (int key = 0; key < count; ++key) {
             [[maybe_unused]] int keyIndex = strings.enter(enumerator.keys[key]);
             if constexpr (mode == Construct) {
-                data[enumOffset++] = keyIndex;
-                data[enumOffset++] = enumerator.values[key];
+                data[enumIndex + 0] = keyIndex;
+                data[enumIndex + 1] = uint(enumerator.values[key]);
             }
+            enumIndex += 2;
+        }
+        bool is64Bit = enumerator.flags.testAnyFlags(EnumIs64Bit);
+        for (int key = 0; is64Bit && key < count; ++key) {
+            if constexpr (mode == Construct) {
+                data[enumIndex] = uint(enumerator.values[key] >> 32);
+            }
+            ++enumIndex;
         }
         dataIndex += QMetaObjectPrivate::IntsPerEnum;
-        enumIndex += 2 * count;
     }
 
     // Output the constructors in the class.
@@ -1412,6 +1387,8 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         }
         dataIndex += QMetaObjectPrivate::IntsPerMethod;
         paramsIndex += 1 + argc * 2;
+        if (ctor.revision)
+            ++paramsIndex;
         parameterMetaTypesIndex += argc;
     }
 
@@ -1438,37 +1415,46 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         size += sizeof(SuperData) * (d->relatedMetaObjects.size() + 1);
     }
 
-    if (d->properties.size() > 0 || d->methods.size() > 0 || d->constructors.size() > 0) {
-        ALIGN(size, QtPrivate::QMetaTypeInterface *);
-        auto types = reinterpret_cast<QtPrivate::QMetaTypeInterface **>(buf + size);
-        if constexpr (mode == Construct) {
-            meta->d.metaTypes = types;
-            for (const auto &prop : d->properties) {
-                QMetaType mt = prop.metaType;
-                *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+    ALIGN(size, QtPrivate::QMetaTypeInterface *);
+    auto types = reinterpret_cast<const QtPrivate::QMetaTypeInterface **>(buf + size);
+    if constexpr (mode == Construct) {
+        meta->d.metaTypes = types;
+        for (const auto &prop : d->properties) {
+            QMetaType mt = prop.metaType;
+            *types = mt.iface();
+            types++;
+        }
+        // add metatypes for enumerators
+        for (const auto &enumerator: d->enumerators) {
+            QMetaType mt = enumerator.metaType;
+            mt.registerType();
+            *types = mt.iface();
+            types++;
+        }
+        // add metatype interface for this metaobject - must be null
+        // as we can't know our metatype
+        *types = nullptr;
+        types++;
+        for (const auto &method: d->methods) {
+            QMetaType mt(QMetaType::fromName(method.returnType).id());
+            *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+            types++;
+            for (const auto &parameterType: method.parameterTypes()) {
+                QMetaType mt = QMetaType::fromName(parameterType);
+                *types = mt.iface();
+                types++;
+            }
+        }
+        for (const auto &constructor : d->constructors) {
+            for (const auto &parameterType : constructor.parameterTypes()) {
+                QMetaType mt = QMetaType::fromName(parameterType);
+                *types = mt.iface();
                 types++;
             }
-            for (const auto &method: d->methods) {
-                QMetaType mt(QMetaType::fromName(method.returnType).id());
-                *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
-                types++;
-                for (const auto &parameterType: method.parameterTypes()) {
-                    QMetaType mt = QMetaType::fromName(parameterType);
-                    *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
-                    types++;
-                }
-            }
-            for (const auto &constructor : d->constructors) {
-                for (const auto &parameterType : constructor.parameterTypes()) {
-                    QMetaType mt = QMetaType::fromName(parameterType);
-                    *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
-                    types++;
-                }
-            }
         }
-        // parameterMetaTypesIndex is equal to the total number of metatypes
-        size += sizeof(QMetaType) * parameterMetaTypesIndex;
     }
+    // parameterMetaTypesIndex is equal to the total number of metatypes
+    size += sizeof(QMetaType) * parameterMetaTypesIndex;
 
     // Align the final size and return it.
     ALIGN(size, void *);
@@ -1489,8 +1475,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 QMetaObject *QMetaObjectBuilder::toMetaObject() const
 {
     int size = buildMetaObject<Prepare>(d, nullptr, 0);
-    char *buf = reinterpret_cast<char *>(malloc(size));
-    memset(buf, 0, size);
+    char *buf = new (calloc(size, 1)) char[size];
     buildMetaObject<Construct>(d, buf, size);
     return reinterpret_cast<QMetaObject *>(buf);
 }
@@ -1526,274 +1511,6 @@ void QMetaObjectBuilder::setStaticMetacallFunction
     d->staticMetacallFunction = value;
 }
 
-#ifndef QT_NO_DATASTREAM
-
-/*!
-    Serializes the contents of the meta object builder onto \a stream.
-
-    \sa deserialize()
-*/
-void QMetaObjectBuilder::serialize(QDataStream &stream) const
-{
-    int index;
-
-    // Write the class and super class names.
-    stream << d->className;
-    if (d->superClass)
-        stream << QByteArray(d->superClass->className());
-    else
-        stream << QByteArray();
-
-    // Write the counts for each type of class member.
-    stream << int(d->classInfoNames.size());
-    stream << int(d->methods.size());
-    stream << int(d->properties.size());
-    stream << int(d->enumerators.size());
-    stream << int(d->constructors.size());
-    stream << int(d->relatedMetaObjects.size());
-
-    // Write the items of class information.
-    for (index = 0; index < d->classInfoNames.size(); ++index) {
-        stream << d->classInfoNames[index];
-        stream << d->classInfoValues[index];
-    }
-
-    // Write the methods.
-    for (const auto &method : d->methods) {
-        stream << method.signature;
-        stream << method.returnType;
-        stream << method.parameterNames;
-        stream << method.tag;
-        stream << method.attributes;
-        if (method.revision)
-            stream << method.revision;
-    }
-
-    // Write the properties.
-    for (const auto &property : d->properties) {
-        stream << property.name;
-        stream << property.type;
-        stream << property.flags;
-        stream << property.notifySignal;
-        stream << property.revision;
-    }
-
-    // Write the enumerators.
-    for (const auto &enumerator : d->enumerators) {
-        stream << enumerator.name;
-        stream << enumerator.isFlag;
-        stream << enumerator.isScoped;
-        stream << enumerator.keys;
-        stream << enumerator.values;
-    }
-
-    // Write the constructors.
-    for (const auto &ctor : d->constructors) {
-        stream << ctor.signature;
-        stream << ctor.returnType;
-        stream << ctor.parameterNames;
-        stream << ctor.tag;
-        stream << ctor.attributes;
-    }
-
-    // Write the related meta objects.
-    for (index = 0; index < d->relatedMetaObjects.size(); ++index) {
-        const QMetaObject *meta = d->relatedMetaObjects[index];
-        stream << QByteArray(meta->className());
-    }
-
-    // Add an extra empty QByteArray for additional data in future versions.
-    // This should help maintain backwards compatibility, allowing older
-    // versions to read newer data.
-    stream << QByteArray();
-}
-
-// Resolve a class name using the name reference map.
-static const QMetaObject *resolveClassName(const QMap<QByteArray, const QMetaObject *> &references,
-                                           const QByteArray &name)
-{
-    if (name == QByteArray("QObject"))
-        return &QObject::staticMetaObject;
-    else
-        return references.value(name, nullptr);
-}
-
-/*!
-    Deserializes a meta object builder from \a stream into
-    this meta object builder.
-
-    The \a references parameter specifies a mapping from class names
-    to QMetaObject instances for resolving the super class name and
-    related meta objects in the object that is deserialized.
-    The meta object for QObject is implicitly added to \a references
-    and does not need to be supplied.
-
-    The QDataStream::status() value on \a stream will be set to
-    QDataStream::ReadCorruptData if the input data is corrupt.
-    The status will be set to QDataStream::ReadPastEnd if the
-    input was exhausted before the full meta object was read.
-
-    \sa serialize()
-*/
-void QMetaObjectBuilder::deserialize
-        (QDataStream& stream,
-         const QMap<QByteArray, const QMetaObject *>& references)
-{
-    QByteArray name;
-    const QMetaObject *cl;
-    int index;
-
-    // Clear all members in the builder to their default states.
-    d->className.clear();
-    d->superClass = &QObject::staticMetaObject;
-    d->classInfoNames.clear();
-    d->classInfoValues.clear();
-    d->methods.clear();
-    d->properties.clear();
-    d->enumerators.clear();
-    d->constructors.clear();
-    d->relatedMetaObjects.clear();
-    d->staticMetacallFunction = nullptr;
-
-    // Read the class and super class names.
-    stream >> d->className;
-    stream >> name;
-    if (name.isEmpty()) {
-        d->superClass = nullptr;
-    } else if ((cl = resolveClassName(references, name)) != nullptr) {
-        d->superClass = cl;
-    } else {
-        stream.setStatus(QDataStream::ReadCorruptData);
-        return;
-    }
-
-    // Read the counts for each type of class member.
-    int classInfoCount, methodCount, propertyCount;
-    int enumeratorCount, constructorCount, relatedMetaObjectCount;
-    stream >> classInfoCount;
-    stream >> methodCount;
-    stream >> propertyCount;
-    stream >> enumeratorCount;
-    stream >> constructorCount;
-    stream >> relatedMetaObjectCount;
-    if (classInfoCount < 0 || methodCount < 0 ||
-        propertyCount < 0 || enumeratorCount < 0 ||
-        constructorCount < 0 || relatedMetaObjectCount < 0) {
-        stream.setStatus(QDataStream::ReadCorruptData);
-        return;
-    }
-
-    // Read the items of class information.
-    for (index = 0; index < classInfoCount; ++index) {
-        if (stream.status() != QDataStream::Ok)
-            return;
-        QByteArray value;
-        stream >> name;
-        stream >> value;
-        addClassInfo(name, value);
-    }
-
-    // Read the member methods.
-    for (index = 0; index < methodCount; ++index) {
-        if (stream.status() != QDataStream::Ok)
-            return;
-        stream >> name;
-        addMethod(name);
-        QMetaMethodBuilderPrivate &method = d->methods[index];
-        stream >> method.returnType;
-        stream >> method.parameterNames;
-        stream >> method.tag;
-        stream >> method.attributes;
-        if (method.attributes & MethodRevisioned)
-            stream >> method.revision;
-        if (method.methodType() == QMetaMethod::Constructor) {
-            // Cannot add a constructor in this set of methods.
-            stream.setStatus(QDataStream::ReadCorruptData);
-            return;
-        }
-    }
-
-    // Read the properties.
-    for (index = 0; index < propertyCount; ++index) {
-        if (stream.status() != QDataStream::Ok)
-            return;
-        QByteArray type;
-        stream >> name;
-        stream >> type;
-        addProperty(name, type);
-        QMetaPropertyBuilderPrivate &property = d->properties[index];
-        stream >> property.flags;
-        stream >> property.notifySignal;
-        if (property.notifySignal < -1 ||
-            property.notifySignal >= int(d->methods.size())) {
-            // Notify signal method index is out of range.
-            stream.setStatus(QDataStream::ReadCorruptData);
-            return;
-        }
-        if (property.notifySignal >= 0 &&
-            d->methods[property.notifySignal].methodType() != QMetaMethod::Signal) {
-            // Notify signal method index does not refer to a signal.
-            stream.setStatus(QDataStream::ReadCorruptData);
-            return;
-        }
-        stream >> property.revision;
-    }
-
-    // Read the enumerators.
-    for (index = 0; index < enumeratorCount; ++index) {
-        if (stream.status() != QDataStream::Ok)
-            return;
-        stream >> name;
-        addEnumerator(name);
-        QMetaEnumBuilderPrivate &enumerator = d->enumerators[index];
-        stream >> enumerator.isFlag;
-        stream >> enumerator.isScoped;
-        stream >> enumerator.keys;
-        stream >> enumerator.values;
-        if (enumerator.keys.size() != enumerator.values.size()) {
-            // Mismatch between number of keys and number of values.
-            stream.setStatus(QDataStream::ReadCorruptData);
-            return;
-        }
-    }
-
-    // Read the constructor methods.
-    for (index = 0; index < constructorCount; ++index) {
-        if (stream.status() != QDataStream::Ok)
-            return;
-        stream >> name;
-        addConstructor(name);
-        QMetaMethodBuilderPrivate &method = d->constructors[index];
-        stream >> method.returnType;
-        stream >> method.parameterNames;
-        stream >> method.tag;
-        stream >> method.attributes;
-        if (method.methodType() != QMetaMethod::Constructor) {
-            // The type must be Constructor.
-            stream.setStatus(QDataStream::ReadCorruptData);
-            return;
-        }
-    }
-
-    // Read the related meta objects.
-    for (index = 0; index < relatedMetaObjectCount; ++index) {
-        if (stream.status() != QDataStream::Ok)
-            return;
-        stream >> name;
-        cl = resolveClassName(references, name);
-        if (!cl) {
-            stream.setStatus(QDataStream::ReadCorruptData);
-            return;
-        }
-        addRelatedMetaObject(cl);
-    }
-
-    // Read the extra data block, which is reserved for future use.
-    stream >> name;
-}
-
-#endif // !QT_NO_DATASTREAM
-
 /*!
     \class QMetaMethodBuilder
     \inmodule QtCore
@@ -1919,6 +1636,7 @@ QList<QByteArray> QMetaMethodBuilder::parameterNames() const
 void QMetaMethodBuilder::setParameterNames(const QList<QByteArray> &value)
 {
     QMetaMethodBuilderPrivate *d = d_func();
+    Q_ASSERT(d->parameterCount() >= value.size());
     if (d)
         d->parameterNames = value;
 }
@@ -1988,7 +1706,7 @@ int QMetaMethodBuilder::attributes() const
 {
     QMetaMethodBuilderPrivate *d = d_func();
     if (d)
-        return (d->attributes >> 4);
+        return (d->attributes >> 4) & 0x7;
     else
         return 0;
 }
@@ -2001,8 +1719,32 @@ int QMetaMethodBuilder::attributes() const
 void QMetaMethodBuilder::setAttributes(int value)
 {
     QMetaMethodBuilderPrivate *d = d_func();
-    if (d)
-        d->attributes = ((d->attributes & 0x0f) | (value << 4));
+    if (d) {
+        d->attributes &= ~0x70;
+        d->attributes |= (value & 0x7) << 4;
+    }
+}
+
+/*!
+    Returns true if the method is const qualified.
+ */
+bool QMetaMethodBuilder::isConst() const
+{
+    QMetaMethodBuilderPrivate *d = d_func();
+    if (!d)
+        return false;
+    return (d->attributes & MethodIsConst);
+}
+
+void QMetaMethodBuilder::setConst(bool methodIsConst)
+{
+    QMetaMethodBuilderPrivate *d = d_func();
+    if (!d)
+        return;
+    if (methodIsConst)
+        d->attributes |= MethodIsConst;
+    else
+        d->attributes &= ~MethodIsConst;
 }
 
 /*!
@@ -2581,6 +2323,34 @@ void QMetaEnumBuilder::setEnumName(const QByteArray &alias)
         d->enumName = alias;
 }
 
+/*!
+    Returns the meta type of the enumerator.
+
+    \since 6.6
+*/
+QMetaType QMetaEnumBuilder::metaType() const
+{
+    if (QMetaEnumBuilderPrivate *d = d_func())
+        return d->metaType;
+    return QMetaType();
+}
+
+/*!
+    Sets this enumerator to have the given \c metaType. The is64Bit() flag will
+    be set to match \a metaType's size.
+
+    \since 6.6
+    \sa metaType()
+*/
+void QMetaEnumBuilder::setMetaType(QMetaType metaType)
+{
+    QMetaEnumBuilderPrivate *d = d_func();
+    if (d) {
+        d->metaType = metaType;
+        setIs64Bit(metaType.sizeOf() > 4);
+    }
+}
+
 /*!
     Returns \c true if this enumerator is used as a flag; otherwise returns
     false.
@@ -2591,7 +2361,7 @@ bool QMetaEnumBuilder::isFlag() const
 {
     QMetaEnumBuilderPrivate *d = d_func();
     if (d)
-        return d->isFlag;
+        return d->flags.toInt() & EnumIsFlag;
     else
         return false;
 }
@@ -2605,7 +2375,7 @@ void QMetaEnumBuilder::setIsFlag(bool value)
 {
     QMetaEnumBuilderPrivate *d = d_func();
     if (d)
-        d->isFlag = value;
+        d->flags.setFlag(EnumIsFlag, value);
 }
 
 /*!
@@ -2617,7 +2387,7 @@ bool QMetaEnumBuilder::isScoped() const
 {
     QMetaEnumBuilderPrivate *d = d_func();
     if (d)
-        return d->isScoped;
+        return d->flags.toInt() & EnumIsScoped;
     return false;
 }
 
@@ -2630,7 +2400,37 @@ void QMetaEnumBuilder::setIsScoped(bool value)
 {
     QMetaEnumBuilderPrivate *d = d_func();
     if (d)
-        d->isScoped = value;
+        d->flags.setFlag(EnumIsScoped, value);
+}
+
+/*!
+    Return \c true if this enumerations in this enumerator are 64-bit.
+
+    This flag is autoamtically enabled if a 64-bit value is added with addKey().
+
+    \sa setIs64Bit()
+*/
+bool QMetaEnumBuilder::is64Bit() const
+{
+    QMetaEnumBuilderPrivate *d = d_func();
+    if (d)
+        return d->flags.toInt() & EnumIs64Bit;
+    return false;
+}
+
+/*!
+    Sets this enumerator to be 64-bit wide if \a value is true. If \a value is
+    false, any stored 64-bit keys will be truncated to 32 bits.
+
+    This flag is autoamtically enabled if a 64-bit value is added with addKey().
+
+    \sa is64Bit()
+*/
+void QMetaEnumBuilder::setIs64Bit(bool value)
+{
+    QMetaEnumBuilderPrivate *d = d_func();
+    if (d)
+        d->flags.setFlag(EnumIs64Bit, value);
 }
 
 /*!
@@ -2666,15 +2466,38 @@ QByteArray QMetaEnumBuilder::key(int index) const
     Returns the value with the given \a index; or returns -1 if there
     is no such value.
 
-    \sa keyCount(), addKey(), key()
+    If this is a 64-bit enumeration (see is64Bit()), this function returns the
+    low 32-bit portion of the value. Use value64() to obtain the full value
+    instead.
+
+    \sa value64(), keyCount(), addKey(), key(), is64Bit()
 */
 int QMetaEnumBuilder::value(int index) const
+{
+    return value64(index).value_or(-1);
+}
+
+/*!
+    \since 6.9
+
+    Returns the value with the given \a index if it exists; or returns a
+    disengaged \c{std::optional} if it doesn't.
+
+    \include qmetaobject.cpp qmetaenum-32bit-signextend-64bit
+
+    \sa keyCount(), key(), addKey()
+*/
+std::optional<quint64> QMetaEnumBuilder::value64(int index) const
 {
     QMetaEnumBuilderPrivate *d = d_func();
-    if (d && index >= 0 && index < d->keys.size())
-        return d->values[index];
-    else
-        return -1;
+    if (d && index >= 0 && index < d->keys.size()) {
+        quint64 v = d->values[index];
+        if (d->flags & EnumIs64Bit)
+            return v;
+        return uint(v);     // return only the low 32 bits
+    } else {
+        return std::nullopt;
+    }
 }
 
 /*!
@@ -2687,15 +2510,33 @@ int QMetaEnumBuilder::addKey(const QByteArray &name, int value)
 {
     QMetaEnumBuilderPrivate *d = d_func();
     if (d) {
-        int index = d->keys.size();
-        d->keys += name;
-        d->values += value;
-        return index;
+        return d->addKey(name, uint(value));
     } else {
         return -1;
     }
 }
 
+/*!
+    \since 6.9
+
+    Adds a new key called \a name to this enumerator, associated
+    with \a value.  Returns the index of the new key.
+
+    Using the 64-bit version of this function automatically makes this
+    enumeration be stored as 64-bit.
+
+    \sa keyCount(), key(), value(), removeKey(), is64Bit()
+*/
+int QMetaEnumBuilder::addKey(const QByteArray &name, quint64 value)
+{
+    QMetaEnumBuilderPrivate *d = d_func();
+    if (d) {
+        setIs64Bit(true);
+        return d->addKey(name, value);
+    }
+    return -1;
+}
+
 /*!
     Removes the key at \a index from this enumerator.
 
