@@ -3,52 +3,48 @@
 // Copyright (C) 2015 Olivier Goffart <ogoffart@woboq.com>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include "qvariant.h"
-#include "qbitarray.h"
-#include "qbytearray.h"
-#include "qdatastream.h"
-#include "qdebug.h"
-#include "qmap.h"
-#include "qhash.h"
-#include "qdatetime.h"
-#if QT_CONFIG(easingcurve)
-#include "qeasingcurve.h"
-#endif
-#include "qlist.h"
-#if QT_CONFIG(regularexpression)
-#include "qregularexpression.h"
-#endif
-#include "qstring.h"
-#include "qstringlist.h"
-#include "qurl.h"
-#include "qlocale.h"
-#include "quuid.h"
+#include "qvariant_p.h"
+
+#include "private/qlocale_p.h"
+#include "qmetatype_p.h"
+
 #if QT_CONFIG(itemmodel)
 #include "qabstractitemmodel.h"
 #endif
-#ifndef QT_BOOTSTRAPPED
+#include "qbitarray.h"
+#include "qbytearray.h"
+#include "qbytearraylist.h"
 #include "qcborarray.h"
 #include "qcborcommon.h"
 #include "qcbormap.h"
-#include "qjsonvalue.h"
-#include "qjsonobject.h"
+#include "qdatastream.h"
+#include "qdatetime.h"
+#include "qdebug.h"
+#if QT_CONFIG(easingcurve)
+#include "qeasingcurve.h"
+#endif
+#include "qhash.h"
 #include "qjsonarray.h"
 #include "qjsondocument.h"
-#include "qbytearraylist.h"
-#endif
-#include "private/qlocale_p.h"
-#include "qmetatype_p.h"
-#include <qmetaobject.h>
-
-#ifndef QT_NO_GEOM_VARIANT
-#include "qsize.h"
+#include "qjsonobject.h"
+#include "qjsonvalue.h"
+#include "qline.h"
+#include "qlist.h"
+#include "qlocale.h"
+#include "qmap.h"
 #include "qpoint.h"
 #include "qrect.h"
-#include "qline.h"
+#if QT_CONFIG(regularexpression)
+#include "qregularexpression.h"
 #endif
+#include "qsize.h"
+#include "qstring.h"
+#include "qstringlist.h"
+#include "qurl.h"
+#include "quuid.h"
 
+#include <memory>
 #include <cmath>
-#include <float.h>
 #include <cstring>
 
 QT_BEGIN_NAMESPACE
@@ -86,12 +82,10 @@ static qlonglong qMetaTypeNumber(const QVariant::Private *d)
         return qRound64(d->get<float>());
     case QMetaType::Double:
         return qRound64(d->get<double>());
-#ifndef QT_BOOTSTRAPPED
     case QMetaType::QJsonValue:
         return d->get<QJsonValue>().toDouble();
     case QMetaType::QCborValue:
         return d->get<QCborValue>().toInteger();
-#endif
     }
     Q_UNREACHABLE_RETURN(0);
 }
@@ -135,7 +129,6 @@ static std::optional<qlonglong> qConvertToNumber(const QVariant::Private *d, boo
         return std::nullopt;
     case QMetaType::Bool:
         return qlonglong(d->get<bool>());
-#ifndef QT_BOOTSTRAPPED
     case QMetaType::QCborValue:
         if (!d->get<QCborValue>().isInteger() && !d->get<QCborValue>().isDouble())
             break;
@@ -144,7 +137,6 @@ static std::optional<qlonglong> qConvertToNumber(const QVariant::Private *d, boo
         if (!d->get<QJsonValue>().isDouble())
             break;
         Q_FALLTHROUGH();
-#endif
     case QMetaType::Double:
     case QMetaType::Int:
     case QMetaType::Char:
@@ -171,20 +163,20 @@ static std::optional<qlonglong> qConvertToNumber(const QVariant::Private *d, boo
     return std::nullopt;
 }
 
-static std::optional<qreal> qConvertToRealNumber(const QVariant::Private *d)
+static std::optional<double> qConvertToRealNumber(const QVariant::Private *d)
 {
     bool ok;
     switch (d->typeInterface()->typeId) {
     case QMetaType::QString:
         if (double r = d->get<QString>().toDouble(&ok); ok)
-            return qreal(r);
+            return r;
         return std::nullopt;
     case QMetaType::Double:
-        return qreal(d->get<double>());
+        return d->get<double>();
     case QMetaType::Float:
-        return qreal(d->get<float>());
+        return double(d->get<float>());
     case QMetaType::Float16:
-        return qreal(d->get<qfloat16>());
+        return double(d->get<qfloat16>());
     case QMetaType::ULongLong:
     case QMetaType::UInt:
     case QMetaType::UChar:
@@ -192,17 +184,15 @@ static std::optional<qreal> qConvertToRealNumber(const QVariant::Private *d)
     case QMetaType::Char32:
     case QMetaType::UShort:
     case QMetaType::ULong:
-        return qreal(qMetaTypeUNumber(d));
-#ifndef QT_BOOTSTRAPPED
+        return double(qMetaTypeUNumber(d));
     case QMetaType::QCborValue:
         return d->get<QCborValue>().toDouble();
     case QMetaType::QJsonValue:
         return d->get<QJsonValue>().toDouble();
-#endif
     default:
         // includes enum conversion as well as invalid types
         if (std::optional<qlonglong> l = qConvertToNumber(d))
-            return qreal(*l);
+            return double(*l);
         return std::nullopt;
     }
 }
@@ -231,24 +221,22 @@ static bool isValidMetaTypeForVariant(const QtPrivate::QMetaTypeInterface *iface
     return true;
 }
 
-template <typename F> static QVariant::PrivateShared *
-customConstructShared(size_t size, size_t align, F &&construct)
-{
-    struct Deleter {
-        void operator()(QVariant::PrivateShared *p) const
-        { QVariant::PrivateShared::free(p); }
-    };
+enum CustomConstructMoveOptions {
+    UseCopy,  // custom construct uses the copy ctor unconditionally
+    // future option: TryMove: uses move ctor if available, else copy ctor
+    ForceMove, // custom construct use the move ctor (which must exist)
+};
 
-    // this is exception-safe
-    std::unique_ptr<QVariant::PrivateShared, Deleter> ptr;
-    ptr.reset(QVariant::PrivateShared::create(size, align));
-    construct(ptr->data());
-    return ptr.release();
-}
+enum CustomConstructNullabilityOption {
+    MaybeNull, // copy might be null, might be non-null
+    NonNull, // copy is guarantueed to be non-null
+    // future option: AlwaysNull?
+};
 
 // the type of d has already been set, but other field are not set
+template <CustomConstructMoveOptions moveOption = UseCopy, CustomConstructNullabilityOption nullability = MaybeNull>
 static void customConstruct(const QtPrivate::QMetaTypeInterface *iface, QVariant::Private *d,
-                            const void *copy)
+                            std::conditional_t<moveOption == ForceMove, void *, const void *> copy)
 {
     using namespace QtMetaTypePrivate;
     Q_ASSERT(iface);
@@ -257,6 +245,10 @@ static void customConstruct(const QtPrivate::QMetaTypeInterface *iface, QVariant
     Q_ASSERT(isCopyConstructible(iface));
     Q_ASSERT(isDestructible(iface));
     Q_ASSERT(copy || isDefaultConstructible(iface));
+    if constexpr (moveOption == ForceMove)
+        Q_ASSERT(isMoveConstructible(iface));
+    if constexpr (nullability == NonNull)
+        Q_ASSERT(copy != nullptr);
 
     // need to check for nullptr_t here, as this can get called by fromValue(nullptr). fromValue() uses
     // std::addressof(value) which in this case returns the address of the nullptr object.
@@ -266,11 +258,17 @@ static void customConstruct(const QtPrivate::QMetaTypeInterface *iface, QVariant
     if (QVariant::Private::canUseInternalSpace(iface)) {
         d->is_shared = false;
         if (!copy && !iface->defaultCtr)
-            return;     // default constructor and it's OK to build in 0-filled storage, which we've already done
-        construct(iface, d->data.data, copy);
+            return;     // trivial default constructor and it's OK to build in 0-filled storage, which we've already done
+        if constexpr (moveOption == ForceMove && nullability == NonNull)
+            moveConstruct(iface, d->data.data, copy);
+        else
+            construct(iface, d->data.data, copy);
     } else {
         d->data.shared = customConstructShared(iface->size, iface->alignment, [=](void *where) {
-            construct(iface, where, copy);
+            if constexpr (moveOption == ForceMove && nullability == NonNull)
+                moveConstruct(iface, where, copy);
+            else
+                construct(iface, where, copy);
         });
         d->is_shared = true;
     }
@@ -295,70 +293,24 @@ static QVariant::Private clonePrivate(const QVariant::Private &other)
     if (d.is_shared) {
         d.data.shared->ref.ref();
     } else if (const QtPrivate::QMetaTypeInterface *iface = d.typeInterface()) {
-        Q_ASSERT(d.canUseInternalSpace(iface));
+        if (Q_LIKELY(d.canUseInternalSpace(iface))) {
+            // if not trivially copyable, ask to copy (if it's trivially
+            // copyable, we've already copied it)
+            if (iface->copyCtr)
+                QtMetaTypePrivate::copyConstruct(iface, d.data.data, other.data.data);
+        } else {
+            // highly unlikely, but possible case: type has changed relocatability
+            // between builds
+            d.data.shared = QVariant::PrivateShared::create(iface->size, iface->alignment);
+            QtMetaTypePrivate::copyConstruct(iface, d.data.shared->data(), other.data.data);
+        }
 
-        // if not trivially copyable, ask to copy
-        if (iface->copyCtr)
-            QtMetaTypePrivate::copyConstruct(iface, d.data.data, other.data.data);
     }
     return d;
 }
 
 } // anonymous used to hide QVariant handlers
 
-inline QVariant::PrivateShared *QVariant::PrivateShared::create(size_t size, size_t align)
-{
-    size += sizeof(PrivateShared);
-    if (align > sizeof(PrivateShared)) {
-        // The alignment is larger than the alignment we can guarantee for the pointer
-        // directly following PrivateShared, so we need to allocate some additional
-        // memory to be able to fit the object into the available memory with suitable
-        // alignment.
-        size += align - sizeof(PrivateShared);
-    }
-    void *data = operator new(size);
-    auto *ps = new (data) QVariant::PrivateShared();
-    ps->offset = int(((quintptr(ps) + sizeof(PrivateShared) + align - 1) & ~(align - 1)) - quintptr(ps));
-    return ps;
-}
-
-inline void QVariant::PrivateShared::free(PrivateShared *p)
-{
-    p->~PrivateShared();
-    operator delete(p);
-}
-
-inline QVariant::Private::Private(const QtPrivate::QMetaTypeInterface *iface) noexcept
-    : is_shared(false), is_null(false), packedType(quintptr(iface) >> 2)
-{
-    Q_ASSERT((quintptr(iface) & 0x3) == 0);
-}
-
-template <typename T> inline
-QVariant::Private::Private(std::piecewise_construct_t, const T &t)
-    : is_shared(!CanUseInternalSpace<T>), is_null(std::is_same_v<T, std::nullptr_t>)
-{
-    // confirm noexceptness
-    static constexpr bool isNothrowQVariantConstructible = noexcept(QVariant(t));
-    static constexpr bool isNothrowCopyConstructible = std::is_nothrow_copy_constructible_v<T>;
-    static constexpr bool isNothrowCopyAssignable = std::is_nothrow_copy_assignable_v<T>;
-
-    const QtPrivate::QMetaTypeInterface *iface = QtPrivate::qMetaTypeInterfaceForType<T>();
-    Q_ASSERT((quintptr(iface) & 0x3) == 0);
-    packedType = quintptr(iface) >> 2;
-
-    if constexpr (CanUseInternalSpace<T>) {
-        static_assert(isNothrowQVariantConstructible == isNothrowCopyConstructible);
-        static_assert(isNothrowQVariantConstructible == isNothrowCopyAssignable);
-        new (data.data) T(t);
-    } else {
-        static_assert(!isNothrowQVariantConstructible); // we allocate memory, even if T doesn't
-        data.shared = customConstructShared(sizeof(T), alignof(T), [=](void *where) {
-            new (where) T(t);
-        });
-    }
-}
-
 /*!
     \class QVariant
     \inmodule QtCore
@@ -367,11 +319,7 @@ QVariant::Private::Private(std::piecewise_construct_t, const T &t)
     \ingroup objectmodel
     \ingroup shared
 
-
-    Because C++ forbids unions from including types that have
-    non-default constructors or destructors, most interesting Qt
-    classes cannot be used in unions. Without QVariant, this would be
-    a problem for QObject::property() and for database work, etc.
+    \compares equality
 
     A QVariant object holds a single value of a single typeId() at a
     time. (Some types are multi-valued, for example a string list.)
@@ -559,7 +507,7 @@ void QVariant::create(int type, const void *copy)
 */
 void QVariant::create(QMetaType type, const void *copy)
 {
-    *this = QVariant(type, copy);
+    *this = QVariant::fromMetaType(type, copy);
 }
 
 /*!
@@ -586,6 +534,105 @@ QVariant::QVariant(const QVariant &p)
 {
 }
 
+/*!
+    \fn template <typename T, typename... Args, QVariant::if_constructible<T, Args...> = true> QVariant::QVariant(std::in_place_type_t<T>, Args&&... args) noexcept(is_noexcept_constructible<q20::remove_cvref_t<T>, Args...>::value)
+
+    \since 6.6
+    Constructs a new variant containing a value of type \c T. The contained
+    value is initialized with the arguments
+    \c{std::forward<Args>(args)...}.
+
+    This constructor is provided for STL/std::any compatibility.
+
+    \overload
+
+    \constraints \c T can be constructed from \a args.
+ */
+
+/*!
+
+    \fn template <typename T, typename U, typename... Args, QVariant::if_constructible<T, std::initializer_list<U> &, Args...> = true> explicit QVariant::QVariant(std::in_place_type_t<T>, std::initializer_list<U> il, Args&&... args) noexcept(is_noexcept_constructible<q20::remove_cvref_t<T>, std::initializer_list<U> &, Args... >::value)
+
+    \since 6.6
+    \overload
+    This overload exists to support types with constructors taking an
+    \c initializer_list. It behaves otherwise equivalent to the
+    non-initializer list \c{in_place_type_t} overload.
+*/
+
+
+/*!
+    \fn template <typename T, typename... Args, QVariant::if_constructible<T, Args...> = true> QVariant::emplace(Args&&... args)
+
+    \since 6.6
+    Replaces the object currently held in \c{*this} with an object of
+    type \c{T}, constructed from \a{args}\c{...}. If \c{*this} was non-null,
+    the previously held object is destroyed first.
+    If possible, this method will reuse memory allocated by the QVariant.
+    Returns a reference to the newly-created object.
+ */
+
+/*!
+    \fn template <typename T, typename U, typename... Args, QVariant::if_constructible<T, std::initializer_list<U> &, Args...> = true> QVariant::emplace(std::initializer_list<U> list, Args&&... args)
+
+    \since 6.6
+    \overload
+    This overload exists to support types with constructors taking an
+    \c initializer_list. It behaves otherwise equivalent to the
+    non-initializer list overload.
+*/
+
+QVariant::QVariant(std::in_place_t, QMetaType type) : d(type.iface())
+{
+    // we query the metatype instead of detecting it at compile time
+    // so that we can change relocatability of internal types
+    if (!Private::canUseInternalSpace(type.iface())) {
+        d.data.shared = PrivateShared::create(type.sizeOf(), type.alignOf());
+        d.is_shared = true;
+    }
+}
+
+/*!
+    \internal
+    Returns a pointer to data suitable for placement new
+    of an object of type \a type
+    Changes the variant's metatype to \a type
+ */
+void *QVariant::prepareForEmplace(QMetaType type)
+{
+    /* There are two cases where we can reuse the existing storage
+       (1) The new type fits in QVariant's SBO storage
+       (2) We are using the externally allocated storage, the variant is
+           detached, and the new type fits into the existing storage.
+       In all other cases (3), we cannot reuse the storage.
+     */
+    auto typeFits = [&] {
+        auto newIface = type.iface();
+        auto oldIface = d.typeInterface();
+        auto newSize = PrivateShared::computeAllocationSize(newIface->size, newIface->alignment);
+        auto oldSize = PrivateShared::computeAllocationSize(oldIface->size, oldIface->alignment);
+        return newSize <= oldSize;
+    };
+    if (Private::canUseInternalSpace(type.iface())) { // (1)
+        clear();
+        d.packedType = quintptr(type.iface()) >> 2;
+        return d.data.data;
+    } else if (d.is_shared && isDetached() && typeFits()) { // (2)
+        QtMetaTypePrivate::destruct(d.typeInterface(), d.data.shared->data());
+        // compare QVariant::PrivateShared::create
+        const auto ps = d.data.shared;
+        const auto align = type.alignOf();
+        ps->offset =  PrivateShared::computeOffset(ps, align);
+        d.packedType = quintptr(type.iface()) >> 2;
+        return ps->data();
+    }
+    // (3)
+    QVariant newVariant(std::in_place, type);
+    swap(newVariant);
+    // const cast is safe, we're in a non-const method
+    return const_cast<void *>(d.storage());
+}
+
 /*!
   \fn QVariant::QVariant(const QString &val) noexcept
 
@@ -853,27 +900,26 @@ QVariant::QVariant(const QVariant &p)
 */
 
 /*!
-    Constructs variant of type \a type, and initializes with
-    \a copy if \a copy is not \nullptr.
+    Constructs a variant of type \a type, and initializes it with
+    a copy of \c{*copy} if \a copy is not \nullptr (in which case, \a copy
+    must point to an object of type \a type).
 
-    Note that you have to pass the address of the variable you want stored.
+    Note that you have to pass the address of the object you want stored.
 
     Usually, you never have to use this constructor, use QVariant::fromValue()
     instead to construct variants from the pointer types represented by
     \c QMetaType::VoidStar, and \c QMetaType::QObjectStar.
 
-    If \a type does not support copy and default construction, the variant will
-    be invalid.
+    If \a type does not support copy construction and \a copy is not \nullptr,
+    the variant will be invalid. Similarly, if \a copy is \nullptr and
+    \a type does not support default construction, the variant will be
+    invalid.
 
-    \sa QVariant::fromValue(), QMetaType::Type
+    \sa QVariant::fromMetaType, QVariant::fromValue(), QMetaType::Type
 */
-QVariant::QVariant(QMetaType type, const void *copy) : d(type.iface())
+QVariant::QVariant(QMetaType type, const void *copy)
+    : QVariant(fromMetaType(type, copy))
 {
-    type.registerType();
-    if (isValidMetaTypeForVariant(type.iface(), copy))
-        customConstruct(type.iface(), &d, copy);
-    else
-        d = {};
 }
 
 QVariant::QVariant(int val) noexcept : d(std::piecewise_construct_t{}, val) {}
@@ -903,7 +949,6 @@ QVariant::QVariant(QLatin1StringView val) : QVariant(QString(val)) {}
 #if QT_CONFIG(easingcurve)
 QVariant::QVariant(const QEasingCurve &val) : d(std::piecewise_construct_t{}, val) {}
 #endif
-#ifndef QT_NO_GEOM_VARIANT
 QVariant::QVariant(QPoint pt) noexcept
     : d(std::piecewise_construct_t{}, pt) {}
 QVariant::QVariant(QPointF pt) noexcept(Private::FitsInInternalSize<sizeof(qreal) * 2>)
@@ -920,23 +965,18 @@ QVariant::QVariant(QSize s) noexcept
     : d(std::piecewise_construct_t{}, s) {}
 QVariant::QVariant(QSizeF s) noexcept(Private::FitsInInternalSize<sizeof(qreal) * 2>)
     : d(std::piecewise_construct_t{}, s) {}
-#endif
-#ifndef QT_BOOTSTRAPPED
 QVariant::QVariant(const QUrl &u) noexcept : d(std::piecewise_construct_t{}, u) {}
-#endif
 QVariant::QVariant(const QLocale &l) noexcept : d(std::piecewise_construct_t{}, l) {}
 #if QT_CONFIG(regularexpression)
 QVariant::QVariant(const QRegularExpression &re) noexcept : d(std::piecewise_construct_t{}, re) {}
 #endif // QT_CONFIG(regularexpression)
 QVariant::QVariant(QUuid uuid) noexcept(Private::FitsInInternalSize<16>) : d(std::piecewise_construct_t{}, uuid) {}
-#ifndef QT_BOOTSTRAPPED
 QVariant::QVariant(const QJsonValue &jsonValue) noexcept(Private::FitsInInternalSize<sizeof(CborValueStandIn)>)
     : d(std::piecewise_construct_t{}, jsonValue)
 { static_assert(sizeof(CborValueStandIn) == sizeof(QJsonValue)); }
 QVariant::QVariant(const QJsonObject &jsonObject) noexcept : d(std::piecewise_construct_t{}, jsonObject) {}
 QVariant::QVariant(const QJsonArray &jsonArray) noexcept : d(std::piecewise_construct_t{}, jsonArray) {}
 QVariant::QVariant(const QJsonDocument &jsonDocument) : d(std::piecewise_construct_t{}, jsonDocument) {}
-#endif // QT_BOOTSTRAPPED
 #if QT_CONFIG(itemmodel)
 QVariant::QVariant(const QModelIndex &modelIndex) noexcept(Private::FitsInInternalSize<8 + 2 * sizeof(quintptr)>)
     : d(std::piecewise_construct_t{}, modelIndex) {}
@@ -989,14 +1029,11 @@ QVariant::QVariant(const QPersistentModelIndex &modelIndex) : d(std::piecewise_c
 */
 
 /*!
+    \fn QMetaType QVariant::metaType() const
     \since 6.0
 
     Returns the QMetaType of the value stored in the variant.
 */
-QMetaType QVariant::metaType() const
-{
-    return d.type();
-}
 
 /*!
     Assigns the value of the variant \a variant to this variant.
@@ -1014,9 +1051,7 @@ QVariant &QVariant::operator=(const QVariant &variant)
 /*!
     \fn void QVariant::swap(QVariant &other)
     \since 4.8
-
-    Swaps variant \a other with this variant. This operation is very
-    fast and never fails.
+    \memberswap{variant}
 */
 
 /*!
@@ -1032,7 +1067,8 @@ void QVariant::detach()
 
     Q_ASSERT(isValidMetaTypeForVariant(d.typeInterface(), constData()));
     Private dd(d.typeInterface());
-    customConstruct(d.typeInterface(), &dd, constData());
+    // null variant is never shared; anything else is NonNull
+    customConstruct<UseCopy, NonNull>(d.typeInterface(), &dd, constData());
     if (!d.data.shared->ref.deref())
         customClear(&d);
     d.data.shared = dd.data.shared;
@@ -1045,15 +1081,13 @@ void QVariant::detach()
 */
 
 /*!
+    \fn const char *QVariant::typeName() const
+
     Returns the name of the type stored in the variant. The returned
     strings describe the C++ datatype used to store the data: for
     example, "QFont", "QString", or "QVariantList". An Invalid
     variant returns 0.
 */
-const char *QVariant::typeName() const
-{
-    return d.type().name();
-}
 
 /*!
     Convert this variant to type QMetaType::UnknownType and free up any resources
@@ -1521,7 +1555,6 @@ QByteArray QVariant::toByteArray() const
     return qvariant_cast<QByteArray>(*this);
 }
 
-#ifndef QT_NO_GEOM_VARIANT
 /*!
     \fn QPoint QVariant::toPoint() const
 
@@ -1629,9 +1662,6 @@ QPointF QVariant::toPointF() const
     return qvariant_cast<QPointF>(*this);
 }
 
-#endif // QT_NO_GEOM_VARIANT
-
-#ifndef QT_BOOTSTRAPPED
 /*!
     \fn QUrl QVariant::toUrl() const
 
@@ -1644,7 +1674,6 @@ QUrl QVariant::toUrl() const
 {
     return qvariant_cast<QUrl>(*this);
 }
-#endif
 
 /*!
     \fn QLocale QVariant::toLocale() const
@@ -1717,7 +1746,6 @@ QUuid QVariant::toUuid() const
     return qvariant_cast<QUuid>(*this);
 }
 
-#ifndef QT_BOOTSTRAPPED
 /*!
     \since 5.0
 
@@ -1769,7 +1797,6 @@ QJsonDocument QVariant::toJsonDocument() const
 {
     return qvariant_cast<QJsonDocument>(*this);
 }
-#endif // QT_BOOTSTRAPPED
 
 /*!
     \fn QChar QVariant::toChar() const
@@ -2001,6 +2028,10 @@ QVariantList QVariant::toList() const
     type, \a type. Such casting is done automatically when calling the
     toInt(), toBool(), ... methods.
 
+    Note this function operates only on the variant's type, not the contents.
+    It indicates whether there is a conversion path from this variant to \a
+    type, not that the conversion will succeed when attempted.
+
     \sa QMetaType::canConvert()
 */
 
@@ -2090,9 +2121,9 @@ bool QVariant::view(int type, void *ptr)
 }
 
 /*!
-    \fn bool QVariant::operator==(const QVariant &v1, const QVariant &v2)
+    \fn bool QVariant::operator==(const QVariant &lhs, const QVariant &rhs)
 
-    Returns \c true if \a v1 and \a v2 are equal; otherwise returns \c false.
+    Returns \c true if \a lhs and \a rhs are equal; otherwise returns \c false.
 
     QVariant uses the equality operator of the type() contained to check for
     equality.
@@ -2116,9 +2147,9 @@ bool QVariant::view(int type, void *ptr)
 */
 
 /*!
-    \fn bool QVariant::operator!=(const QVariant &v1, const QVariant &v2)
+    \fn bool QVariant::operator!=(const QVariant &lhs, const QVariant &rhs)
 
-    Returns \c false if \a v1 and \a v2 are equal; otherwise returns \c true.
+    Returns \c false if \a lhs and \a rhs are equal; otherwise returns \c true.
 
     QVariant uses the equality operator of the type() contained to check for
     equality.
@@ -2143,6 +2174,7 @@ static bool qIsNumericType(uint tp)
             Q_UINT64_C(1) << QMetaType::QString |
             Q_UINT64_C(1) << QMetaType::Bool |
             Q_UINT64_C(1) << QMetaType::Double |
+            Q_UINT64_C(1) << QMetaType::Float16 |
             Q_UINT64_C(1) << QMetaType::Float |
             Q_UINT64_C(1) << QMetaType::Char |
             Q_UINT64_C(1) << QMetaType::Char16 |
@@ -2258,23 +2290,6 @@ static int numericTypePromotion(const QtPrivate::QMetaTypeInterface *iface1,
     return QMetaType::Int;
 }
 
-template <typename Numeric> static QPartialOrdering spaceShip(Numeric lhs, Numeric rhs)
-{
-    if (lhs == rhs)
-        return QPartialOrdering::Equivalent;
-    if constexpr (std::numeric_limits<Numeric>::has_quiet_NaN) {
-        if (std::isnan(lhs) || std::isnan(rhs))
-            return QPartialOrdering::Unordered;
-    }
-
-    bool smaller;
-    if constexpr (std::is_same_v<Numeric, QObject *>)
-        smaller = std::less<QObject *>()(lhs, rhs); // can't use less all the time because of bool
-    else
-        smaller = lhs < rhs;
-    return smaller ? QPartialOrdering::Less : QPartialOrdering::Greater;
-}
-
 static QPartialOrdering integralCompare(uint promotedType, const QVariant::Private *d1, const QVariant::Private *d2)
 {
     // use toLongLong to retrieve the data, it gets us all the bits
@@ -2283,13 +2298,13 @@ static QPartialOrdering integralCompare(uint promotedType, const QVariant::Priva
     if (!l1 || !l2)
         return QPartialOrdering::Unordered;
     if (promotedType == QMetaType::UInt)
-        return spaceShip<uint>(*l1, *l2);
+        return Qt::compareThreeWay(uint(*l1), uint(*l2));
     if (promotedType == QMetaType::LongLong)
-        return spaceShip<qlonglong>(*l1, *l2);
+        return Qt::compareThreeWay(qlonglong(*l1), qlonglong(*l2));
     if (promotedType == QMetaType::ULongLong)
-        return spaceShip<qulonglong>(*l1, *l2);
+        return Qt::compareThreeWay(qulonglong(*l1), qulonglong(*l2));
 
-    return spaceShip<int>(*l1, *l2);
+    return Qt::compareThreeWay(int(*l1), int(*l2));
 }
 
 static QPartialOrdering numericCompare(const QVariant::Private *d1, const QVariant::Private *d2)
@@ -2298,18 +2313,15 @@ static QPartialOrdering numericCompare(const QVariant::Private *d1, const QVaria
     if (promotedType != QMetaType::QReal)
         return integralCompare(promotedType, d1, d2);
 
-    // qreal comparisons
-    std::optional<qreal> r1 = qConvertToRealNumber(d1);
-    std::optional<qreal> r2 = qConvertToRealNumber(d2);
+    // floating point comparison
+    const auto r1 = qConvertToRealNumber(d1);
+    const auto r2 = qConvertToRealNumber(d2);
     if (!r1 || !r2)
         return QPartialOrdering::Unordered;
-    if (*r1 == *r2)
-        return QPartialOrdering::Equivalent;
 
-    return spaceShip<qreal>(*r1, *r2);
+    return Qt::compareThreeWay(*r1, *r2);
 }
 
-#ifndef QT_BOOTSTRAPPED
 static bool qvCanConvertMetaObject(QMetaType fromType, QMetaType toType)
 {
     if ((fromType.flags() & QMetaType::PointerToQObject)
@@ -2323,9 +2335,9 @@ static bool qvCanConvertMetaObject(QMetaType fromType, QMetaType toType)
 
 static QPartialOrdering pointerCompare(const QVariant::Private *d1, const QVariant::Private *d2)
 {
-    return spaceShip<QObject *>(d1->get<QObject *>(), d2->get<QObject *>());
+    return Qt::compareThreeWay(Qt::totally_ordered_wrapper(d1->get<QObject *>()),
+                               Qt::totally_ordered_wrapper(d2->get<QObject *>()));
 }
-#endif
 
 /*!
     \internal
@@ -2338,11 +2350,9 @@ bool QVariant::equals(const QVariant &v) const
         // try numeric comparisons, with C++ type promotion rules (no conversion)
         if (canBeNumericallyCompared(metatype.iface(), v.d.type().iface()))
             return numericCompare(&d, &v.d) == QPartialOrdering::Equivalent;
-#ifndef QT_BOOTSTRAPPED
         // if both types are related pointers to QObjects, check if they point to the same object
         if (qvCanConvertMetaObject(metatype, v.metaType()))
             return pointerCompare(&d, &v.d) == QPartialOrdering::Equivalent;
-#endif
         return false;
     }
 
@@ -2382,10 +2392,8 @@ QPartialOrdering QVariant::compare(const QVariant &lhs, const QVariant &rhs)
         // try numeric comparisons, with C++ type promotion rules (no conversion)
         if (canBeNumericallyCompared(lhs.d.type().iface(), rhs.d.type().iface()))
             return numericCompare(&lhs.d, &rhs.d);
-#ifndef QT_BOOTSTRAPPED
         if (qvCanConvertMetaObject(lhs.metaType(), rhs.metaType()))
             return pointerCompare(&lhs.d, &rhs.d);
-#endif
         return QPartialOrdering::Unordered;
     }
     return t.compare(lhs.constData(), rhs.constData());
@@ -2398,7 +2406,7 @@ QPartialOrdering QVariant::compare(const QVariant &lhs, const QVariant &rhs)
     Returns a pointer to the contained object as a generic void* that cannot be
     written to.
 
-    \sa QMetaType
+    \sa get_if(), QMetaType
  */
 
 /*!
@@ -2408,7 +2416,7 @@ QPartialOrdering QVariant::compare(const QVariant &lhs, const QVariant &rhs)
     This function detaches the QVariant. When called on a \l{isNull}{null-QVariant},
     the QVariant will not be null after the call.
 
-    \sa QMetaType
+    \sa get_if(), QMetaType
 */
 void *QVariant::data()
 {
@@ -2418,6 +2426,42 @@ void *QVariant::data()
     return const_cast<void *>(constData());
 }
 
+/*!
+    \since 6.6
+    \fn template <typename T> const T* QVariant::get_if(const QVariant *v)
+    \fn template <typename T> T* QVariant::get_if(QVariant *v)
+
+    If \a v contains an object of type \c T, returns a pointer to the contained
+    object, otherwise returns \nullptr.
+
+    The overload taking a mutable \a v detaches \a v: When called on a
+    \l{isNull()}{null} \a v with matching type \c T, \a v will not be null
+    after the call.
+
+    These functions are provided for compatibility with \c{std::variant}.
+
+    \sa data()
+*/
+
+/*!
+    \since 6.6
+    \fn template <typename T> T &QVariant::get(QVariant &v)
+    \fn template <typename T> const T &QVariant::get(const QVariant &v)
+    \fn template <typename T> T &&QVariant::get(QVariant &&v)
+    \fn template <typename T> const T &&QVariant::get(const QVariant &&v)
+
+    If \a v contains an object of type \c T, returns a reference to the contained
+    object, otherwise the call has undefined behavior.
+
+    The overloads taking a mutable \a v detach \a v: When called on a
+    \l{isNull()}{null} \a v with matching type \c T, \a v will not be null
+    after the call.
+
+    These functions are provided for compatibility with \c{std::variant}.
+
+    \sa get_if(), data()
+*/
+
 /*!
     Returns \c true if this is a null variant, false otherwise.
 
@@ -2456,6 +2500,22 @@ QDebug QVariant::qdebugHelper(QDebug dbg) const
     return dbg;
 }
 
+QVariant QVariant::moveConstruct(QMetaType type, void *data)
+{
+    QVariant var;
+    var.d = QVariant::Private(type.d_ptr);
+    customConstruct<ForceMove, NonNull>(type.d_ptr, &var.d, data);
+    return var;
+}
+
+QVariant QVariant::copyConstruct(QMetaType type, const void *data)
+{
+    QVariant var;
+    var.d = QVariant::Private(type.d_ptr);
+    customConstruct<UseCopy, NonNull>(type.d_ptr, &var.d, data);
+    return var;
+}
+
 #if QT_DEPRECATED_SINCE(6, 0)
 QT_WARNING_PUSH
 QT_WARNING_DISABLE_DEPRECATED
@@ -2475,7 +2535,7 @@ QT_WARNING_POP
 
 #endif
 
-/*! \fn template<typename T> void QVariant::setValue(T &&value)
+/*! \fn template<typename T, typename = std::enable_if_t<!std::is_same_v<std::decay_t<T>, QVariant>>> void QVariant::setValue(T &&value)
 
     Stores a copy of \a value. If \c{T} is a type that QVariant
     doesn't support, QMetaType is used to store the value. A compile
@@ -2488,19 +2548,19 @@ QT_WARNING_POP
     \sa value(), fromValue(), canConvert()
  */
 
-/*! \fn template<typename T> void QVariant::setValue(const QVariant &value)
+/*! \fn void QVariant::setValue(const QVariant &value)
 
     Copies \a value over this QVariant. It is equivalent to simply
     assigning \a value to this QVariant.
 */
 
-/*! \fn template<typename T> void QVariant::setValue(QVariant &&value)
+/*! \fn void QVariant::setValue(QVariant &&value)
 
     Moves \a value over this QVariant. It is equivalent to simply
     move assigning \a value to this QVariant.
 */
 
-/*! \fn template<typename T> T QVariant::value() const
+/*! \fn template<typename T> T QVariant::value() const &
 
     Returns the stored value converted to the template type \c{T}.
     Call canConvert() to find out whether a type can be converted.
@@ -2540,7 +2600,7 @@ QT_WARNING_POP
     \sa canView(), Q_DECLARE_SEQUENTIAL_CONTAINER_METATYPE()
 */
 
-/*! \fn bool QVariant::canConvert() const
+/*! \fn template<typename T> bool QVariant::canConvert() const
 
     Returns \c true if the variant can be converted to the template type \c{T},
     otherwise false.
@@ -2556,7 +2616,7 @@ QT_WARNING_POP
     \sa convert()
 */
 
-/*! \fn bool QVariant::canView() const
+/*! \fn template<typename T> bool QVariant::canView() const
 
     Returns \c true if a mutable view of the template type \c{T} can be created on this variant,
     otherwise \c false.
@@ -2576,6 +2636,12 @@ QT_WARNING_POP
     \sa setValue(), value()
 */
 
+/*! \fn template<typename T, QVariant::if_rvalue<T> = true> static QVariant QVariant::fromValue(T &&value)
+
+    \since 6.6
+    \overload
+*/
+
 /*! \fn template<typename... Types> QVariant QVariant::fromStdVariant(const std::variant<Types...> &value)
     \since 5.11
 
@@ -2589,6 +2655,47 @@ QT_WARNING_POP
     \sa fromValue()
 */
 
+/*!
+    \fn template<typename... Types> QVariant QVariant::fromStdVariant(std::variant<Types...> &&value)
+    \since 6.6
+    \overload
+*/
+
+
+/*!
+    \since 6.7
+
+    Creates a variant of type \a type, and initializes it with
+    a copy of \c{*copy} if \a copy is not \nullptr (in which case, \a copy
+    must point to an object of type \a type).
+
+    Note that you have to pass the address of the object you want stored.
+
+    Usually, you never have to use this constructor, use QVariant::fromValue()
+    instead to construct variants from the pointer types represented by
+    \c QMetaType::VoidStar, and \c QMetaType::QObjectStar.
+
+    If \a type does not support copy construction and \a copy is not \nullptr,
+    the variant will be invalid. Similarly, if \a copy is \nullptr and
+    \a type does not support default construction, the variant will be
+    invalid.
+
+    Returns the QVariant created as described above.
+
+    \sa QVariant::fromValue(), QMetaType::Type
+*/
+QVariant QVariant::fromMetaType(QMetaType type, const void *copy)
+{
+    QVariant result;
+    type.registerType();
+    const auto iface = type.iface();
+    if (isValidMetaTypeForVariant(iface, copy)) {
+        result.d = Private(iface);
+        customConstruct(iface, &result.d, copy);
+    }
+    return result;
+}
+
 /*!
     \fn template<typename T> T qvariant_cast(const QVariant &value)
     \relates QVariant
@@ -2600,6 +2707,14 @@ QT_WARNING_POP
     \sa QVariant::value()
 */
 
+/*!
+    \fn template<typename T> T QVariant::qvariant_cast(QVariant &&value)
+    \overload
+    \since 6.7
+
+    Returns the given \a value converted to the template type \c{T}.
+*/
+
 /*! \fn template<typename T> T qVariantValue(const QVariant &value)
     \relates QVariant
     \deprecated
