@@ -138,6 +138,7 @@ QT_BEGIN_NAMESPACE
     \list
     \li \c QLatin1Char
     \li \c QChar::SpecialCharacter
+    \li \c wchar_t (where it's a 32-bit type, i.e. Unix) (since 6.10)
     \li \c char32_t
     \endlist
 
@@ -152,7 +153,7 @@ QT_BEGIN_NAMESPACE
     and UTF-8 multibyte sequences count as two, three or four,
     depending on their length).
 
-    \sa QUtf8StringView, QStringView
+    \sa {Which string class to use?}, QUtf8StringView, QStringView
 */
 
 /*!
