@@ -4,8 +4,6 @@
 
 #include "qchar.h"
 
-#include "qdatastream.h"
-
 #include "qunicodetables_p.h"
 #include "qunicodetables.cpp"
 
@@ -833,8 +831,9 @@ bool QChar::isPrint(char32_t ucs4) noexcept
 */
 bool QT_FASTCALL QChar::isSpace_helper(char32_t ucs4) noexcept
 {
-    if (ucs4 > LastValidCodePoint)
+    if (ucs4 > MaxSeparatorCodepoint)
         return false;
+
     const int test = FLAG(Separator_Space) |
                      FLAG(Separator_Line) |
                      FLAG(Separator_Paragraph);
@@ -1750,36 +1749,6 @@ char32_t QChar::toCaseFolded(char32_t ucs4) noexcept
     \sa toLatin1(), unicode()
 */
 
-#ifndef QT_NO_DATASTREAM
-/*!
-    \relates QChar
-
-    Writes the char \a chr to the stream \a out.
-
-    \sa {Serializing Qt Data Types}
-*/
-QDataStream &operator<<(QDataStream &out, QChar chr)
-{
-    out << quint16(chr.unicode());
-    return out;
-}
-
-/*!
-    \relates QChar
-
-    Reads a char from the stream \a in into char \a chr.
-
-    \sa {Serializing Qt Data Types}
-*/
-QDataStream &operator>>(QDataStream &in, QChar &chr)
-{
-    quint16 u;
-    in >> u;
-    chr.unicode() = char16_t(u);
-    return in;
-}
-#endif // QT_NO_DATASTREAM
-
 /*!
     \fn QChar::unicode()
 
