@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qlocale_p.h"
 
@@ -50,12 +15,52 @@
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
+#include <QtCore/qloggingcategory.h>
+#include <QtCore/qcoreapplication.h>
+
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /******************************************************************************
 ** Wrappers for Mac locale system functions
 */
 
+Q_STATIC_LOGGING_CATEGORY(lcLocale, "qt.core.locale")
+
+static void printLocalizationInformation()
+{
+    if (!lcLocale().isDebugEnabled())
+        return;
+
+#if defined(Q_OS_MACOS)
+    // Trigger initialization of standard user defaults, so that Foundation picks
+    // up -AppleLanguages and -AppleLocale passed on the command line.
+    Q_UNUSED(NSUserDefaults.standardUserDefaults);
+#endif
+
+    auto singleLineDescription = [](NSArray *array) {
+        NSString *str = [array description];
+        str = [str stringByReplacingOccurrencesOfString:@"\n" withString:@""];
+        return [str stringByReplacingOccurrencesOfString:@"    " withString:@""];
+    };
+
+    bool allowMixedLocalizations = [NSBundle.mainBundle.infoDictionary[@"CFBundleAllowMixedLocalizations"] boolValue];
+
+    NSBundle *foundation = [NSBundle bundleForClass:NSBundle.class];
+    qCDebug(lcLocale).nospace() << "Launched with locale \"" << NSLocale.currentLocale.localeIdentifier
+        << "\" based on user's preferred languages " << singleLineDescription(NSLocale.preferredLanguages)
+        << ", main bundle localizations " << singleLineDescription(NSBundle.mainBundle.localizations)
+        << ", and allowing mixed localizations " << allowMixedLocalizations
+        << "; resulting in main bundle preferred localizations "
+        << singleLineDescription(NSBundle.mainBundle.preferredLocalizations)
+        << " and Foundation preferred localizations "
+        << singleLineDescription(foundation.preferredLocalizations);
+    qCDebug(lcLocale) << "Reflected by Qt as system locale"
+        << QLocale::system() << "with UI languges " << QLocale::system().uiLanguages();
+}
+Q_COREAPP_STARTUP_FUNCTION(printLocalizationInformation);
+
 static QString getMacLocaleName()
 {
     QCFType<CFLocaleRef> l = CFLocaleCopyCurrent();
@@ -152,14 +157,26 @@ static QVariant macDayName(int day, QSystemLocale::QueryType type)
 
 static QString macZeroDigit()
 {
-    QCFType<CFLocaleRef> locale = CFLocaleCopyCurrent();
-    QCFType<CFNumberFormatterRef> numberFormatter =
-            CFNumberFormatterCreate(nullptr, locale, kCFNumberFormatterNoStyle);
-    const int zeroDigit = 0;
-    QCFType<CFStringRef> value
-        = CFNumberFormatterCreateStringWithValue(nullptr, numberFormatter,
-                                                 kCFNumberIntType, &zeroDigit);
-    return QString::fromCFString(value);
+    static QString cachedZeroDigit;
+
+    if (cachedZeroDigit.isNull()) {
+        QCFType<CFLocaleRef> locale = CFLocaleCopyCurrent();
+        QCFType<CFNumberFormatterRef> numberFormatter =
+                CFNumberFormatterCreate(nullptr, locale, kCFNumberFormatterNoStyle);
+        const int zeroDigit = 0;
+        QCFType<CFStringRef> value
+            = CFNumberFormatterCreateStringWithValue(nullptr, numberFormatter,
+                                                     kCFNumberIntType, &zeroDigit);
+        cachedZeroDigit = QString::fromCFString(value);
+    }
+
+    static QMacNotificationObserver localeChangeObserver = QMacNotificationObserver(
+        nil, NSCurrentLocaleDidChangeNotification, [&] {
+            qCDebug(lcLocale) << "System locale changed";
+            cachedZeroDigit = QString();
+    });
+
+    return cachedZeroDigit;
 }
 
 static QString zeroPad(QString &&number, qsizetype minDigits, const QString &zero)
@@ -222,12 +239,16 @@ static QString fourDigitYear(int year, const QString &zero)
 
 static QString macDateToStringImpl(QDate date, CFDateFormatterStyle style)
 {
-    QCFType<CFDateRef> myDate = date.startOfDay().toCFDate();
+    // Use noon on the given date, to avoid complications that can arise for
+    // dates before 1900 (see QTBUG-54955) using different UTC offset than
+    // QDateTime extrapolates backwards from time_t functions that only work
+    // back to 1900. (Alaska and Phillipines may still be borked, though.)
+    QCFType<CFDateRef> myDate = QDateTime(date, QTime(12, 0)).toCFDate();
     QCFType<CFLocaleRef> mylocale = CFLocaleCopyCurrent();
     QCFType<CFDateFormatterRef> myFormatter
         = CFDateFormatterCreate(kCFAllocatorDefault, mylocale, style,
                                 kCFDateFormatterNoStyle);
-    QCFType<CFStringRef> text = CFDateFormatterCreateStringWithDate(0, myFormatter, myDate);
+    QCFType<CFStringRef> text = CFDateFormatterCreateStringWithDate(nullptr, myFormatter, myDate);
     return QString::fromCFString(text);
 }
 
@@ -235,12 +256,14 @@ static QVariant macDateToString(QDate date, bool short_format)
 {
     const int year = date.year();
     QString fakeYear, trueYear;
-    if (year < 0) {
+    if (year < 1583) {
         // System API (in macOS 11.0, at least) discards sign :-(
         // Simply negating the year won't do as the resulting year typically has
         // a different pattern of week-days.
+        // Furthermore (see QTBUG-54955), Darwin uses the Julian calendar for
+        // dates before 1582-10-15, leading to discrepancies.
         int matcher = QGregorianCalendar::yearSharingWeekDays(date);
-        Q_ASSERT(matcher > 0);
+        Q_ASSERT(matcher >= 1583);
         Q_ASSERT(matcher % 100 != date.month());
         Q_ASSERT(matcher % 100 != date.day());
         // i.e. there can't be any confusion between the two-digit year and
@@ -253,7 +276,7 @@ static QVariant macDateToString(QDate date, bool short_format)
     QString text = macDateToStringImpl(date, short_format
                                        ? kCFDateFormatterShortStyle
                                        : kCFDateFormatterLongStyle);
-    if (year < 0) {
+    if (year < 1583) {
         if (text.contains(fakeYear))
             return std::move(text).replace(fakeYear, trueYear);
         // Cope with two-digit year:
@@ -288,114 +311,120 @@ static QVariant macTimeToString(QTime time, bool short_format)
 static QVariant macToQtFormat(QStringView sys_fmt)
 {
     QString result;
-    int i = 0;
+    qsizetype i = 0;
 
     while (i < sys_fmt.size()) {
         if (sys_fmt.at(i).unicode() == '\'') {
             QString text = qt_readEscapedFormatString(sys_fmt, &i);
-            if (text == QLatin1String("'"))
-                result += QLatin1String("''");
+            if (text == "'"_L1)
+                result += "''"_L1;
             else
-                result += QLatin1Char('\'') + text + QLatin1Char('\'');
+                result += u'\'' + text + u'\'';
             continue;
         }
 
         QChar c = sys_fmt.at(i);
-        qsizetype repeat = qt_repeatCount(sys_fmt.mid(i));
+        qsizetype repeat = qt_repeatCount(sys_fmt.sliced(i));
 
         switch (c.unicode()) {
             // Qt does not support the following options
-            case 'G': // Era (1..5): 4 = long, 1..3 = short, 5 = narrow
-            case 'Y': // Year of Week (1..n): 1..n = padded number
-            case 'U': // Cyclic Year Name (1..5): 4 = long, 1..3 = short, 5 = narrow
-            case 'Q': // Quarter (1..4): 4 = long, 3 = short, 1..2 = padded number
-            case 'q': // Standalone Quarter (1..4): 4 = long, 3 = short, 1..2 = padded number
-            case 'w': // Week of Year (1..2): 1..2 = padded number
-            case 'W': // Week of Month (1): 1 = number
-            case 'D': // Day of Year (1..3): 1..3 = padded number
-            case 'F': // Day of Week in Month (1): 1 = number
-            case 'g': // Modified Julian Day (1..n): 1..n = padded number
-            case 'A': // Milliseconds in Day (1..n): 1..n = padded number
-                break;
+        case 'A': // Milliseconds in Day (1..n): 1..n = padded number
+        case 'C': // Input skeleton symbol.
+        case 'D': // Day of Year (1..3): 1..3 = padded number
+        case 'F': // Day of Week in Month (1): 1 = number
+        case 'g': // Modified Julian Day (1..n): 1..n = padded number
+        case 'G': // Era (1..5): 4 = long, 1..3 = short, 5 = narrow
+        case 'j': // Input skeleton symbol.
+        case 'J': // Input skeleton symbol.
+        case 'l': // Deprecated Chinese leap month indicator.
+        case 'q': // Standalone Quarter (1..4): 4 = long, 3 = short, 1,2 = padded number
+        case 'Q': // Quarter (1..4): 4 = long, 3 = short, 1,2 = padded number
+        case 'U': // Cyclic Year Name (1..5): 4 = long, 1..3 = short, 5 = narrow
+        case 'w': // Week of Year (1,2): 1,2 = padded number
+        case 'W': // Week of Month (1): 1 = number
+        case 'Y': // Year for Week-of-year calendars (1..n): 1..n = padded number
+            break;
 
-            case 'y': // Year (1..n): 2 = short year, 1 & 3..n = padded number
-            case 'u': // Extended Year (1..n): 2 = short year, 1 & 3..n = padded number
-                // Qt only supports long (4) or short (2) year, use long for all others
-                if (repeat == 2)
-                    result += QLatin1String("yy");
-                else
-                    result += QLatin1String("yyyy");
-                break;
-            case 'M': // Month (1..5): 4 = long, 3 = short, 1..2 = number, 5 = narrow
-            case 'L': // Standalone Month (1..5): 4 = long, 3 = short, 1..2 = number, 5 = narrow
-                // Qt only supports long, short and number, use short for narrow
-                if (repeat == 5)
-                    result += QLatin1String("MMM");
-                else
-                    result += QString(repeat, QLatin1Char('M'));
-                break;
-            case 'd': // Day of Month (1..2): 1..2 padded number
+        case 'u': // Extended Year (1..n), padded number.
+            // Explicitly has no special case for 'uu' as only the last two digits.
+            result += "yyyy"_L1;
+            break;
+        case 'y': // Year (1..n): 2 = short year, 1 & 3..n = padded number
+            // Qt only supports long (4) or short (2) year, use long for all others
+            if (repeat == 2)
+                result += "yy"_L1;
+            else
+                result += "yyyy"_L1;
+            break;
+        case 'L': // Standalone Month (1..5): 4 = long, 3 = short, 1,2 = number, 5 = narrow
+        case 'M': // Month (1..5): 4 = long, 3 = short, 1,2 = number, 5 = narrow
+            // Qt only supports long, short and number, use short for narrow
+            if (repeat == 5)
+                result += "MMM"_L1;
+            else
+                result += QString(repeat, u'M');
+            break;
+        case 'd': // Day of Month (1,2): 1,2 padded number
+            result += QString(repeat, c);
+            break;
+        case 'c': // Standalone version of 'e'
+        case 'e': // Local Day of Week (1..6): 4 = long, 3 = short, 5,6 = narrow, 1,2 padded number
+            // "Local" only affects numeric form: depends on locale's start-day of the week.
+        case 'E': // Day of Week (1..6): 4 = long, 1..3 = short, 5,6 = narrow
+            // Qt only supports long, short: use short for narrow and padded number.
+            if (repeat == 4)
+                result += "dddd"_L1;
+            else
+                result += "ddd"_L1;
+            break;
+        case 'a': // AM/PM (1..n): Qt supports no distinctions
+        case 'b': // Like a, but also distinguishing noon, midnight (ignore difference).
+        case 'B': // Flexible day period (at night, &c.)
+            // Translate to Qt AM/PM, using locale-appropriate case:
+            result += "Ap"_L1;
+            break;
+        case 'h': // Hour [1..12] (1,2): 1,2 = padded number
+        case 'K': // Hour [0..11] (1,2): 1,2 = padded number
+            result += QString(repeat, 'h'_L1);
+            break;
+        case 'H': // Hour [0..23] (1,2): 1,2 = padded number
+        case 'k': // Hour [1..24] (1,2): 1,2 = padded number
+            // Qt H is 0..23 hour
+            result += QString(repeat, 'H'_L1);
+            break;
+        case 'm': // Minutes (1,2): 1,2 = padded number
+        case 's': // Seconds (1,2): 1,2 = padded number
+            result += QString(repeat, c);
+            break;
+        case 'S': // Fractional second (1..n): 1..n = truncates to decimal places
+            // Qt uses msecs either unpadded or padded to 3 places
+            if (repeat < 3)
+                result += u'z';
+            else
+                result += "zzz"_L1;
+            break;
+        case 'O': // Time Zone (1, 4)
+            result += u't';
+            break;
+        case 'v': // Time Zone (1, 4)
+        case 'V': // Time Zone (1..4)
+            result += "tttt"_L1;
+            break;
+        case 'x': // Time Zone (1..5)
+        case 'X': // Time Zone (1..5)
+            result += (repeat > 1 && (repeat & 1)) ? "ttt"_L1 : "tt"_L1;
+            break;
+        case 'z': // Time Zone (1..4)
+        case 'Z': // Time Zone (1..5)
+            result += repeat < 4 ? "tt"_L1 : repeat > 4 ? "ttt"_L1 : "t"_L1;
+            break;
+        default:
+            // a..z and A..Z are reserved for format codes, so any occurrence of these not
+            // already processed are not known and so unsupported formats to be ignored.
+            // All other chars are allowed as literals.
+            if (c < u'A' || c > u'z' || (c > u'Z' && c < u'a'))
                 result += QString(repeat, c);
-                break;
-            case 'E': // Day of Week (1..6): 4 = long, 1..3 = short, 5..6 = narrow
-                // Qt only supports long, short and padded number, use short for narrow
-                if (repeat == 4)
-                    result += QLatin1String("dddd");
-                else
-                    result += QLatin1String("ddd");
-                break;
-            case 'e': // Local Day of Week (1..6): 4 = long, 3 = short, 5..6 = narrow, 1..2 padded number
-            case 'c': // Standalone Local Day of Week (1..6): 4 = long, 3 = short, 5..6 = narrow, 1..2 padded number
-                // Qt only supports long, short and padded number, use short for narrow
-                if (repeat >= 5)
-                    result += QLatin1String("ddd");
-                else
-                    result += QString(repeat, QLatin1Char('d'));
-                break;
-            case 'a': // AM/PM (1): 1 = short
-                // Translate to Qt uppercase AM/PM
-                result += QLatin1String("AP");
-                break;
-            case 'h': // Hour [1..12] (1..2): 1..2 = padded number
-            case 'K': // Hour [0..11] (1..2): 1..2 = padded number
-            case 'j': // Local Hour [12 or 24] (1..2): 1..2 = padded number
-                // Qt h is local hour
-                result += QString(repeat, QLatin1Char('h'));
-                break;
-            case 'H': // Hour [0..23] (1..2): 1..2 = padded number
-            case 'k': // Hour [1..24] (1..2): 1..2 = padded number
-                // Qt H is 0..23 hour
-                result += QString(repeat, QLatin1Char('H'));
-                break;
-            case 'm': // Minutes (1..2): 1..2 = padded number
-            case 's': // Seconds (1..2): 1..2 = padded number
-                result += QString(repeat, c);
-                break;
-            case 'S': // Fractional second (1..n): 1..n = truncates to decimal places
-                // Qt uses msecs either unpadded or padded to 3 places
-                if (repeat < 3)
-                    result += QLatin1Char('z');
-                else
-                    result += QLatin1String("zzz");
-                break;
-            case 'z': // Time Zone (1..4)
-            case 'Z': // Time Zone (1..5)
-            case 'O': // Time Zone (1, 4)
-            case 'v': // Time Zone (1, 4)
-            case 'V': // Time Zone (1..4)
-            case 'X': // Time Zone (1..5)
-            case 'x': // Time Zone (1..5)
-                result += QLatin1Char('t');
-                break;
-            default:
-                // a..z and A..Z are reserved for format codes, so any occurrence of these not
-                // already processed are not known and so unsupported formats to be ignored.
-                // All other chars are allowed as literals.
-                if (c < QLatin1Char('A') || c > QLatin1Char('z') ||
-                    (c > QLatin1Char('Z') && c < QLatin1Char('a'))) {
-                    result += QString(repeat, c);
-                }
-                break;
+            break;
         }
 
         i += repeat;
@@ -404,6 +433,28 @@ static QVariant macToQtFormat(QStringView sys_fmt)
     return !result.isEmpty() ? QVariant::fromValue(result) : QVariant();
 }
 
+static QVariant getGroupingSizes()
+{
+    // It does not seem like you can directly query the group sizes from CFLocale as there
+    // is no key that corresponds to it, see:
+    // https://developer.apple.com/documentation/corefoundation/cflocalekey
+    // We have to create a number formatter for the locale and query the data from there.
+    // see: https://developer.apple.com/documentation/corefoundation/1390801-cfnumberformattercopyproperty
+    QLocaleData::GroupSizes sizes;
+    QCFType<CFLocaleRef> locale = CFLocaleCopyCurrent();
+    QCFType<CFNumberFormatterRef> numberFormatter =
+            CFNumberFormatterCreate(NULL, locale, kCFNumberFormatterDecimalStyle);
+    CFTypeRef numTref =
+            CFNumberFormatterCopyProperty(numberFormatter, kCFNumberFormatterGroupingSize);
+    CFNumberRef num = static_cast<CFNumberRef>(numTref);
+    int value;
+    if (CFNumberGetValue(num, kCFNumberIntType, &value) && value > 0) {
+        sizes.least = value;
+        sizes.higher = value;
+    }
+    return QVariant::fromValue(sizes);
+}
+
 static QVariant getMacDateFormat(CFDateFormatterStyle style)
 {
     QCFType<CFLocaleRef> l = CFLocaleCopyCurrent();
@@ -433,7 +484,7 @@ static QVariant macMeasurementSystem()
 {
     QCFType<CFLocaleRef> locale = CFLocaleCopyCurrent();
     CFStringRef system = static_cast<CFStringRef>(CFLocaleGetValue(locale, kCFLocaleMeasurementSystem));
-    if (QString::fromCFString(system) == QLatin1String("Metric")) {
+    if (QString::fromCFString(system) == "Metric"_L1) {
         return QLocale::MetricSystem;
     } else {
         return QLocale::ImperialSystem;
@@ -553,7 +604,7 @@ static QLocale::Language codeToLanguage(QStringView s)
     return QLocalePrivate::codeToLanguage(s);
 }
 
-QVariant QSystemLocale::query(QueryType type, QVariant in) const
+QVariant QSystemLocale::query(QueryType type, QVariant &&in) const
 {
     QMacAutoReleasePool pool;
 
@@ -566,6 +617,8 @@ QVariant QSystemLocale::query(QueryType type, QVariant in) const
         return getLocaleValue<QLocalePrivate::codeToScript>(kCFLocaleScriptCode);
     case DecimalPoint:
         return getCFLocaleValue(kCFLocaleDecimalSeparator);
+    case Grouping:
+        return getGroupingSizes();
     case GroupSeparator:
         return getCFLocaleValue(kCFLocaleGroupingSeparator);
     case DateFormatLong:
@@ -621,13 +674,13 @@ QVariant QSystemLocale::query(QueryType type, QVariant in) const
     case CurrencySymbol:
         return macCurrencySymbol(QLocale::CurrencySymbolFormat(in.toUInt()));
     case CurrencyToString:
-        return macFormatCurrency(in.value<QSystemLocale::CurrencyToStringArgument>());
+        return macFormatCurrency(in.value<CurrencyToStringArgument>());
     case UILanguages: {
         QStringList result;
         QCFType<CFArrayRef> languages = CFLocaleCopyPreferredLanguages();
-        const int cnt = CFArrayGetCount(languages);
+        const CFIndex cnt = CFArrayGetCount(languages);
         result.reserve(cnt);
-        for (int i = 0; i < cnt; ++i) {
+        for (CFIndex i = 0; i < cnt; ++i) {
             const QString lang = QString::fromCFString(
                 static_cast<CFStringRef>(CFArrayGetValueAtIndex(languages, i)));
             result.append(lang);
@@ -645,4 +698,40 @@ QVariant QSystemLocale::query(QueryType type, QVariant in) const
 
 #endif // QT_NO_SYSTEMLOCALE
 
+#if !QT_CONFIG(icu)
+
+static QString localeConvertString(const QByteArray &localeID, const QString &str, bool *ok,
+                                   bool toLowerCase)
+{
+    QMacAutoReleasePool pool;
+    Q_ASSERT(ok);
+    NSString *localestring = [[NSString alloc] initWithData:localeID.toNSData()
+                                                   encoding:NSUTF8StringEncoding];
+    NSLocale *locale = [NSLocale localeWithLocaleIdentifier:localestring];
+    if (!locale) {
+        *ok = false;
+        return QString();
+    }
+    *ok = true;
+    NSString *nsstring = str.toNSString();
+    if (toLowerCase)
+        nsstring = [nsstring lowercaseStringWithLocale:locale];
+    else
+        nsstring = [nsstring uppercaseStringWithLocale:locale];
+
+    return QString::fromNSString(nsstring);
+}
+
+QString QLocalePrivate::toLower(const QString &str, bool *ok) const
+{
+    return localeConvertString(bcp47Name('-'), str, ok, true);
+}
+
+QString QLocalePrivate::toUpper(const QString &str, bool *ok) const
+{
+    return localeConvertString(bcp47Name('-'), str, ok, false);
+}
+
+#endif
+
 QT_END_NAMESPACE
