@@ -165,6 +165,14 @@ void QReadWriteLock::destroyRecursive(QReadWriteLockPrivate *d)
     \sa unlock(), lockForRead()
 */
 
+static Q_ALWAYS_INLINE bool fastTryLock(QAtomicPointer<QReadWriteLockPrivate> &d_ptr,
+                                        QReadWriteLockPrivate *dummyValue,
+                                        QReadWriteLockPrivate *&d)
+{
+    // Succeed fast if not contended
+    return d == nullptr && d_ptr.testAndSetAcquire(nullptr, dummyValue, d);
+}
+
 /*!
     \overload
     \since 6.6
@@ -184,9 +192,8 @@ void QReadWriteLock::destroyRecursive(QReadWriteLockPrivate *d)
 */
 bool QReadWriteLock::tryLockForRead(QDeadlineTimer timeout)
 {
-    // Fast case: non contended:
     QReadWriteLockPrivate *d = d_ptr.loadRelaxed();
-    if (d == nullptr && d_ptr.testAndSetAcquire(nullptr, dummyLockedForRead, d))
+    if (fastTryLock(d_ptr, dummyLockedForRead, d))
         return true;
     return contendedTryLockForRead(d_ptr, timeout, d);
 }
@@ -195,10 +202,11 @@ Q_NEVER_INLINE static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLock
                                                    QDeadlineTimer timeout, QReadWriteLockPrivate *d)
 {
     while (true) {
+        qYieldCpu();
         if (d == nullptr) {
-            if (!d_ptr.testAndSetAcquire(nullptr, dummyLockedForRead, d))
-                continue;
-            return true;
+            if (fastTryLock(d_ptr, dummyLockedForRead, d))
+                return true;
+            continue;
         }
 
         if ((quintptr(d) & StateMask) == StateLockedForRead) {
@@ -302,9 +310,8 @@ Q_NEVER_INLINE static bool contendedTryLockForRead(QAtomicPointer<QReadWriteLock
 */
 bool QReadWriteLock::tryLockForWrite(QDeadlineTimer timeout)
 {
-    // Fast case: non contended:
     QReadWriteLockPrivate *d = d_ptr.loadRelaxed();
-    if (d == nullptr && d_ptr.testAndSetAcquire(nullptr, dummyLockedForWrite, d))
+    if (fastTryLock(d_ptr, dummyLockedForWrite, d))
         return true;
     return contendedTryLockForWrite(d_ptr, timeout, d);
 }
@@ -313,10 +320,11 @@ Q_NEVER_INLINE static bool contendedTryLockForWrite(QAtomicPointer<QReadWriteLoc
                                                     QDeadlineTimer timeout, QReadWriteLockPrivate *d)
 {
     while (true) {
+        qYieldCpu();
         if (d == nullptr) {
-            if (!d_ptr.testAndSetAcquire(d, dummyLockedForWrite, d))
-                continue;
-            return true;
+            if (fastTryLock(d_ptr, dummyLockedForWrite, d))
+                return true;
+            continue;
         }
 
         if (isUncontendedLocked(d)) {
