@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2019 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Giuseppe D'Angelo <giuseppe.dangelo@kdab.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2019 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Giuseppe D'Angelo <giuseppe.dangelo@kdab.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbrush.h"
 #include "qpixmap.h"
@@ -54,9 +18,17 @@
 #include <QtCore/qnumeric.h>
 #include <QtCore/qfile.h>
 #include <QtCore/qmutex.h>
+#include <QtCore/private/qoffsetstringarray_p.h>
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0)
+// Avoid an ABI break due to the QScopedPointer->std::unique_ptr change
+static_assert(sizeof(QBrush::DataPtr) == sizeof(QScopedPointer<QBrushData, QBrushDataPointerDeleter>));
+#endif
+
 const uchar *qt_patternForBrush(int brushStyle, bool invert)
 {
     Q_ASSERT(brushStyle > Qt::SolidPattern && brushStyle < Qt::LinearGradientPattern);
@@ -109,7 +81,7 @@ Q_GUI_EXPORT QPixmap qt_pixmapForBrush(int brushStyle, bool invert)
 {
 
     QPixmap pm;
-    QString key = QLatin1String("$qt-brush$")
+    QString key = "$qt-brush$"_L1
                   % HexString<uint>(brushStyle)
                   % QLatin1Char(invert ? '1' : '0');
     if (!QPixmapCache::find(key, &pm)) {
@@ -176,6 +148,10 @@ Q_GUI_EXPORT QImage qt_imageForBrush(int brushStyle, bool invert)
     return qt_brushPatternImageCache()->getImage(brushStyle, invert);
 }
 
+struct QBasicBrushData : public QBrushData
+{
+};
+
 struct QTexturedBrushData : public QBrushData
 {
     QTexturedBrushData() {
@@ -231,7 +207,7 @@ bool Q_GUI_EXPORT qHasPixmapTexture(const QBrush& brush)
 {
     if (brush.style() != Qt::TexturePattern)
         return false;
-    QTexturedBrushData *tx_data = static_cast<QTexturedBrushData *>(brush.d.data());
+    QTexturedBrushData *tx_data = static_cast<QTexturedBrushData *>(brush.d.get());
     return tx_data->m_has_pixmap_texture;
 }
 
@@ -240,31 +216,42 @@ struct QGradientBrushData : public QBrushData
     QGradient gradient;
 };
 
-struct QBrushDataPointerDeleter
+static void deleteData(QBrushData *d)
 {
-    static inline void deleteData(QBrushData *d)
-    {
-        switch (d->style) {
-        case Qt::TexturePattern:
-            delete static_cast<QTexturedBrushData*>(d);
-            break;
-        case Qt::LinearGradientPattern:
-        case Qt::RadialGradientPattern:
-        case Qt::ConicalGradientPattern:
-            delete static_cast<QGradientBrushData*>(d);
-            break;
-        default:
-            delete d;
-        }
+    switch (d->style) {
+    case Qt::TexturePattern:
+        delete static_cast<QTexturedBrushData*>(d);
+        break;
+    case Qt::LinearGradientPattern:
+    case Qt::RadialGradientPattern:
+    case Qt::ConicalGradientPattern:
+        delete static_cast<QGradientBrushData*>(d);
+        break;
+    case Qt::NoBrush:
+    case Qt::SolidPattern:
+    case Qt::Dense1Pattern:
+    case Qt::Dense2Pattern:
+    case Qt::Dense3Pattern:
+    case Qt::Dense4Pattern:
+    case Qt::Dense5Pattern:
+    case Qt::Dense6Pattern:
+    case Qt::Dense7Pattern:
+    case Qt::HorPattern:
+    case Qt::VerPattern:
+    case Qt::CrossPattern:
+    case Qt::BDiagPattern:
+    case Qt::FDiagPattern:
+    case Qt::DiagCrossPattern:
+        delete static_cast<QBasicBrushData*>(d);
+        break;
     }
+}
 
-    static inline void cleanup(QBrushData *d)
-    {
-        if (d && !d->ref.deref()) {
-            deleteData(d);
-        }
-    }
-};
+void QBrushDataPointerDeleter::operator()(QBrushData *d) const noexcept
+{
+    if (d && !d->ref.deref())
+        deleteData(d);
+}
 
 /*!
     \class QBrush
@@ -350,8 +337,8 @@ struct QBrushDataPointerDeleter
 class QNullBrushData
 {
 public:
-    QBrushData *brush;
-    QNullBrushData() : brush(new QBrushData)
+    QBasicBrushData *brush;
+    QNullBrushData() : brush(new QBasicBrushData)
     {
         brush->ref.storeRelaxed(1);
         brush->style = Qt::BrushStyle(0);
@@ -408,8 +395,21 @@ void QBrush::init(const QColor &color, Qt::BrushStyle style)
     case Qt::ConicalGradientPattern:
         d.reset(new QGradientBrushData);
         break;
-    default:
-        d.reset(new QBrushData);
+    case Qt::SolidPattern:
+    case Qt::Dense1Pattern:
+    case Qt::Dense2Pattern:
+    case Qt::Dense3Pattern:
+    case Qt::Dense4Pattern:
+    case Qt::Dense5Pattern:
+    case Qt::Dense6Pattern:
+    case Qt::Dense7Pattern:
+    case Qt::HorPattern:
+    case Qt::VerPattern:
+    case Qt::CrossPattern:
+    case Qt::BDiagPattern:
+    case Qt::FDiagPattern:
+    case Qt::DiagCrossPattern:
+        d.reset(new QBasicBrushData);
         break;
     }
     d->ref.storeRelaxed(1);
@@ -532,7 +532,7 @@ QBrush::QBrush(Qt::GlobalColor color, const QPixmap &pixmap)
 */
 
 QBrush::QBrush(const QBrush &other)
-    : d(other.d.data())
+    : d(other.d.get())
 {
     d->ref.ref();
 }
@@ -559,7 +559,7 @@ QBrush::QBrush(const QGradient &gradient)
     };
 
     init(QColor(), enum_table[gradient.type()]);
-    QGradientBrushData *grad = static_cast<QGradientBrushData *>(d.data());
+    QGradientBrushData *grad = static_cast<QGradientBrushData *>(d.get());
     grad->gradient = gradient;
 }
 
@@ -571,12 +571,7 @@ QBrush::~QBrush()
 {
 }
 
-void QBrush::cleanUp(QBrushData *x)
-{
-    QBrushDataPointerDeleter::deleteData(x);
-}
-
-static Q_DECL_CONSTEXPR inline bool use_same_brushdata(Qt::BrushStyle lhs, Qt::BrushStyle rhs)
+static constexpr inline bool use_same_brushdata(Qt::BrushStyle lhs, Qt::BrushStyle rhs)
 {
     return lhs == rhs // includes Qt::TexturePattern
         || (lhs >= Qt::NoBrush && lhs <= Qt::DiagCrossPattern && rhs >= Qt::NoBrush && rhs <= Qt::DiagCrossPattern)
@@ -591,12 +586,12 @@ void QBrush::detach(Qt::BrushStyle newStyle)
         return;
     }
 
-    QScopedPointer<QBrushData, QBrushDataPointerDeleter> x;
+    DataPtr x;
     switch(newStyle) {
     case Qt::TexturePattern: {
         QTexturedBrushData *tbd = new QTexturedBrushData;
         if (d->style == Qt::TexturePattern) {
-            QTexturedBrushData *data = static_cast<QTexturedBrushData *>(d.data());
+            QTexturedBrushData *data = static_cast<QTexturedBrushData *>(d.get());
             if (data->m_has_pixmap_texture)
                 tbd->setPixmap(data->pixmap());
             else
@@ -614,7 +609,7 @@ void QBrush::detach(Qt::BrushStyle newStyle)
         case Qt::RadialGradientPattern:
         case Qt::ConicalGradientPattern:
             gbd->gradient =
-                    static_cast<QGradientBrushData *>(d.data())->gradient;
+                    static_cast<QGradientBrushData *>(d.get())->gradient;
             break;
         default:
             break;
@@ -622,8 +617,22 @@ void QBrush::detach(Qt::BrushStyle newStyle)
         x.reset(gbd);
         break;
         }
-    default:
-        x.reset(new QBrushData);
+    case Qt::NoBrush:
+    case Qt::SolidPattern:
+    case Qt::Dense1Pattern:
+    case Qt::Dense2Pattern:
+    case Qt::Dense3Pattern:
+    case Qt::Dense4Pattern:
+    case Qt::Dense5Pattern:
+    case Qt::Dense6Pattern:
+    case Qt::Dense7Pattern:
+    case Qt::HorPattern:
+    case Qt::VerPattern:
+    case Qt::CrossPattern:
+    case Qt::BDiagPattern:
+    case Qt::FDiagPattern:
+    case Qt::DiagCrossPattern:
+        x.reset(new QBasicBrushData);
         break;
     }
     x->ref.storeRelaxed(1); // must be first lest the QBrushDataPointerDeleter turns into a no-op
@@ -635,19 +644,49 @@ void QBrush::detach(Qt::BrushStyle newStyle)
 
 
 /*!
-    \fn QBrush &QBrush::operator=(const QBrush &brush)
-
     Assigns the given \a brush to \e this brush and returns a
     reference to \e this brush.
 */
 
-QBrush &QBrush::operator=(const QBrush &b)
+QBrush &QBrush::operator=(const QBrush &brush)
 {
-    if (d == b.d)
+    if (d == brush.d)
         return *this;
 
-    b.d->ref.ref();
-    d.reset(b.d.data());
+    brush.d->ref.ref();
+    d.reset(brush.d.get());
+    return *this;
+}
+
+/*!
+    \fn QBrush &QBrush::operator=(QColor color)
+    \fn QBrush &QBrush::operator=(Qt::GlobalColor color)
+    \overload
+    \since 6.9
+
+    Makes this brush a solid pattern brush of the given \a color,
+    and returns a reference to \e this brush.
+*/
+QBrush &QBrush::operator=(QColor color)
+{
+    detach(Qt::SolidPattern);
+    d->color = color;
+    d->transform = {};
+    return *this;
+}
+
+/*!
+    \overload
+    \since 6.9
+
+    Makes this brush a black brush of the given \a style,
+    and returns a reference to \e this brush.
+*/
+QBrush &QBrush::operator=(Qt::BrushStyle style)
+{
+    detach(style);
+    d->color = Qt::black;
+    d->transform = {};
     return *this;
 }
 
@@ -662,9 +701,7 @@ QBrush &QBrush::operator=(const QBrush &b)
 /*!
     \fn void QBrush::swap(QBrush &other)
     \since 4.8
-
-    Swaps brush \a other with this brush. This operation is very
-    fast and never fails.
+    \memberswap{brush}
 */
 
 /*!
@@ -672,7 +709,7 @@ QBrush &QBrush::operator=(const QBrush &b)
 */
 QBrush::operator QVariant() const
 {
-    return QVariant(QMetaType::QBrush, this);
+    return QVariant::fromValue(*this);
 }
 
 /*!
@@ -748,7 +785,7 @@ void QBrush::setColor(const QColor &c)
 QPixmap QBrush::texture() const
 {
     return d->style == Qt::TexturePattern
-                     ? (static_cast<QTexturedBrushData *>(d.data()))->pixmap()
+                     ? (static_cast<QTexturedBrushData *>(d.get()))->pixmap()
                      : QPixmap();
 }
 
@@ -766,7 +803,7 @@ void QBrush::setTexture(const QPixmap &pixmap)
 {
     if (!pixmap.isNull()) {
         detach(Qt::TexturePattern);
-        QTexturedBrushData *data = static_cast<QTexturedBrushData *>(d.data());
+        QTexturedBrushData *data = static_cast<QTexturedBrushData *>(d.get());
         data->setPixmap(pixmap);
     } else {
         detach(Qt::NoBrush);
@@ -789,7 +826,7 @@ void QBrush::setTexture(const QPixmap &pixmap)
 QImage QBrush::textureImage() const
 {
     return d->style == Qt::TexturePattern
-                     ? (static_cast<QTexturedBrushData *>(d.data()))->image()
+                     ? (static_cast<QTexturedBrushData *>(d.get()))->image()
                      : QImage();
 }
 
@@ -814,7 +851,7 @@ void QBrush::setTextureImage(const QImage &image)
 {
     if (!image.isNull()) {
         detach(Qt::TexturePattern);
-        QTexturedBrushData *data = static_cast<QTexturedBrushData *>(d.data());
+        QTexturedBrushData *data = static_cast<QTexturedBrushData *>(d.get());
         data->setImage(image);
     } else {
         detach(Qt::NoBrush);
@@ -830,7 +867,7 @@ const QGradient *QBrush::gradient() const
     if (d->style == Qt::LinearGradientPattern
         || d->style == Qt::RadialGradientPattern
         || d->style == Qt::ConicalGradientPattern) {
-        return &static_cast<const QGradientBrushData *>(d.data())->gradient;
+        return &static_cast<const QGradientBrushData *>(d.get())->gradient;
     }
     return nullptr;
 }
@@ -866,7 +903,7 @@ Q_GUI_EXPORT bool qt_isExtendedRadialGradient(const QBrush &brush)
 
 bool QBrush::isOpaque() const
 {
-    bool opaqueColor = d->color.alpha() == 255;
+    bool opaqueColor = d->color.alphaF() >= 1.0f;
 
     // Test awfully simple case first
     if (d->style == Qt::SolidPattern)
@@ -880,7 +917,7 @@ bool QBrush::isOpaque() const
         || d->style == Qt::ConicalGradientPattern) {
         QGradientStops stops = gradient()->stops();
         for (int i=0; i<stops.size(); ++i)
-            if (stops.at(i).second.alpha() != 255)
+            if (stops.at(i).second.alphaF() < 1.0f)
                 return false;
         return true;
     } else if (d->style == Qt::TexturePattern) {
@@ -892,26 +929,6 @@ bool QBrush::isOpaque() const
     return false;
 }
 
-
-#if QT_DEPRECATED_SINCE(5, 15)
-/*!
-    \since 4.2
-    \obsolete
-
-    Use setTransform() instead.
-
-    Sets \a matrix as an explicit transformation matrix on the
-    current brush. The brush transformation matrix is merged with
-    QPainter transformation matrix to produce the final result.
-
-    \sa matrix()
-*/
-void QBrush::setMatrix(const QMatrix &matrix)
-{
-    setTransform(QTransform(matrix));
-}
-#endif // QT_DEPRECATED_SINCE(5, 15)
-
 /*!
     \since 4.3
 
@@ -928,20 +945,6 @@ void QBrush::setTransform(const QTransform &matrix)
 }
 
 
-#if QT_DEPRECATED_SINCE(5, 15)
-/*!
-    \fn void QBrush::matrix() const
-    \since 4.2
-    \obsolete
-
-    Use transform() instead.
-
-    Returns the current transformation matrix for the brush.
-
-    \sa setMatrix()
-*/
-#endif // QT_DEPRECATED_SINCE(5, 15)
-
 /*!
     \fn bool QBrush::operator!=(const QBrush &brush) const
 
@@ -982,16 +985,16 @@ bool QBrush::operator==(const QBrush &b) const
             const QPixmap *us = nullptr, *them = nullptr;
             qint64 cacheKey1, cacheKey2;
             if (qHasPixmapTexture(*this)) {
-                us = (static_cast<QTexturedBrushData *>(d.data()))->m_pixmap;
+                us = (static_cast<QTexturedBrushData *>(d.get()))->m_pixmap;
                 cacheKey1 = us->cacheKey();
             } else
-                cacheKey1 = (static_cast<QTexturedBrushData *>(d.data()))->image().cacheKey();
+                cacheKey1 = (static_cast<QTexturedBrushData *>(d.get()))->image().cacheKey();
 
             if (qHasPixmapTexture(b)) {
-                them = (static_cast<QTexturedBrushData *>(b.d.data()))->m_pixmap;
+                them = (static_cast<QTexturedBrushData *>(b.d.get()))->m_pixmap;
                 cacheKey2 = them->cacheKey();
             } else
-                cacheKey2 = (static_cast<QTexturedBrushData *>(b.d.data()))->image().cacheKey();
+                cacheKey2 = (static_cast<QTexturedBrushData *>(b.d.get()))->image().cacheKey();
 
             if (cacheKey1 != cacheKey2)
                 return false;
@@ -1008,8 +1011,8 @@ bool QBrush::operator==(const QBrush &b) const
     case Qt::RadialGradientPattern:
     case Qt::ConicalGradientPattern:
         {
-            const QGradientBrushData *d1 = static_cast<QGradientBrushData *>(d.data());
-            const QGradientBrushData *d2 = static_cast<QGradientBrushData *>(b.d.data());
+            const QGradientBrushData *d1 = static_cast<QGradientBrushData *>(d.get());
+            const QGradientBrushData *d2 = static_cast<QGradientBrushData *>(b.d.get());
             return d1->gradient == d2->gradient;
         }
     default:
@@ -1017,13 +1020,40 @@ bool QBrush::operator==(const QBrush &b) const
     }
 }
 
+/*!
+    \internal
+*/
+bool QBrush::doCompareEqualColor(QColor rhs) const noexcept
+{
+    return style() == Qt::SolidPattern && color() == rhs && d->transform.isIdentity();
+}
+
+/*!
+    \internal
+*/
+bool QBrush::doCompareEqualStyle(Qt::BrushStyle rhs) const noexcept
+{
+    switch (rhs) {
+    case Qt::NoBrush:
+    case Qt::TexturePattern:
+    case Qt::LinearGradientPattern:
+    case Qt::RadialGradientPattern:
+    case Qt::ConicalGradientPattern:
+        // A brush constructed only from one of those styles will end up
+        // using NoBrush (see qbrush_check_type)
+        return style() == Qt::NoBrush;
+    default:
+        return style() == rhs && color() == QColor(0, 0, 0);
+    }
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 /*!
   \internal
 */
 QDebug operator<<(QDebug dbg, const QBrush &b)
 {
-    static const char BRUSH_STYLES[][24] = {
+    static constexpr auto BRUSH_STYLES = qOffsetStringArray(
      "NoBrush",
      "SolidPattern",
      "Dense1Pattern",
@@ -1044,7 +1074,7 @@ QDebug operator<<(QDebug dbg, const QBrush &b)
      "ConicalGradientPattern",
      "", "", "", "", "", "",
      "TexturePattern" // 24
-    };
+    );
 
     QDebugStateSaver saver(dbg);
     dbg.nospace() << "QBrush(" << b.color() << ',' << BRUSH_STYLES[b.style()] << ')';
@@ -1105,11 +1135,11 @@ QDataStream &operator<<(QDataStream &s, const QBrush &b)
             // ensure that we write doubles here instead of streaming the stops
             // directly; otherwise, platforms that redefine qreal might generate
             // data that cannot be read on other platforms.
-            QVector<QGradientStop> stops = gradient->stops();
+            QList<QGradientStop> stops = gradient->stops();
             s << quint32(stops.size());
             for (int i = 0; i < stops.size(); ++i) {
                 const QGradientStop &stop = stops.at(i);
-                s << QPair<double, QColor>(double(stop.first), stop.second);
+                s << std::pair<double, QColor>(double(stop.first), stop.second);
             }
         }
 
@@ -1120,6 +1150,8 @@ QDataStream &operator<<(QDataStream &s, const QBrush &b)
             s << static_cast<const QRadialGradient *>(gradient)->center();
             s << static_cast<const QRadialGradient *>(gradient)->focalPoint();
             s << (double) static_cast<const QRadialGradient *>(gradient)->radius();
+            if (s.version() >= QDataStream::Qt_6_0)
+                s << (double) static_cast<const QRadialGradient *>(gradient)->focalRadius();
         } else { // type == Conical
             s << static_cast<const QConicalGradient *>(gradient)->center();
             s << (double) static_cast<const QConicalGradient *>(gradient)->angle();
@@ -1193,7 +1225,7 @@ QDataStream &operator>>(QDataStream &s, QBrush &b)
             stops.reserve(numStops);
             for (quint32 i = 0; i < numStops; ++i) {
                 s >> n >> c;
-                stops << QPair<qreal, QColor>(n, c);
+                stops << std::pair<qreal, QColor>(n, c);
             }
         }
 
@@ -1210,6 +1242,7 @@ QDataStream &operator>>(QDataStream &s, QBrush &b)
         } else if (type == QGradient::RadialGradient) {
             QPointF center, focal;
             double radius;
+            double focalRadius = 0;
             s >> center;
             s >> focal;
             s >> radius;
@@ -1218,6 +1251,9 @@ QDataStream &operator>>(QDataStream &s, QBrush &b)
             rg.setSpread(spread);
             rg.setCoordinateMode(cmode);
             rg.setInterpolationMode(imode);
+            if (s.version() >= QDataStream::Qt_6_0)
+                s >> focalRadius;
+            rg.setFocalRadius(focalRadius);
             b = QBrush(rg);
         } else { // type == QGradient::ConicalGradient
             QPointF center;
@@ -1305,7 +1341,7 @@ QDataStream &operator>>(QDataStream &s, QBrush &b)
     \snippet brush/brush.cpp 1
 
     It is possible to repeat or reflect the gradient outside its area
-    by specifiying the \l {QGradient::Spread}{spread method} using the
+    by specifying the \l {QGradient::Spread}{spread method} using the
     setSpread() function. The default is to pad the outside area with
     the color at the closest stop point. The currently set \l
     {QGradient::Spread}{spread method} can be retrieved using the
@@ -1346,7 +1382,7 @@ QDataStream &operator>>(QDataStream &s, QBrush &b)
     \internal
 */
 QGradient::QGradient()
-    : m_type(NoGradient), dummy(nullptr)
+    : m_type(NoGradient)
 {
 }
 
@@ -1355,7 +1391,176 @@ QGradient::QGradient()
     \since 5.12
 
     This enum specifies a set of predefined presets for QGradient,
-    based on the gradients from https://webgradients.com/.
+    based on the gradients from \l {https://webgradients.com/}.
+
+    \value WarmFlame
+    \value NightFade
+    \value SpringWarmth
+    \value JuicyPeach
+    \value YoungPassion
+    \value LadyLips
+    \value SunnyMorning
+    \value RainyAshville
+    \value FrozenDreams
+    \value WinterNeva
+    \value DustyGrass
+    \value TemptingAzure
+    \value HeavyRain
+    \value AmyCrisp
+    \value MeanFruit
+    \value DeepBlue
+    \value RipeMalinka
+    \value CloudyKnoxville
+    \value MalibuBeach
+    \value NewLife
+    \value TrueSunset
+    \value MorpheusDen
+    \value RareWind
+    \value NearMoon
+    \value WildApple
+    \value SaintPetersburg
+    \value PlumPlate
+    \value EverlastingSky
+    \value HappyFisher
+    \value Blessing
+    \value SharpeyeEagle
+    \value LadogaBottom
+    \value LemonGate
+    \value ItmeoBranding
+    \value ZeusMiracle
+    \value OldHat
+    \value StarWine
+    \value HappyAcid
+    \value AwesomePine
+    \value NewYork
+    \value ShyRainbow
+    \value MixedHopes
+    \value FlyHigh
+    \value StrongBliss
+    \value FreshMilk
+    \value SnowAgain
+    \value FebruaryInk
+    \value KindSteel
+    \value SoftGrass
+    \value GrownEarly
+    \value SharpBlues
+    \value ShadyWater
+    \value DirtyBeauty
+    \value GreatWhale
+    \value TeenNotebook
+    \value PoliteRumors
+    \value SweetPeriod
+    \value WideMatrix
+    \value SoftCherish
+    \value RedSalvation
+    \value BurningSpring
+    \value NightParty
+    \value SkyGlider
+    \value HeavenPeach
+    \value PurpleDivision
+    \value AquaSplash
+    \value SpikyNaga
+    \value LoveKiss
+    \value CleanMirror
+    \value PremiumDark
+    \value ColdEvening
+    \value CochitiLake
+    \value SummerGames
+    \value PassionateBed
+    \value MountainRock
+    \value DesertHump
+    \value JungleDay
+    \value PhoenixStart
+    \value OctoberSilence
+    \value FarawayRiver
+    \value AlchemistLab
+    \value OverSun
+    \value PremiumWhite
+    \value MarsParty
+    \value EternalConstance
+    \value JapanBlush
+    \value SmilingRain
+    \value CloudyApple
+    \value BigMango
+    \value HealthyWater
+    \value AmourAmour
+    \value RiskyConcrete
+    \value StrongStick
+    \value ViciousStance
+    \value PaloAlto
+    \value HappyMemories
+    \value MidnightBloom
+    \value Crystalline
+    \value PartyBliss
+    \value ConfidentCloud
+    \value LeCocktail
+    \value RiverCity
+    \value FrozenBerry
+    \value ChildCare
+    \value FlyingLemon
+    \value NewRetrowave
+    \value HiddenJaguar
+    \value AboveTheSky
+    \value Nega
+    \value DenseWater
+    \value Seashore
+    \value MarbleWall
+    \value CheerfulCaramel
+    \value NightSky
+    \value MagicLake
+    \value YoungGrass
+    \value ColorfulPeach
+    \value GentleCare
+    \value PlumBath
+    \value HappyUnicorn
+    \value AfricanField
+    \value SolidStone
+    \value OrangeJuice
+    \value GlassWater
+    \value NorthMiracle
+    \value FruitBlend
+    \value MillenniumPine
+    \value HighFlight
+    \value MoleHall
+    \value SpaceShift
+    \value ForestInei
+    \value RoyalGarden
+    \value RichMetal
+    \value JuicyCake
+    \value SmartIndigo
+    \value SandStrike
+    \value NorseBeauty
+    \value AquaGuidance
+    \value SunVeggie
+    \value SeaLord
+    \value BlackSea
+    \value GrassShampoo
+    \value LandingAircraft
+    \value WitchDance
+    \value SleeplessNight
+    \value AngelCare
+    \value CrystalRiver
+    \value SoftLipstick
+    \value SaltMountain
+    \value PerfectWhite
+    \value FreshOasis
+    \value StrictNovember
+    \value MorningSalad
+    \value DeepRelief
+    \value SeaStrike
+    \value NightCall
+    \value SupremeSky
+    \value LightBlue
+    \value MindCrawl
+    \value LilyMeadow
+    \value SugarLollipop
+    \value SweetDessert
+    \value MagicRay
+    \value TeenParty
+    \value FrozenHeat
+    \value GagarinView
+    \value FabledSunset
+    \value PerfectBlue
 */
 
 #include "webgradients.cpp"
@@ -1372,10 +1577,9 @@ QGradient::QGradient()
 */
 QGradient::QGradient(Preset preset)
     : m_type(LinearGradient)
-    , m_spread(PadSpread)
     , m_stops(qt_preset_gradient_stops(preset))
     , m_data(qt_preset_gradient_data[preset - 1])
-    , dummy(qt_preset_gradient_dummy())
+    , m_coordinateMode(ObjectMode)
 {
 }
 
@@ -1504,7 +1708,6 @@ static inline bool ok(const QGradientStops &stops)
 */
 void QGradient::setStops(const QGradientStops &stops)
 {
-    // ## Qt 6: consider taking \a stops by value, so we can move into m_stops
     if (Q_LIKELY(ok(stops))) {
         // fast path for the common case: if everything is ok with the stops, just copy them
         m_stops = stops;
@@ -1529,15 +1732,14 @@ void QGradient::setStops(const QGradientStops &stops)
 QGradientStops QGradient::stops() const
 {
     if (m_stops.isEmpty()) {
-        QGradientStops tmp;
-        tmp << QGradientStop(0, Qt::black) << QGradientStop(1, Qt::white);
-        return tmp;
+        static constexpr QGradientStop blackAndWhite[] = {
+            {0, QColorConstants::Black}, {1, QColorConstants::White},
+        };
+        return QGradientStops::fromReadOnlyData(blackAndWhite);
     }
     return m_stops;
 }
 
-#define Q_DUMMY_ACCESSOR union {void *p; uint i;}; p = dummy;
-
 /*!
     \enum QGradient::CoordinateMode
     \since 4.4
@@ -1570,8 +1772,7 @@ QGradientStops QGradient::stops() const
 */
 QGradient::CoordinateMode QGradient::coordinateMode() const
 {
-    Q_DUMMY_ACCESSOR
-    return CoordinateMode(i & 0x03);
+    return m_coordinateMode;
 }
 
 /*!
@@ -1582,10 +1783,7 @@ QGradient::CoordinateMode QGradient::coordinateMode() const
 */
 void QGradient::setCoordinateMode(CoordinateMode mode)
 {
-    Q_DUMMY_ACCESSOR
-    i &= ~0x03;
-    i |= uint(mode);
-    dummy = p;
+    m_coordinateMode = mode;
 }
 
 /*!
@@ -1608,8 +1806,7 @@ void QGradient::setCoordinateMode(CoordinateMode mode)
 */
 QGradient::InterpolationMode QGradient::interpolationMode() const
 {
-    Q_DUMMY_ACCESSOR
-    return InterpolationMode((i >> 2) & 0x01);
+    return m_interpolationMode;
 }
 
 /*!
@@ -1621,10 +1818,7 @@ QGradient::InterpolationMode QGradient::interpolationMode() const
 */
 void QGradient::setInterpolationMode(InterpolationMode mode)
 {
-    Q_DUMMY_ACCESSOR
-    i &= ~(1 << 2);
-    i |= (uint(mode) << 2);
-    dummy = p;
+    m_interpolationMode = mode;
 }
 
 /*!
@@ -1647,7 +1841,8 @@ bool QGradient::operator==(const QGradient &gradient) const
 {
     if (gradient.m_type != m_type
         || gradient.m_spread != m_spread
-        || gradient.dummy != dummy) return false;
+        || gradient.m_coordinateMode != m_coordinateMode
+        || gradient.m_interpolationMode != m_interpolationMode) return false;
 
     if (m_type == LinearGradient) {
         if (m_data.linear.x1 != gradient.m_data.linear.x1
@@ -1660,7 +1855,8 @@ bool QGradient::operator==(const QGradient &gradient) const
             || m_data.radial.cy != gradient.m_data.radial.cy
             || m_data.radial.fx != gradient.m_data.radial.fx
             || m_data.radial.fy != gradient.m_data.radial.fy
-            || m_data.radial.cradius != gradient.m_data.radial.cradius)
+            || m_data.radial.cradius != gradient.m_data.radial.cradius
+            || m_data.radial.fradius != gradient.m_data.radial.fradius)
             return false;
     } else { // m_type == ConicalGradient
         if (m_data.conical.cx != gradient.m_data.conical.cx
@@ -1934,6 +2130,7 @@ QRadialGradient::QRadialGradient(const QPointF &center, qreal radius, const QPoi
     m_data.radial.cx = center.x();
     m_data.radial.cy = center.y();
     m_data.radial.cradius = radius;
+    m_data.radial.fradius = 0;
 
     QPointF adapted_focal = qt_radial_gradient_adapt_focal_point(center, radius, focalPoint);
     m_data.radial.fx = adapted_focal.x();
@@ -1953,6 +2150,7 @@ QRadialGradient::QRadialGradient(const QPointF &center, qreal radius)
     m_data.radial.cx = center.x();
     m_data.radial.cy = center.y();
     m_data.radial.cradius = radius;
+    m_data.radial.fradius = 0;
     m_data.radial.fx = center.x();
     m_data.radial.fy = center.y();
 }
@@ -1998,6 +2196,7 @@ QRadialGradient::QRadialGradient()
     m_data.radial.cx = 0;
     m_data.radial.cy = 0;
     m_data.radial.cradius = 1;
+    m_data.radial.fradius = 0;
     m_data.radial.fx = 0;
     m_data.radial.fy = 0;
 }
@@ -2015,10 +2214,10 @@ QRadialGradient::QRadialGradient(const QPointF &center, qreal centerRadius, cons
     m_data.radial.cx = center.x();
     m_data.radial.cy = center.y();
     m_data.radial.cradius = centerRadius;
+    m_data.radial.fradius = focalRadius;
 
     m_data.radial.fx = focalPoint.x();
     m_data.radial.fy = focalPoint.y();
-    setFocalRadius(focalRadius);
 }
 
 /*!
@@ -2035,10 +2234,10 @@ QRadialGradient::QRadialGradient(qreal cx, qreal cy, qreal centerRadius, qreal f
     m_data.radial.cx = cx;
     m_data.radial.cy = cy;
     m_data.radial.cradius = centerRadius;
+    m_data.radial.fradius = focalRadius;
 
     m_data.radial.fx = fx;
     m_data.radial.fy = fy;
-    setFocalRadius(focalRadius);
 }
 
 /*!
@@ -2154,12 +2353,7 @@ void QRadialGradient::setCenterRadius(qreal radius)
 qreal QRadialGradient::focalRadius() const
 {
     Q_ASSERT(m_type == RadialGradient);
-    Q_DUMMY_ACCESSOR
-
-    // mask away low three bits
-    union { float f; quint32 i; } u;
-    u.i = i & ~0x07;
-    return u.f;
+    return m_data.radial.fradius;
 }
 
 /*!
@@ -2171,17 +2365,7 @@ qreal QRadialGradient::focalRadius() const
 void QRadialGradient::setFocalRadius(qreal radius)
 {
     Q_ASSERT(m_type == RadialGradient);
-    Q_DUMMY_ACCESSOR
-
-    // Since there's no QGradientData, we only have the dummy void * to
-    // store additional data in. The three lowest bits are already
-    // taken, thus we cut the three lowest bits from the significand
-    // and store the radius as a float.
-    union { float f; quint32 i; } u;
-    u.f = float(radius);
-    // add 0x04 to round up when we drop the three lowest bits
-    i |= (u.i + 0x04) & ~0x07;
-    dummy = p;
+    m_data.radial.fradius = radius;
 }
 
 /*!
@@ -2391,14 +2575,14 @@ void QConicalGradient::setAngle(qreal angle)
     \typedef QGradientStop
     \relates QGradient
 
-    Typedef for QPair<\l qreal, QColor>.
+    Typedef for std::pair<\l qreal, QColor>.
 */
 
 /*!
     \typedef QGradientStops
     \relates QGradient
 
-    Typedef for QVector<QGradientStop>.
+    Typedef for QList<QGradientStop>.
 */
 
 /*!
@@ -2426,6 +2610,6 @@ void QConicalGradient::setAngle(qreal angle)
     \sa setTransform()
 */
 
-#undef Q_DUMMY_ACCESSOR
-
 QT_END_NAMESPACE
+
+#include "moc_qbrush.cpp"
