@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 #include "qpaintengine.h"
 #include "qpaintengine_p.h"
 #include "qpainter_p.h"
@@ -52,6 +16,7 @@
 #include <private/qpaintengineex_p.h>
 #include <private/qtextengine_p.h>
 
+#include <memory>
 
 QT_BEGIN_NAMESPACE
 
@@ -185,7 +150,7 @@ QFont QTextItem::font() const
   emulated results. Some features cannot be emulated: AlphaBlend and PorterDuff.
 
   \value AlphaBlend         The engine can alpha blend primitives.
-  \value Antialiasing       The engine can use antialising to improve the appearance
+  \value Antialiasing       The engine can use antialiasing to improve the appearance
                             of rendered primitives.
   \value BlendModes         The engine supports blending modes.
   \value BrushStroke        The engine supports drawing strokes that
@@ -516,6 +481,8 @@ void QPaintEngine::drawEllipse(const QRectF &rect)
 }
 
 /*!
+    \overload
+
     The default implementation of this function calls the floating
     point version of this function
 */
@@ -1009,9 +976,9 @@ QPixmap QPaintEngine::createPixmap(QSize size)
         return QPixmap();
     }
 
-    QScopedPointer<QPlatformPixmap> data(QGuiApplicationPrivate::platformIntegration()->createPlatformPixmap(QPlatformPixmap::PixmapType));
+    std::unique_ptr<QPlatformPixmap> data(QGuiApplicationPrivate::platformIntegration()->createPlatformPixmap(QPlatformPixmap::PixmapType));
     data->resize(size.width(), size.height());
-    return QPixmap(data.take());
+    return QPixmap(data.release());
 }
 
 /*!
@@ -1026,12 +993,12 @@ QPixmap QPaintEngine::createPixmapFromImage(QImage image, Qt::ImageConversionFla
         return QPixmap();
     }
 
-    QScopedPointer<QPlatformPixmap> data(QGuiApplicationPrivate::platformIntegration()->createPlatformPixmap(QPlatformPixmap::PixmapType));
+    std::unique_ptr<QPlatformPixmap> data(QGuiApplicationPrivate::platformIntegration()->createPlatformPixmap(QPlatformPixmap::PixmapType));
     if (image.isDetached())
         data->fromImageInPlace(image, flags);
     else
         data->fromImage(image, flags);
-    return QPixmap(data.take());
+    return QPixmap(data.release());
 }
 
 QPaintEnginePrivate::~QPaintEnginePrivate()
