@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qfontsubset_p.h"
 #include <qdebug.h>
@@ -401,7 +402,7 @@ static QTtfTable generateHead(const qttf_head_table &head)
 {
     const int head_size = 54;
     QTtfTable t;
-    t.tag = MAKE_TAG('h', 'e', 'a', 'd');
+    t.tag = QFont::Tag("head").value();
     t.data.resize(head_size);
 
     QTtfStream s(t.data);
@@ -472,7 +473,7 @@ static QTtfTable generateHhea(const qttf_hhea_table &hhea)
 {
     const int hhea_size = 36;
     QTtfTable t;
-    t.tag = MAKE_TAG('h', 'h', 'e', 'a');
+    t.tag = QFont::Tag("hhea").value();
     t.data.resize(hhea_size);
 
     QTtfStream s(t.data);
@@ -523,7 +524,7 @@ static QTtfTable generateMaxp(const qttf_maxp_table &maxp)
 {
     const int maxp_size = 32;
     QTtfTable t;
-    t.tag = MAKE_TAG('m', 'a', 'x', 'p');
+    t.tag = QFont::Tag("maxp").value();
     t.data.resize(maxp_size);
 
     QTtfStream s(t.data);
@@ -603,7 +604,7 @@ static QTtfTable generateName(const QList<QTtfNameRecord> &name)
     const int char_size = 2;
 
     QTtfTable t;
-    t.tag = MAKE_TAG('n', 'a', 'm', 'e');
+    t.tag = QFont::Tag("name").value();
 
     const int name_size = 6 + 12*name.size();
     int string_size = 0;
@@ -958,15 +959,15 @@ static QList<QTtfTable> generateGlyphTables(qttf_font_tables &tables, const QLis
     tables.hhea.numberOfHMetrics = nGlyphs;
 
     QTtfTable glyf;
-    glyf.tag = MAKE_TAG('g', 'l', 'y', 'f');
+    glyf.tag = QFont::Tag("glyf").value();
 
     QTtfTable loca;
-    loca.tag = MAKE_TAG('l', 'o', 'c', 'a');
+    loca.tag = QFont::Tag("loca").value();
     loca.data.resize(glyf_size < max_size_small ? (nGlyphs+1)*sizeof(quint16) : (nGlyphs+1)*sizeof(quint32));
     QTtfStream ls(loca.data);
 
     QTtfTable hmtx;
-    hmtx.tag = MAKE_TAG('h', 'm', 't', 'x');
+    hmtx.tag = QFont::Tag("hmtx").value();
     hmtx.data.resize(nGlyphs*4);
     QTtfStream hs(hmtx.data);
 
@@ -1066,7 +1067,7 @@ static QByteArray bindFont(const QList<QTtfTable>& _tables)
         for (int i = 0; i < tables.size(); ++i) {
             const QTtfTable &t = tables.at(i);
             const quint32 size = (t.data.size() + 3) & ~3;
-            if (t.tag == MAKE_TAG('h', 'e', 'a', 'd'))
+            if (t.tag == QFont::Tag("head").value())
                 head_offset = table_offset;
             f << t.tag
               << checksum(t.data)
@@ -1186,7 +1187,7 @@ QByteArray QFontSubset::toTruetype() const
     tables.append(generateMaxp(font.maxp));
     // name
     QTtfTable name_table;
-    name_table.tag = MAKE_TAG('n', 'a', 'm', 'e');
+    name_table.tag = QFont::Tag("name").value();
     if (!noEmbed)
         name_table.data = fontEngine->getSfntTable(name_table.tag);
     if (name_table.data.isEmpty()) {
@@ -1195,7 +1196,7 @@ QByteArray QFontSubset::toTruetype() const
             name.copyright = "Fake font"_L1;
         else
             name.copyright = QLatin1StringView(properties.copyright);
-        name.family = fontEngine->fontDef.families.first();
+        name.family = fontEngine->fontDef.families.constFirst();
         name.subfamily = "Regular"_L1; // ######
         name.postscript_name = QLatin1StringView(properties.postscriptName);
         name_table = generateName(name);
@@ -1204,7 +1205,7 @@ QByteArray QFontSubset::toTruetype() const
 
     if (!noEmbed) {
         QTtfTable os2;
-        os2.tag = MAKE_TAG('O', 'S', '/', '2');
+        os2.tag = QFont::Tag("OS/2").value();
         os2.data = fontEngine->getSfntTable(os2.tag);
         if (!os2.data.isEmpty())
             tables.append(os2);
