@@ -1070,8 +1070,8 @@ QRectF QGridLayoutEngine::cellRect(const QRectF &contentsGeometry, int row, int
 
     ensureGeometries(contentsGeometry.size(), styleInfo);
 
-    int lastColumn = qMax(column + columnSpan, columnCount()) - 1;
-    int lastRow = qMax(row + rowSpan, rowCount()) - 1;
+    int lastColumn = qMin(column + columnSpan, columnCount()) - 1;
+    int lastRow = qMin(row + rowSpan, rowCount()) - 1;
 
     qreal x = q_xx[column];
     qreal y = q_yy[row];
