@@ -4,11 +4,15 @@
 
 #include "qandroidplatformintegration.h"
 
+#if QT_CONFIG(accessibility)
 #include "androidjniaccessibility.h"
+#endif
 #include "androidjnimain.h"
 #include "qabstracteventdispatcher.h"
 #include "qandroideventdispatcher.h"
+#if QT_CONFIG(accessibility)
 #include "qandroidplatformaccessibility.h"
+#endif
 #include "qandroidplatformclipboard.h"
 #include "qandroidplatformfontdatabase.h"
 #include "qandroidplatformforeignwindow.h"
@@ -46,16 +50,11 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-Q_CONSTINIT QSize QAndroidPlatformIntegration::m_defaultScreenSize = QSize(320, 455);
-Q_CONSTINIT QRect QAndroidPlatformIntegration::m_defaultAvailableGeometry = QRect(0, 0, 320, 455);
-Q_CONSTINIT QSize QAndroidPlatformIntegration::m_defaultPhysicalSize = QSize(50, 71);
-
 Qt::ScreenOrientation QAndroidPlatformIntegration::m_orientation = Qt::PrimaryOrientation;
 Qt::ScreenOrientation QAndroidPlatformIntegration::m_nativeOrientation = Qt::PrimaryOrientation;
 
 bool QAndroidPlatformIntegration::m_showPasswordEnabled = false;
 
-Q_DECLARE_JNI_CLASS(QtNative, "org/qtproject/qt/android/QtNative")
 Q_DECLARE_JNI_CLASS(QtDisplayManager, "org/qtproject/qt/android/QtDisplayManager")
 Q_DECLARE_JNI_CLASS(Display, "android/view/Display")
 
@@ -217,6 +216,7 @@ QAndroidPlatformIntegration::QAndroidPlatformIntegration(const QStringList &para
     m_mainThread = QThread::currentThread();
 
     m_androidFDB = new QAndroidPlatformFontDatabase();
+    m_androidPlatformServices.reset(new QAndroidPlatformServices);
 
 #ifndef QT_NO_CLIPBOARD
     m_androidPlatformClipboard = new QAndroidPlatformClipboard();
@@ -453,9 +453,6 @@ QPlatformNativeInterface *QAndroidPlatformIntegration::nativeInterface() const
 
 QPlatformServices *QAndroidPlatformIntegration::services() const
 {
-    if (m_androidPlatformServices.isNull())
-        m_androidPlatformServices.reset(new QAndroidPlatformServices);
-
     return m_androidPlatformServices.data();
 }
 
@@ -495,17 +492,6 @@ QPlatformTheme *QAndroidPlatformIntegration::createPlatformTheme(const QString &
     return 0;
 }
 
-void QAndroidPlatformIntegration::setDefaultDisplayMetrics(int availableLeft, int availableTop,
-                                                           int availableWidth, int availableHeight,
-                                                           int physicalWidth, int physicalHeight,
-                                                           int screenWidth, int screenHeight)
-{
-    m_defaultAvailableGeometry = QRect(availableLeft, availableTop,
-                                       availableWidth, availableHeight);
-    m_defaultPhysicalSize = QSize(physicalWidth, physicalHeight);
-    m_defaultScreenSize = QSize(screenWidth, screenHeight);
-}
-
 void QAndroidPlatformIntegration::setScreenOrientation(Qt::ScreenOrientation currentOrientation,
                                                        Qt::ScreenOrientation nativeOrientation)
 {
@@ -515,10 +501,8 @@ void QAndroidPlatformIntegration::setScreenOrientation(Qt::ScreenOrientation cur
 
 void QAndroidPlatformIntegration::flushPendingUpdates()
 {
-    if (m_primaryScreen) {
-        m_primaryScreen->setSizeParameters(m_defaultPhysicalSize, m_defaultScreenSize,
-                                           m_defaultAvailableGeometry);
-    }
+    if (m_primaryScreen)
+        m_primaryScreen->setAvailableGeometry(m_primaryScreen->availableGeometry());
 }
 
 #if QT_CONFIG(accessibility)
@@ -528,6 +512,16 @@ QPlatformAccessibility *QAndroidPlatformIntegration::accessibility() const
 }
 #endif
 
+extern "C" JNIEXPORT bool JNICALL
+Java_org_qtproject_qt_android_QtNativeAccessibility_accessibilitySupported(JNIEnv *, jobject)
+{
+    #if QT_CONFIG(accessibility)
+        return true;
+    #endif // QT_CONFIG(accessibility)
+
+    return false;
+}
+
 void QAndroidPlatformIntegration::setAvailableGeometry(const QRect &availableGeometry)
 {
     if (m_primaryScreen)
@@ -558,17 +552,6 @@ void QAndroidPlatformIntegration::updateColorScheme(Qt::ColorScheme colorScheme)
                     [] () { QAndroidPlatformTheme::instance()->updateColorScheme();});
 }
 
-void QAndroidPlatformIntegration::setScreenSizeParameters(const QSize &physicalSize,
-                                                          const QSize &screenSize,
-                                                          const QRect &availableGeometry)
-{
-    if (m_primaryScreen) {
-        QMetaObject::invokeMethod(m_primaryScreen, "setSizeParameters", Qt::AutoConnection,
-                                  Q_ARG(QSize, physicalSize), Q_ARG(QSize, screenSize),
-                                  Q_ARG(QRect, availableGeometry));
-    }
-}
-
 void QAndroidPlatformIntegration::setRefreshRate(qreal refreshRate)
 {
     if (m_primaryScreen)
@@ -599,10 +582,17 @@ void QAndroidPlatformIntegration::handleScreenChanged(int displayId)
     if (it == m_screens.end() || it->second == nullptr) {
         handleScreenAdded(displayId);
     }
-    // We do not do anything more here as handling of change of
-    // rotation and refresh rate is done in QtActivityDelegate java class
-    // which calls QAndroidPlatformIntegration::setOrientation, and
-    // QAndroidPlatformIntegration::setRefreshRate accordingly.
+
+    if (QAndroidPlatformScreen *screen = it->second) {
+        QSize size = QAndroidPlatformScreen::sizeForDisplayId(displayId);
+        if (screen->geometry().size() != size) {
+            screen->setPhysicalSizeFromPixels(size);
+            screen->setSize(size);
+        }
+    }
+
+    // We do not do handle changes in rotation, refresh rate and density
+    // as they are done under QtDisplayManager.
 }
 
 void QAndroidPlatformIntegration::handleScreenRemoved(int displayId)
