@@ -33,11 +33,16 @@
 #include <QtCore/private/qcore_mac_p.h>
 #include <QtGui/private/qcoregraphics_p.h>
 #include <QtGui/private/qmacmimeregistry_p.h>
-#include <QtGui/private/qopenglcontext_p.h>
+#ifndef QT_NO_OPENGL
+#  include <QtGui/private/qopenglcontext_p.h>
+#endif
 #include <QtGui/private/qrhibackingstore_p.h>
 #include <QtGui/private/qfontengine_coretext_p.h>
 
 #include <IOKit/graphics/IOGraphicsLib.h>
+#include <UniformTypeIdentifiers/UTCoreTypes.h>
+
+#include <inttypes.h>
 
 static void initResources()
 {
@@ -71,6 +76,8 @@ static void logVersionInformation()
             qtDeploymentTarget.majorVersion(), qtDeploymentTarget.minorVersion(), qtDeploymentTarget.microVersion(),
             appBuildSDK.majorVersion(), appBuildSDK.minorVersion(), appBuildSDK.microVersion(),
             appDeploymentTarget.majorVersion(), appDeploymentTarget.minorVersion(), appDeploymentTarget.microVersion());
+
+    qCInfo(lcQpa) << "Running with Liquid Glass enabled:" << qt_apple_runningWithLiquidGlass();
 }
 
 
@@ -104,7 +111,6 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
 #endif
     , mCocoaDrag(new QCocoaDrag)
     , mNativeInterface(new QCocoaNativeInterface)
-    , mServices(new QCocoaServices)
     , mKeyboardMapper(new QAppleKeyMapper)
 {
     logVersionInformation();
@@ -120,9 +126,9 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
 #endif
         mFontDb.reset(new QCoreTextFontDatabaseEngineFactory<QCoreTextFontEngine>);
 
-    QString icStr = QPlatformInputContextFactory::requested();
-    icStr.isNull() ? mInputContext.reset(new QCocoaInputContext)
-                   : mInputContext.reset(QPlatformInputContextFactory::create(icStr));
+    auto icStrs = QPlatformInputContextFactory::requested();
+    icStrs.isEmpty() ? mInputContext.reset(new QCocoaInputContext)
+                     : mInputContext.reset(QPlatformInputContextFactory::create(icStrs));
 
     initResources();
     QMacAutoReleasePool pool;
@@ -137,16 +143,6 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
         // wants to be foreground applications so change the process type. (But
         // see the function implementation for exceptions.)
         qt_mac_transformProccessToForegroundApplication();
-
-        // Move the application window to front to make it take focus, also when launching
-        // from the terminal. On 10.12+ this call has been moved to applicationDidFinishLauching
-        // to work around issues with loss of focus at startup.
-        if (QOperatingSystemVersion::current() < QOperatingSystemVersion::MacOSSierra) {
-            // Ignoring other apps is necessary (we must ignore the terminal), but makes
-            // Qt apps play slightly less nice with other apps when lanching from Finder
-            // (See the activateIgnoringOtherApps docs.)
-            [cocoaApplication activateIgnoringOtherApps : YES];
-        }
     }
 
     // Qt 4 also does not set the application delegate, so that behavior
@@ -173,6 +169,12 @@ QCocoaIntegration::QCocoaIntegration(const QStringList &paramList)
 
     connect(qGuiApp, &QGuiApplication::focusWindowChanged,
         this, &QCocoaIntegration::focusWindowChanged);
+
+    // Opening of a native menu should close all popup windows
+    m_menuTrackingObserver = QMacNotificationObserver(nil,
+        NSMenuDidBeginTrackingNotification, ^{
+            QGuiApplicationPrivate::instance()->closeAllPopups();
+        });
 }
 
 QCocoaIntegration::~QCocoaIntegration()
@@ -190,6 +192,9 @@ QCocoaIntegration::~QCocoaIntegration()
         [[NSApplication sharedApplication] setDelegate:nil];
     }
 
+    // Stop global mouse event and app activation monitoring
+    QCocoaWindow::removePopupMonitor();
+
 #ifndef QT_NO_CLIPBOARD
     // Delete the clipboard integration and destroy mime type converters.
     // Deleting the clipboard integration flushes promised pastes using
@@ -238,6 +243,7 @@ bool QCocoaIntegration::hasCapability(QPlatformIntegration::Capability cap) cons
     case RasterGLSurface:
     case ApplicationState:
     case ApplicationIcon:
+    case BackingStoreStaticContents:
         return true;
     default:
         return QPlatformIntegration::hasCapability(cap);
@@ -305,6 +311,18 @@ QPlatformBackingStore *QCocoaIntegration::createPlatformBackingStore(QWindow *wi
         return new QCALayerBackingStore(window);
     case QSurface::MetalSurface:
     case QSurface::OpenGLSurface:
+    case QSurface::VulkanSurface:
+        // If the window is a widget window, we know that the QWidgetRepaintManager
+        // will explicitly use rhiFlush() for the window owning the backingstore,
+        // and any child window with the same surface format. This means we can
+        // safely return a QCALayerBackingStore here, to ensure that any plain
+        // flush() for child windows that don't have a matching surface format
+        // will still work, by setting the layer's contents property.
+        if (window->inherits("QWidgetWindow"))
+            return new QCALayerBackingStore(window);
+
+        // Otherwise we return a QRhiBackingStore, that implements flush() in
+        // terms of rhiFlush().
         return new QRhiBackingStore(window);
     default:
         return nullptr;
@@ -377,6 +395,9 @@ QPlatformTheme *QCocoaIntegration::createPlatformTheme(const QString &name) cons
 
 QCocoaServices *QCocoaIntegration::services() const
 {
+    if (mServices.isNull())
+        mServices.reset(new QCocoaServices);
+
     return mServices.data();
 }
 
@@ -395,14 +416,9 @@ QVariant QCocoaIntegration::styleHint(StyleHint hint) const
     return QPlatformIntegration::styleHint(hint);
 }
 
-Qt::KeyboardModifiers QCocoaIntegration::queryKeyboardModifiers() const
+QPlatformKeyMapper *QCocoaIntegration::keyMapper() const
 {
-    return QAppleKeyMapper::queryKeyboardModifiers();
-}
-
-QList<int> QCocoaIntegration::possibleKeys(const QKeyEvent *event) const
-{
-    return mKeyboardMapper->possibleKeys(event);
+    return mKeyboardMapper.data();
 }
 
 void QCocoaIntegration::setApplicationIcon(const QIcon &icon) const
@@ -435,8 +451,8 @@ void QCocoaIntegration::focusWindowChanged(QWindow *focusWindow)
         return;
 
     static bool hasDefaultApplicationIcon = [](){
-        NSImage *genericApplicationIcon = [[NSWorkspace sharedWorkspace]
-            iconForFileType:NSFileTypeForHFSTypeCode(kGenericApplicationIcon)];
+        NSImage *genericApplicationIcon = [NSWorkspace.sharedWorkspace
+            iconForContentType:UTTypeApplicationBundle];
         NSImage *applicationIcon = [NSImage imageNamed:NSImageNameApplicationIcon];
 
         NSRect rect = NSMakeRect(0, 0, 32, 32);
