@@ -13,8 +13,7 @@
 
 @property (readonly) QAccessible::Id axid;
 
-- (instancetype)initWithId:(QAccessible::Id)anId withAccessibilityContainer:(id)view;
-+ (instancetype)elementWithId:(QAccessible::Id)anId withAccessibilityContainer:(id)view;
++ (instancetype)elementWithId:(QAccessible::Id)anId;
 
 @end
 
