@@ -74,7 +74,6 @@ QOffscreenIntegration::QOffscreenIntegration(const QStringList& paramList)
 #if QT_CONFIG(draganddrop)
     m_drag.reset(new QOffscreenDrag);
 #endif
-    m_services.reset(new QPlatformServices);
 
     QJsonObject config = resolveConfigFileConfiguration(paramList).value_or(defaultConfiguration());
     setConfiguration(config);
@@ -82,8 +81,8 @@ QOffscreenIntegration::QOffscreenIntegration(const QStringList& paramList)
 
 QOffscreenIntegration::~QOffscreenIntegration()
 {
-    for (auto screen : std::as_const(m_screens))
-        QWindowSystemInterface::handleScreenRemoved(screen);
+    while (!m_screens.isEmpty())
+        QWindowSystemInterface::handleScreenRemoved(m_screens.takeLast());
 }
 
 /*
@@ -417,6 +416,9 @@ QPlatformDrag *QOffscreenIntegration::drag() const
 
 QPlatformServices *QOffscreenIntegration::services() const
 {
+    if (m_services.isNull())
+        m_services.reset(new QPlatformServices);
+
     return m_services.data();
 }
 
