@@ -8,6 +8,7 @@
 #include "qqnxabstractcover.h"
 
 #include <QtCore/QScopedPointer>
+#include <QtCore/QLoggingCategory>
 
 #if !defined(QT_NO_OPENGL)
 #include <EGL/egl.h>
@@ -17,6 +18,8 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(lcQpaWindow);
+
 // all surfaces double buffered
 #define MAX_BUFFER_COUNT    2
 
@@ -72,6 +75,8 @@ public:
     void windowPosted();
     void handleActivationEvent();
 
+    void setWindowTitle(const QString &title);
+
 protected:
     virtual int pixelFormat() const = 0;
     virtual void resetBuffers() = 0;
@@ -92,6 +97,7 @@ private:
     void setFocus(screen_window_t newFocusWindow);
     bool showWithoutActivating() const;
     bool focusable() const;
+    void notifyManager(const QString &msg);
 
     void addContextPermission();
     void removeContextPermission();
@@ -116,6 +122,13 @@ private:
 
     bool m_isTopLevel;
     bool m_firstActivateHandled;
+    int m_desktopNotify;
+
+    enum {
+        DesktopNotifyTitle = 0x1,
+        DesktopNotifyPosition = 0x2,
+        DesktopNotifyVisible = 0x2
+    };
 };
 
 QT_END_NAMESPACE
