@@ -126,6 +126,8 @@ public:
     static QWindowsBaseWindow *baseWindowOf(const QWindow *w);
     static HWND handleOf(const QWindow *w);
 
+    bool windowEvent(QEvent *event) override;
+
 protected:
     HWND parentHwnd() const { return GetAncestor(handle(), GA_PARENT); }
     bool isTopLevel_sys() const;
@@ -293,6 +295,7 @@ public:
     void handleMoved();
     void handleResized(int wParam, LPARAM lParam);
     void handleHidden();
+    void handleCompositionSettingsChanged();
     void handleDpiScaledSize(WPARAM wParam, LPARAM lParam, LRESULT *result);
     void handleDpiChanged(HWND hwnd, WPARAM wParam, LPARAM lParam);
     void handleDpiChangedAfterParent(HWND hwnd);
@@ -304,7 +307,6 @@ public:
     static QWindow *topLevelOf(QWindow *w);
     static inline void *userDataOf(HWND hwnd);
     static inline void setUserDataOf(HWND hwnd, void *ud);
-    static bool isWindowArranged(HWND hwnd);
 
     static bool hasNoNativeFrame(HWND hwnd, Qt::WindowFlags flags);
     static bool setWindowLayered(HWND hwnd, Qt::WindowFlags flags, bool hasAlpha, qreal opacity);
@@ -314,6 +316,7 @@ public:
     void releaseDC();
     void getSizeHints(MINMAXINFO *mmi) const;
     bool handleNonClientHitTest(const QPoint &globalPos, LRESULT *result) const;
+    bool handleNonClientActivate(LRESULT *result) const;
     void updateCustomTitlebar();
 
 #ifndef QT_NO_CURSOR
@@ -338,7 +341,7 @@ public:
     void alertWindow(int durationMs = 0);
     void stopAlertWindow();
 
-    enum ScreenChangeMode { FromGeometryChange, FromDpiChange };
+    enum ScreenChangeMode { FromGeometryChange, FromDpiChange, FromScreenAdded };
     void checkForScreenChanged(ScreenChangeMode mode = FromGeometryChange);
 
     void registerTouchWindow();
@@ -355,12 +358,10 @@ public:
     int savedDpi() const { return m_savedDpi; }
     qreal dpiRelativeScale(const UINT dpi) const;
 
-    bool isFrameless() const { return m_data.flags.testFlag(Qt::FramelessWindowHint); }
+    bool isClientAreaExpanded() const { return m_data.flags.testFlag(Qt::ExpandedClientAreaHint); }
 
     void requestUpdate() override;
 
-    void transitionAnimatedCustomTitleBar();
-
 private:
     inline void show_sys() const;
     inline QWindowsWindowData setWindowFlags_sys(Qt::WindowFlags wt, unsigned flags = 0) const;
