@@ -18,15 +18,17 @@
 #include "qwindowsuiagriditemprovider.h"
 #include "qwindowsuiawindowprovider.h"
 #include "qwindowsuiaexpandcollapseprovider.h"
-#include "qwindowscombase.h"
 #include "qwindowscontext.h"
 #include "qwindowsuiautils.h"
 #include "qwindowsuiaprovidercache.h"
 
 #include <QtCore/qloggingcategory.h>
+#include <QtGui/private/qaccessiblebridgeutils_p.h>
 #include <QtGui/qaccessible.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qwindow.h>
+#include <qpa/qplatforminputcontextfactory_p.h>
+#include <QtCore/private/qcomvariant_p.h>
 
 #if !defined(Q_CC_BOR) && !defined (Q_CC_GNU)
 #include <comdef.h>
@@ -38,32 +40,25 @@ QT_BEGIN_NAMESPACE
 
 using namespace QWindowsUiAutomation;
 
-QMutex QWindowsUiaMainProvider::m_mutex;
-
 // Returns a cached instance of the provider for a specific accessible interface.
-QWindowsUiaMainProvider *QWindowsUiaMainProvider::providerForAccessible(QAccessibleInterface *accessible)
+ComPtr<QWindowsUiaMainProvider> QWindowsUiaMainProvider::providerForAccessible(QAccessibleInterface *accessible)
 {
-    QMutexLocker locker(&m_mutex);
-
     if (!accessible)
         return nullptr;
 
     QAccessible::Id id = QAccessible::uniqueId(accessible);
     QWindowsUiaProviderCache *providerCache = QWindowsUiaProviderCache::instance();
-    auto *provider = qobject_cast<QWindowsUiaMainProvider *>(providerCache->providerForId(id));
+    ComPtr<QWindowsUiaMainProvider> provider = providerCache->providerForId(id);
 
-    if (provider) {
-        provider->AddRef();
-    } else {
-        provider = new QWindowsUiaMainProvider(accessible);
-        providerCache->insert(id, provider);
+    if (!provider) {
+        provider = makeComObject<QWindowsUiaMainProvider>(accessible);
+        providerCache->insert(id, provider.Get()); // Cache holds weak references
     }
     return provider;
 }
 
-QWindowsUiaMainProvider::QWindowsUiaMainProvider(QAccessibleInterface *a, int initialRefCount)
-    : QWindowsUiaBaseProvider(QAccessible::uniqueId(a)),
-      m_ref(initialRefCount)
+QWindowsUiaMainProvider::QWindowsUiaMainProvider(QAccessibleInterface *a)
+    : QWindowsUiaBaseProvider(QAccessible::uniqueId(a))
 {
 }
 
@@ -79,8 +74,8 @@ void QWindowsUiaMainProvider::notifyFocusChange(QAccessibleEvent *event)
             if (QAccessibleInterface *child = accessible->focusChild())
                 accessible = child;
         }
-        if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible))
-            QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_AutomationFocusChangedEventId);
+        if (auto provider = providerForAccessible(accessible))
+            UiaRaiseAutomationEvent(provider.Get(), UIA_AutomationFocusChangedEventId);
     }
 }
 
@@ -90,29 +85,32 @@ void QWindowsUiaMainProvider::notifyStateChange(QAccessibleStateChangeEvent *eve
         if (event->changedStates().checked || event->changedStates().checkStateMixed) {
            // Notifies states changes in checkboxes.
            if (accessible->role() == QAccessible::CheckBox) {
-                if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
-                    VARIANT oldVal, newVal;
-                    clearVariant(&oldVal);
-                    int toggleState = ToggleState_Off;
+                if (auto provider = providerForAccessible(accessible)) {
+                    long toggleState = ToggleState_Off;
                     if (accessible->state().checked)
                         toggleState = accessible->state().checkStateMixed ? ToggleState_Indeterminate : ToggleState_On;
-                    setVariantI4(toggleState, &newVal);
-                    QWindowsUiaWrapper::instance()->raiseAutomationPropertyChangedEvent(provider, UIA_ToggleToggleStatePropertyId, oldVal, newVal);
+
+                    QComVariant oldVal;
+                    QComVariant newVal{toggleState};
+                    UiaRaiseAutomationPropertyChangedEvent(
+                            provider.Get(), UIA_ToggleToggleStatePropertyId, oldVal.get(), newVal.get());
                 }
             }
         }
         if (event->changedStates().active) {
             if (accessible->role() == QAccessible::Window) {
                 // Notifies window opened/closed.
-                if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
+                if (auto provider = providerForAccessible(accessible)) {
                     if (accessible->state().active) {
-                        QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_Window_WindowOpenedEventId);
+                        UiaRaiseAutomationEvent(provider.Get(), UIA_Window_WindowOpenedEventId);
                         if (QAccessibleInterface *focused = accessible->focusChild()) {
-                            if (QWindowsUiaMainProvider *focusedProvider = providerForAccessible(focused))
-                                QWindowsUiaWrapper::instance()->raiseAutomationEvent(focusedProvider, UIA_AutomationFocusChangedEventId);
+                            if (auto focusedProvider = providerForAccessible(focused)) {
+                                UiaRaiseAutomationEvent(focusedProvider.Get(),
+                                                        UIA_AutomationFocusChangedEventId);
+                            }
                         }
                     } else {
-                        QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_Window_WindowClosedEventId);
+                        UiaRaiseAutomationEvent(provider.Get(), UIA_Window_WindowClosedEventId);
                     }
                 }
             }
@@ -140,20 +138,20 @@ void QWindowsUiaMainProvider::notifyValueChange(QAccessibleValueChangeEvent *eve
             }
         }
         if (event->value().typeId() == QMetaType::QString) {
-            if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
+            if (auto provider = providerForAccessible(accessible)) {
                 // Notifies changes in string values.
-                VARIANT oldVal, newVal;
-                clearVariant(&oldVal);
-                setVariantString(event->value().toString(), &newVal);
-                QWindowsUiaWrapper::instance()->raiseAutomationPropertyChangedEvent(provider, UIA_ValueValuePropertyId, oldVal, newVal);
+                const QComVariant oldVal;
+                const QComVariant newVal{ event->value().toString() };
+                UiaRaiseAutomationPropertyChangedEvent(provider.Get(), UIA_ValueValuePropertyId,
+                                                       oldVal.get(), newVal.get());
             }
         } else if (QAccessibleValueInterface *valueInterface = accessible->valueInterface()) {
-            if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
+            if (auto provider = providerForAccessible(accessible)) {
                 // Notifies changes in values of controls supporting the value interface.
-                VARIANT oldVal, newVal;
-                clearVariant(&oldVal);
-                setVariantDouble(valueInterface->currentValue().toDouble(), &newVal);
-                QWindowsUiaWrapper::instance()->raiseAutomationPropertyChangedEvent(provider, UIA_RangeValueValuePropertyId, oldVal, newVal);
+                const QComVariant oldVal;
+                const QComVariant newVal{ valueInterface->currentValue().toDouble() };
+                UiaRaiseAutomationPropertyChangedEvent(
+                        provider.Get(), UIA_RangeValueValuePropertyId, oldVal.get(), newVal.get());
             }
         }
     }
@@ -162,15 +160,14 @@ void QWindowsUiaMainProvider::notifyValueChange(QAccessibleValueChangeEvent *eve
 void QWindowsUiaMainProvider::notifyNameChange(QAccessibleEvent *event)
 {
     if (QAccessibleInterface *accessible = event->accessibleInterface()) {
-        // Restrict notification to combo boxes, which need it for accessibility,
-        // in order to avoid slowdowns with unnecessary notifications.
-        if (accessible->role() == QAccessible::ComboBox) {
-            if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
-                VARIANT oldVal, newVal;
-                clearVariant(&oldVal);
-                setVariantString(accessible->text(QAccessible::Name), &newVal);
-                QWindowsUiaWrapper::instance()->raiseAutomationPropertyChangedEvent(provider, UIA_NamePropertyId, oldVal, newVal);
-                ::SysFreeString(newVal.bstrVal);
+        // Restrict notification to combo boxes and the currently focused element, which
+        // need it for accessibility, in order to avoid slowdowns with unnecessary notifications.
+        if (accessible->role() == QAccessible::ComboBox || accessible->state().focused) {
+            if (auto provider = providerForAccessible(accessible)) {
+                QComVariant oldVal;
+                QComVariant newVal{ accessible->text(QAccessible::Name) };
+                UiaRaiseAutomationPropertyChangedEvent(provider.Get(), UIA_NamePropertyId,
+                                                       oldVal.get(), newVal.get());
             }
         }
     }
@@ -179,9 +176,8 @@ void QWindowsUiaMainProvider::notifyNameChange(QAccessibleEvent *event)
 void QWindowsUiaMainProvider::notifySelectionChange(QAccessibleEvent *event)
 {
     if (QAccessibleInterface *accessible = event->accessibleInterface()) {
-        if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
-            QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_SelectionItem_ElementSelectedEventId);
-        }
+        if (auto provider = providerForAccessible(accessible))
+            UiaRaiseAutomationEvent(provider.Get(), UIA_SelectionItem_ElementSelectedEventId);
     }
 }
 
@@ -190,50 +186,69 @@ void QWindowsUiaMainProvider::notifyTextChange(QAccessibleEvent *event)
 {
     if (QAccessibleInterface *accessible = event->accessibleInterface()) {
         if (accessible->textInterface()) {
-            if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
+            if (auto provider = providerForAccessible(accessible)) {
                 if (event->type() == QAccessible::TextSelectionChanged) {
-                    QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_Text_TextSelectionChangedEventId);
+                    UiaRaiseAutomationEvent(provider.Get(), UIA_Text_TextSelectionChangedEventId);
                 } else if (event->type() == QAccessible::TextCaretMoved) {
                     if (!accessible->state().readOnly) {
-                        QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_Text_TextSelectionChangedEventId);
+                        UiaRaiseAutomationEvent(provider.Get(),
+                                                UIA_Text_TextSelectionChangedEventId);
                     }
                 } else {
-                    QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_Text_TextChangedEventId);
+                    UiaRaiseAutomationEvent(provider.Get(), UIA_Text_TextChangedEventId);
                 }
             }
         }
     }
 }
 
+void QWindowsUiaMainProvider::raiseNotification(QAccessibleAnnouncementEvent *event)
+{
+    if (QAccessibleInterface *accessible = event->accessibleInterface()) {
+        if (auto provider = providerForAccessible(accessible)) {
+            QBStr message{ event->message() };
+            QAccessible::AnnouncementPoliteness prio = event->politeness();
+            NotificationProcessing processing = (prio == QAccessible::AnnouncementPoliteness::Assertive)
+                    ? NotificationProcessing_ImportantAll
+                    : NotificationProcessing_All;
+            QBStr activityId{ QString::fromLatin1("") };
+#if !defined(Q_CC_MSVC) || !defined(QT_WIN_SERVER_2016_COMPAT)
+            UiaRaiseNotificationEvent(provider.Get(), NotificationKind_Other, processing, message.bstr(),
+                                      activityId.bstr());
+#else
+            HMODULE uiautomationcore = GetModuleHandleW(L"UIAutomationCore.dll");
+            if (uiautomationcore != NULL) {
+                typedef HRESULT (WINAPI *EVENTFUNC)(IRawElementProviderSimple *, NotificationKind,
+                                                    NotificationProcessing, BSTR, BSTR);
+
+                EVENTFUNC uiaRaiseNotificationEvent =
+                    (EVENTFUNC)GetProcAddress(uiautomationcore, "UiaRaiseNotificationEvent");
+                if (uiaRaiseNotificationEvent != NULL) {
+                    uiaRaiseNotificationEvent(provider.Get(), NotificationKind_Other, processing,
+                                              message.bstr(), activityId.bstr());
+                }
+            }
+#endif
+        }
+    }
+}
+
 HRESULT STDMETHODCALLTYPE QWindowsUiaMainProvider::QueryInterface(REFIID iid, LPVOID *iface)
 {
-    if (!iface)
-        return E_INVALIDARG;
-    *iface = nullptr;
+    HRESULT result = QComObject::QueryInterface(iid, iface);
 
-    QAccessibleInterface *accessible = accessibleInterface();
-
-    const bool result = qWindowsComQueryUnknownInterfaceMulti<IRawElementProviderSimple>(this, iid, iface)
-        || qWindowsComQueryInterface<IRawElementProviderSimple>(this, iid, iface)
-        || qWindowsComQueryInterface<IRawElementProviderFragment>(this, iid, iface)
-        || (accessible && hwndForAccessible(accessible) && qWindowsComQueryInterface<IRawElementProviderFragmentRoot>(this, iid, iface));
-    return result ? S_OK : E_NOINTERFACE;
-}
-
-ULONG QWindowsUiaMainProvider::AddRef()
-{
-    return ++m_ref;
-}
-
-ULONG STDMETHODCALLTYPE QWindowsUiaMainProvider::Release()
-{
-    QMutexLocker locker(&m_mutex);
-
-    if (!--m_ref) {
-        delete this;
-        return 0;
+    if (SUCCEEDED(result) && iid == __uuidof(IRawElementProviderFragmentRoot)) {
+        QAccessibleInterface *accessible = accessibleInterface();
+        if (accessible && hwndForAccessible(accessible)) {
+            result = S_OK;
+        } else {
+            Release();
+            result = E_NOINTERFACE;
+            *iface = nullptr;
+        }
     }
-    return m_ref;
+
+    return result;
 }
 
 HRESULT QWindowsUiaMainProvider::get_ProviderOptions(ProviderOptions *pRetVal)
@@ -261,79 +276,82 @@ HRESULT QWindowsUiaMainProvider::GetPatternProvider(PATTERNID idPattern, IUnknow
     switch (idPattern) {
     case UIA_WindowPatternId:
         if (accessible->parent() && (accessible->parent()->role() == QAccessible::Application)) {
-            *pRetVal = new QWindowsUiaWindowProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaWindowProvider>(id()).Detach();
         }
         break;
     case UIA_TextPatternId:
     case UIA_TextPattern2Id:
         // All text controls.
         if (accessible->textInterface()) {
-            *pRetVal = new QWindowsUiaTextProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaTextProvider>(id()).Detach();
         }
         break;
     case UIA_ValuePatternId:
         // All non-static controls support the Value pattern.
         if (accessible->role() != QAccessible::StaticText)
-            *pRetVal = new QWindowsUiaValueProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaValueProvider>(id()).Detach();
         break;
     case UIA_RangeValuePatternId:
         // Controls providing a numeric value within a range (e.g., sliders, scroll bars, dials).
         if (accessible->valueInterface()) {
-            *pRetVal = new QWindowsUiaRangeValueProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaRangeValueProvider>(id()).Detach();
         }
         break;
     case UIA_TogglePatternId:
         // Checkboxes and other checkable controls.
         if (accessible->state().checkable)
-            *pRetVal = new QWindowsUiaToggleProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaToggleProvider>(id()).Detach();
         break;
     case UIA_SelectionPatternId:
-        // Lists of items.
-        if (accessible->role() == QAccessible::List
+    case UIA_SelectionPattern2Id:
+        // Selections via QAccessibleSelectionInterface or lists of items.
+        if (accessible->selectionInterface()
+                || accessible->role() == QAccessible::List
                 || accessible->role() == QAccessible::PageTabList) {
-            *pRetVal = new QWindowsUiaSelectionProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaSelectionProvider>(id()).Detach();
         }
         break;
     case UIA_SelectionItemPatternId:
-        // Items within a list and radio buttons.
-        if ((accessible->role() == QAccessible::RadioButton)
+        // Parent supports selection interface or items within a list and radio buttons.
+        if ((accessible->parent() && accessible->parent()->selectionInterface())
+                || (accessible->role() == QAccessible::RadioButton)
                 || (accessible->role() == QAccessible::ListItem)
                 || (accessible->role() == QAccessible::PageTab)) {
-            *pRetVal = new QWindowsUiaSelectionItemProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaSelectionItemProvider>(id()).Detach();
         }
         break;
     case UIA_TablePatternId:
         // Table/tree.
         if (accessible->tableInterface()
                 && ((accessible->role() == QAccessible::Table) || (accessible->role() == QAccessible::Tree))) {
-            *pRetVal = new QWindowsUiaTableProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaTableProvider>(id()).Detach();
         }
         break;
     case UIA_TableItemPatternId:
         // Item within a table/tree.
         if (accessible->tableCellInterface()
                 && ((accessible->role() == QAccessible::Cell) || (accessible->role() == QAccessible::TreeItem))) {
-            *pRetVal = new QWindowsUiaTableItemProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaTableItemProvider>(id()).Detach();
         }
         break;
     case UIA_GridPatternId:
         // Table/tree.
         if (accessible->tableInterface()
                 && ((accessible->role() == QAccessible::Table) || (accessible->role() == QAccessible::Tree))) {
-            *pRetVal = new QWindowsUiaGridProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaGridProvider>(id()).Detach();
         }
         break;
     case UIA_GridItemPatternId:
         // Item within a table/tree.
         if (accessible->tableCellInterface()
                 && ((accessible->role() == QAccessible::Cell) || (accessible->role() == QAccessible::TreeItem))) {
-            *pRetVal = new QWindowsUiaGridItemProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaGridItemProvider>(id()).Detach();
         }
         break;
     case UIA_InvokePatternId:
         // Things that have an invokable action (e.g., simple buttons).
         if (accessible->actionInterface()) {
-            *pRetVal = new QWindowsUiaInvokeProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaInvokeProvider>(id()).Detach();
         }
         break;
     case UIA_ExpandCollapsePatternId:
@@ -343,7 +361,7 @@ HRESULT QWindowsUiaMainProvider::GetPatternProvider(PATTERNID idPattern, IUnknow
                 && accessible->child(0)->role() == QAccessible::PopupMenu)
             || accessible->role() == QAccessible::ComboBox
             || (accessible->role() == QAccessible::TreeItem && accessible->state().expandable)) {
-            *pRetVal = new QWindowsUiaExpandCollapseProvider(id());
+            *pRetVal = makeComObject<QWindowsUiaExpandCollapseProvider>(id()).Detach();
         }
         break;
     default:
@@ -353,6 +371,133 @@ HRESULT QWindowsUiaMainProvider::GetPatternProvider(PATTERNID idPattern, IUnknow
     return S_OK;
 }
 
+void QWindowsUiaMainProvider::setLabelledBy(QAccessibleInterface *accessible, VARIANT *pRetVal)
+{
+    Q_ASSERT(accessible);
+
+    typedef std::pair<QAccessibleInterface*, QAccessible::Relation> RelationPair;
+    const QList<RelationPair> relationInterfaces = accessible->relations(QAccessible::Label);
+    if (relationInterfaces.empty())
+        return;
+
+    // UIA_LabeledByPropertyId only supports one relation
+    ComPtr<IRawElementProviderSimple> provider = providerForAccessible(relationInterfaces.first().first);
+    if (!provider)
+        return;
+
+    pRetVal->vt = VT_UNKNOWN;
+    pRetVal->punkVal = provider.Detach();
+}
+
+void QWindowsUiaMainProvider::fillVariantArrayForRelation(QAccessibleInterface* accessible,
+                                                          QAccessible::Relation relation, VARIANT *pRetVal)
+{
+    Q_ASSERT(accessible);
+
+    typedef std::pair<QAccessibleInterface*, QAccessible::Relation> RelationPair;
+    const QList<RelationPair> relationInterfaces = accessible->relations(relation);
+    if (relationInterfaces.empty())
+        return;
+
+    SAFEARRAY *elements = SafeArrayCreateVector(VT_UNKNOWN, 0, relationInterfaces.size());
+    for (LONG i = 0; i < relationInterfaces.size(); ++i) {
+        if (ComPtr<IRawElementProviderSimple> provider =
+                    providerForAccessible(relationInterfaces.at(i).first)) {
+            SafeArrayPutElement(elements, &i, provider.Get());
+        }
+    }
+
+    pRetVal->vt = VT_UNKNOWN | VT_ARRAY;
+    pRetVal->parray = elements;
+}
+
+void QWindowsUiaMainProvider::setAriaProperties(QAccessibleInterface *accessible, VARIANT *pRetVal)
+{
+    Q_ASSERT(accessible);
+
+    QAccessibleAttributesInterface *attributesIface = accessible->attributesInterface();
+    if (!attributesIface)
+        return;
+
+    QString ariaString;
+    const QList<QAccessible::Attribute> attrKeys = attributesIface->attributeKeys();
+    for (qsizetype i = 0; i < attrKeys.size(); ++i) {
+        if (i != 0)
+            ariaString += QStringLiteral(";");
+        const QAccessible::Attribute key = attrKeys.at(i);
+        const QVariant value = attributesIface->attributeValue(key);
+        // see "Core Accessibility API Mappings" spec: https://www.w3.org/TR/core-aam-1.2/
+        switch (key) {
+        case QAccessible::Attribute::Custom:
+        {
+            // forward custom attributes as-is
+            Q_ASSERT((value.canConvert<QHash<QString, QString>>()));
+            const QHash<QString, QString> attrMap = value.value<QHash<QString, QString>>();
+            for (auto [name, val] : attrMap.asKeyValueRange()) {
+                if (name != *attrMap.keyBegin())
+                    ariaString += QStringLiteral(";");
+                ariaString += name + QStringLiteral("=") + val;
+            }
+            break;
+        }
+        case QAccessible::Attribute::Level:
+            Q_ASSERT(value.canConvert<int>());
+            ariaString += QStringLiteral("level=") + QString::number(value.toInt());
+            break;
+        default:
+            break;
+        }
+    }
+
+    *pRetVal = QComVariant{ ariaString }.release();
+}
+
+void QWindowsUiaMainProvider::setStyle(QAccessibleInterface *accessible, VARIANT *pRetVal)
+{
+    Q_ASSERT(accessible);
+
+    QAccessibleAttributesInterface *attributesIface = accessible->attributesInterface();
+    if (!attributesIface)
+        return;
+
+    // currently, only heading styles are implemented here
+    if (accessible->role() != QAccessible::Role::Heading)
+        return;
+
+    const QVariant levelVariant = attributesIface->attributeValue(QAccessible::Attribute::Level);
+    if (!levelVariant.isValid())
+        return;
+
+    Q_ASSERT(levelVariant.canConvert<int>());
+    // UIA only has styles for heading levels 1-9
+    const int level = levelVariant.toInt();
+    if (level < 1 || level > 9)
+        return;
+
+    const long styleId = styleIdForHeadingLevel(level);
+    *pRetVal = QComVariant{ styleId }.release();
+}
+
+int QWindowsUiaMainProvider::styleIdForHeadingLevel(int headingLevel)
+{
+    // only heading levels 1-9 have a corresponding UIA style ID
+    Q_ASSERT(headingLevel > 0 && headingLevel <= 9);
+
+    static constexpr int styles[] = {
+        StyleId_Heading1,
+        StyleId_Heading2,
+        StyleId_Heading3,
+        StyleId_Heading4,
+        StyleId_Heading5,
+        StyleId_Heading6,
+        StyleId_Heading7,
+        StyleId_Heading8,
+        StyleId_Heading9,
+    };
+
+    return styles[headingLevel - 1];
+}
+
 HRESULT QWindowsUiaMainProvider::GetPropertyValue(PROPERTYID idProp, VARIANT *pRetVal)
 {
     qCDebug(lcQpaUiAutomation) << __FUNCTION__ << idProp;
@@ -370,37 +515,70 @@ HRESULT QWindowsUiaMainProvider::GetPropertyValue(PROPERTYID idProp, VARIANT *pR
     switch (idProp) {
     case UIA_ProcessIdPropertyId:
         // PID
-        setVariantI4(int(GetCurrentProcessId()), pRetVal);
+        *pRetVal = QComVariant{ static_cast<long>(GetCurrentProcessId()) }.release();
         break;
     case UIA_AccessKeyPropertyId:
         // Accelerator key.
-        setVariantString(accessible->text(QAccessible::Accelerator), pRetVal);
+        *pRetVal = QComVariant{ accessible->text(QAccessible::Accelerator) }.release();
+        break;
+    case UIA_AriaRolePropertyId:
+        if (accessible->role() == QAccessible::Heading)
+            *pRetVal = QComVariant{ QStringLiteral("heading") }.release();
+        break;
+    case UIA_AriaPropertiesPropertyId:
+        setAriaProperties(accessible, pRetVal);
         break;
     case UIA_AutomationIdPropertyId:
         // Automation ID, which can be used by tools to select a specific control in the UI.
-        setVariantString(automationIdForAccessible(accessible), pRetVal);
+        *pRetVal = QComVariant{ QAccessibleBridgeUtils::accessibleId(accessible) }.release();
         break;
     case UIA_ClassNamePropertyId:
         // Class name.
         if (QObject *o = accessible->object()) {
             QString className = QLatin1StringView(o->metaObject()->className());
-            setVariantString(className, pRetVal);
+            *pRetVal = QComVariant{ className }.release();
         }
         break;
+    case UIA_CulturePropertyId:
+    {
+        QLocale locale;
+        if (QAccessibleAttributesInterface *attributesIface = accessible->attributesInterface()) {
+            const QVariant localeVariant = attributesIface->attributeValue(QAccessible::Attribute::Locale);
+            if (localeVariant.isValid()) {
+                Q_ASSERT(localeVariant.canConvert<QLocale>());
+                locale = localeVariant.toLocale();
+            }
+        }
+        LCID lcid = LocaleNameToLCID(qUtf16Printable(locale.bcp47Name()), 0);
+        *pRetVal = QComVariant{ long(lcid) }.release();
+        break;
+    }
+    case UIA_DescribedByPropertyId:
+        fillVariantArrayForRelation(accessible, QAccessible::DescriptionFor, pRetVal);
+        break;
+    case UIA_FlowsFromPropertyId:
+        fillVariantArrayForRelation(accessible, QAccessible::FlowsTo, pRetVal);
+        break;
+    case UIA_FlowsToPropertyId:
+        fillVariantArrayForRelation(accessible, QAccessible::FlowsFrom, pRetVal);
+        break;
+    case UIA_LabeledByPropertyId:
+        setLabelledBy(accessible, pRetVal);
+        break;
     case UIA_FrameworkIdPropertyId:
-        setVariantString(QStringLiteral("Qt"), pRetVal);
+        *pRetVal = QComVariant{ QStringLiteral("Qt") }.release();
         break;
     case UIA_ControlTypePropertyId:
         if (topLevelWindow) {
             // Reports a top-level widget as a window, instead of "custom".
-            setVariantI4(UIA_WindowControlTypeId, pRetVal);
+            *pRetVal = QComVariant{ UIA_WindowControlTypeId }.release();
         } else {
             // Control type converted from role.
             auto controlType = roleToControlTypeId(accessible->role());
 
             // The native OSK should be disabled if the Qt OSK is in use,
             // or if disabled via application attribute.
-            static bool imModuleEmpty = qEnvironmentVariableIsEmpty("QT_IM_MODULE");
+            static bool imModuleEmpty = QPlatformInputContextFactory::requested().isEmpty();
             bool nativeVKDisabled = QCoreApplication::testAttribute(Qt::AA_DisableNativeVirtualKeyboard);
 
             // If we want to disable the native OSK auto-showing
@@ -408,90 +586,83 @@ HRESULT QWindowsUiaMainProvider::GetPropertyValue(PROPERTYID idProp, VARIANT *pR
             if (controlType == UIA_EditControlTypeId && (!imModuleEmpty || nativeVKDisabled))
                 controlType = UIA_TextControlTypeId;
 
-            setVariantI4(controlType, pRetVal);
+            *pRetVal = QComVariant{ controlType }.release();
         }
         break;
     case UIA_HelpTextPropertyId:
-        setVariantString(accessible->text(QAccessible::Help), pRetVal);
+        *pRetVal = QComVariant{ accessible->text(QAccessible::Help) }.release();
         break;
     case UIA_HasKeyboardFocusPropertyId:
         if (topLevelWindow) {
             // Windows set the active state to true when they are focused
-            setVariantBool(accessible->state().active, pRetVal);
+            *pRetVal = QComVariant{ accessible->state().active ? true : false }.release();
         } else {
-            setVariantBool(accessible->state().focused, pRetVal);
+            *pRetVal = QComVariant{ accessible->state().focused ? true : false }.release();
         }
         break;
     case UIA_IsKeyboardFocusablePropertyId:
         if (topLevelWindow) {
             // Windows should always be focusable
-            setVariantBool(true, pRetVal);
+            *pRetVal = QComVariant{ true }.release();
         } else {
-            setVariantBool(accessible->state().focusable, pRetVal);
+            *pRetVal = QComVariant{ accessible->state().focusable ? true : false }.release();
         }
         break;
     case UIA_IsOffscreenPropertyId:
-        setVariantBool(accessible->state().offscreen, pRetVal);
+        *pRetVal = QComVariant{ accessible->state().offscreen ? true : false }.release();
         break;
     case UIA_IsContentElementPropertyId:
-        setVariantBool(true, pRetVal);
+        *pRetVal = QComVariant{ true }.release();
         break;
     case UIA_IsControlElementPropertyId:
-        setVariantBool(true, pRetVal);
+        *pRetVal = QComVariant{ true }.release();
         break;
     case UIA_IsEnabledPropertyId:
-        setVariantBool(!accessible->state().disabled, pRetVal);
+        *pRetVal = QComVariant{ !accessible->state().disabled }.release();
         break;
     case UIA_IsPasswordPropertyId:
-        setVariantBool(accessible->role() == QAccessible::EditableText
-                       && accessible->state().passwordEdit, pRetVal);
+        *pRetVal = QComVariant{ accessible->role() == QAccessible::EditableText
+                                && accessible->state().passwordEdit }
+                           .release();
         break;
     case UIA_IsPeripheralPropertyId:
         // True for peripheral UIs.
         if (QWindow *window = windowForAccessible(accessible)) {
             const Qt::WindowType wt = window->type();
-            setVariantBool(wt == Qt::Popup || wt == Qt::ToolTip || wt == Qt::SplashScreen, pRetVal);
+            *pRetVal = QComVariant{ wt == Qt::Popup || wt == Qt::ToolTip || wt == Qt::SplashScreen }
+                               .release();
         }
         break;
     case UIA_IsDialogPropertyId:
-        setVariantBool(accessible->role() == QAccessible::Dialog
-                       || accessible->role() == QAccessible::AlertMessage, pRetVal);
+        *pRetVal = QComVariant{ accessible->role() == QAccessible::Dialog
+                                || accessible->role() == QAccessible::AlertMessage }
+                           .release();
         break;
     case UIA_FullDescriptionPropertyId:
-        setVariantString(accessible->text(QAccessible::Description), pRetVal);
+        *pRetVal = QComVariant{ accessible->text(QAccessible::Description) }.release();
+        break;
+    case UIA_LocalizedControlTypePropertyId:
+        // see Core Accessibility API Mappings spec:
+        // https://www.w3.org/TR/core-aam-1.2/#role-map-blockquote
+        if (accessible->role() == QAccessible::BlockQuote)
+            *pRetVal = QComVariant{ tr("blockquote") }.release();
         break;
     case UIA_NamePropertyId: {
         QString name = accessible->text(QAccessible::Name);
         if (name.isEmpty() && topLevelWindow)
            name = QCoreApplication::applicationName();
-        setVariantString(name, pRetVal);
+        *pRetVal = QComVariant{ name }.release();
         break;
     }
+    case UIA_StyleIdAttributeId:
+        setStyle(accessible, pRetVal);
+        break;
     default:
         break;
     }
     return S_OK;
 }
 
-// Generates an ID based on the name of the controls and their parents.
-QString QWindowsUiaMainProvider::automationIdForAccessible(const QAccessibleInterface *accessible)
-{
-    QString result;
-    if (accessible) {
-        QObject *obj = accessible->object();
-        while (obj) {
-            QString name = obj->objectName();
-            if (name.isEmpty())
-                return result;
-            if (!result.isEmpty())
-                result.prepend(u'.');
-            result.prepend(name);
-            obj = obj->parent();
-        }
-    }
-    return result;
-}
-
 HRESULT QWindowsUiaMainProvider::get_HostRawElementProvider(IRawElementProviderSimple **pRetVal)
 {
     qCDebug(lcQpaUiAutomation) << __FUNCTION__ << this;
@@ -503,7 +674,7 @@ HRESULT QWindowsUiaMainProvider::get_HostRawElementProvider(IRawElementProviderS
     // Returns a host provider only for controls associated with a native window handle. Others should return NULL.
     if (QAccessibleInterface *accessible = accessibleInterface()) {
         if (HWND hwnd = hwndForAccessible(accessible)) {
-            return QWindowsUiaWrapper::instance()->hostProviderFromHwnd(hwnd, pRetVal);
+            return UiaHostProviderFromHwnd(hwnd, pRetVal);
         }
     }
     return S_OK;
@@ -574,7 +745,7 @@ HRESULT QWindowsUiaMainProvider::Navigate(NavigateDirection direction, IRawEleme
     }
 
     if (targetacc)
-        *pRetVal = providerForAccessible(targetacc);
+        *pRetVal = providerForAccessible(targetacc).Detach();
     return S_OK;
 }
 
@@ -662,9 +833,8 @@ HRESULT QWindowsUiaMainProvider::get_FragmentRoot(IRawElementProviderFragmentRoo
     // non-native controls/fragments.
     if (QAccessibleInterface *accessible = accessibleInterface()) {
         if (QWindow *window = windowForAccessible(accessible)) {
-            if (QAccessibleInterface *rootacc = window->accessibleRoot()) {
-                *pRetVal = providerForAccessible(rootacc);
-            }
+            if (QAccessibleInterface *rootacc = window->accessibleRoot())
+                *pRetVal = providerForAccessible(rootacc).Detach();
         }
     }
     return S_OK;
@@ -704,7 +874,7 @@ HRESULT QWindowsUiaMainProvider::ElementProviderFromPoint(double x, double y, IR
             if (targetacc->textInterface()) break;
             acc = acc->childAt(point.x(), point.y());
         }
-        *pRetVal = providerForAccessible(targetacc);
+        *pRetVal = providerForAccessible(targetacc).Detach();
     }
     return S_OK;
 }
@@ -720,7 +890,7 @@ HRESULT QWindowsUiaMainProvider::GetFocus(IRawElementProviderFragment **pRetVal)
 
     if (QAccessibleInterface *accessible = accessibleInterface()) {
         if (QAccessibleInterface *focusacc = accessible->focusChild()) {
-            *pRetVal = providerForAccessible(focusacc);
+            *pRetVal = providerForAccessible(focusacc).Detach();
         }
     }
     return S_OK;
