@@ -25,7 +25,7 @@
 #include <xcb/xcb.h>
 
 #include <QtGui/private/qgenericunixfontdatabase_p.h>
-#include <QtGui/private/qgenericunixservices_p.h>
+#include <QtGui/private/qdesktopunixservices_p.h>
 
 #include <stdio.h>
 
@@ -43,7 +43,10 @@
 #endif
 
 #include <qpa/qplatforminputcontextfactory_p.h>
-#include <private/qgenericunixthemes_p.h>
+#include <private/qgenericunixtheme_p.h>
+#if QT_CONFIG(dbus)
+#include <private/qkdetheme_p.h>
+#endif
 #include <qpa/qplatforminputcontext.h>
 
 #include <QtGui/QOpenGLContext>
@@ -71,7 +74,7 @@ using namespace Qt::StringLiterals;
 // or, for older Linuxes, read out 'cmdline'.
 static bool runningUnderDebugger()
 {
-#if defined(QT_DEBUG) && defined(Q_OS_LINUX)
+#if defined(Q_OS_LINUX)
     const QString parentProc = "/proc/"_L1 + QString::number(getppid());
     const QFileInfo parentProcExe(parentProc + "/exe"_L1);
     if (parentProcExe.isSymLink())
@@ -93,10 +96,19 @@ static bool runningUnderDebugger()
 #endif
 }
 
+class QXcbUnixServices : public QDesktopUnixServices, public QXcbObject
+{
+public:
+    QString portalWindowIdentifier(QWindow *window) override;
+    void registerDBusMenuForWindow(QWindow *window, const QString &service, const QString &path) override;
+    void unregisterDBusMenuForWindow(QWindow *window) override;
+};
+
+
 QXcbIntegration *QXcbIntegration::m_instance = nullptr;
 
 QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char **argv)
-    : m_services(new QGenericUnixServices)
+    : m_services(new QXcbUnixServices)
     , m_instanceName(nullptr)
     , m_canGrab(true)
     , m_defaultVisualId(UINT_MAX)
@@ -148,12 +160,10 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
         doGrabArg = false;
     }
 
-#if defined(QT_DEBUG)
     if (!noGrabArg && !doGrabArg && underDebugger) {
         qCDebug(lcQpaXcb, "Qt: gdb: -nograb added to command-line options.\n"
                 "\t Use the -dograb option to enforce grabbing.");
     }
-#endif
     m_canGrab = (!underDebugger && !noGrabArg) || (underDebugger && doGrabArg);
 
     static bool canNotGrabEnv = qEnvironmentVariableIsSet("QT_XCB_NO_GRAB_SERVER");
@@ -166,6 +176,7 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
         m_connection = nullptr;
         return;
     }
+    m_services->setConnection(m_connection);
 
     m_fontDatabase.reset(new QGenericUnixFontDatabase());
 
@@ -197,7 +208,7 @@ QPlatformPixmap *QXcbIntegration::createPlatformPixmap(QPlatformPixmap::PixelTyp
 QPlatformWindow *QXcbIntegration::createPlatformWindow(QWindow *window) const
 {
     QXcbGlIntegration *glIntegration = nullptr;
-    const bool isTrayIconWindow = QXcbWindow::isTrayIconWindow(window);;
+    const bool isTrayIconWindow = QXcbWindow::isTrayIconWindow(window);
     if (window->type() != Qt::Desktop && !isTrayIconWindow) {
         if (window->supportsOpenGL()) {
             glIntegration = connection()->glIntegration();
@@ -337,11 +348,12 @@ void QXcbIntegration::initialize()
     const auto defaultInputContext = "compose"_L1;
     // Perform everything that may potentially need the event dispatcher (timers, socket
     // notifiers) here instead of the constructor.
-    QString icStr = QPlatformInputContextFactory::requested();
-    if (icStr.isNull())
-        icStr = defaultInputContext;
-    m_inputContext.reset(QPlatformInputContextFactory::create(icStr));
-    if (!m_inputContext && icStr != defaultInputContext && icStr != "none"_L1)
+    auto icStrs = QPlatformInputContextFactory::requested();
+    if (icStrs.isEmpty())
+        icStrs = { defaultInputContext };
+    m_inputContext.reset(QPlatformInputContextFactory::create(icStrs));
+    if (!m_inputContext && !icStrs.contains(defaultInputContext)
+        && icStrs != QStringList{"none"_L1})
         m_inputContext.reset(QPlatformInputContextFactory::create(defaultInputContext));
 
     connection()->keyboard()->initialize();
@@ -422,14 +434,9 @@ QPlatformServices *QXcbIntegration::services() const
     return m_services.data();
 }
 
-Qt::KeyboardModifiers QXcbIntegration::queryKeyboardModifiers() const
+QPlatformKeyMapper *QXcbIntegration::keyMapper() const
 {
-    return m_connection->queryKeyboardModifiers();
-}
-
-QList<int> QXcbIntegration::possibleKeys(const QKeyEvent *e) const
-{
-    return m_connection->keyboard()->possibleKeys(e);
+    return m_connection->keyboard();
 }
 
 QStringList QXcbIntegration::themeNames() const
@@ -472,21 +479,23 @@ QVariant QXcbIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
     case QPlatformIntegration::StartDragVelocity:
     case QPlatformIntegration::UseRtlExtensions:
     case QPlatformIntegration::PasswordMaskCharacter:
+    case QPlatformIntegration::FlickMaximumVelocity:
+    case QPlatformIntegration::FlickDeceleration:
         // TODO using various xcb, gnome or KDE settings
         break; // Not implemented, use defaults
+    case QPlatformIntegration::FlickStartDistance:
     case QPlatformIntegration::StartDragDistance: {
         RETURN_VALID_XSETTINGS(xsNetDndDragThreshold);
-
         // The default (in QPlatformTheme::defaultThemeHint) is 10 pixels, but
         // on a high-resolution screen it makes sense to increase it.
-        qreal dpi = 100.0;
+        qreal dpi = 100;
         if (const QXcbScreen *screen = connection()->primaryScreen()) {
             if (screen->logicalDpi().first > dpi)
                 dpi = screen->logicalDpi().first;
             if (screen->logicalDpi().second > dpi)
                 dpi = screen->logicalDpi().second;
         }
-        return 10.0 * dpi / 100.0;
+        return (hint == QPlatformIntegration::FlickStartDistance ? qreal(15) : qreal(10)) * dpi / qreal(100);
     }
     case QPlatformIntegration::ShowIsFullScreen:
         // X11 always has support for windows, but the
@@ -523,7 +532,7 @@ QByteArray QXcbIntegration::wmClass() const
         if (m_instanceName)
             name = QString::fromLocal8Bit(m_instanceName);
         if (name.isEmpty() && qEnvironmentVariableIsSet(resourceNameVar))
-            name = QString::fromLocal8Bit(qgetenv(resourceNameVar));
+            name = qEnvironmentVariable(resourceNameVar);
         if (name.isEmpty())
             name = argv0BaseName();
 
@@ -585,4 +594,41 @@ QPlatformVulkanInstance *QXcbIntegration::createPlatformVulkanInstance(QVulkanIn
 }
 #endif
 
+void QXcbIntegration::setApplicationBadge(qint64 number)
+{
+    auto unixServices = dynamic_cast<QDesktopUnixServices *>(services());
+    unixServices->setApplicationBadge(number);
+}
+
+QString QXcbUnixServices::portalWindowIdentifier(QWindow *window)
+{
+    return "x11:"_L1 + QString::number(window->winId(), 16);
+}
+
+void QXcbUnixServices::registerDBusMenuForWindow(QWindow *window, const QString &service, const QString &path)
+{
+    const QByteArray serviceValue = service.toLatin1();
+    const QByteArray pathValue = path.toLatin1();
+
+    xcb_change_property(xcb_connection(),
+                        XCB_PROP_MODE_REPLACE, window->winId(),
+                        atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_SERVICE_NAME),
+                        XCB_ATOM_STRING, 8,
+                        serviceValue.length(),
+                        serviceValue.constData());
+
+    xcb_change_property(xcb_connection(),
+                        XCB_PROP_MODE_REPLACE, window->winId(),
+                        atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_OBJECT_PATH),
+                        XCB_ATOM_STRING, 8,
+                        pathValue.length(),
+                        pathValue.constData());
+}
+
+void QXcbUnixServices::unregisterDBusMenuForWindow(QWindow *window)
+{
+    xcb_delete_property(xcb_connection(), window->winId(), atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_SERVICE_NAME));
+    xcb_delete_property(xcb_connection(), window->winId(), atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_OBJECT_PATH));
+}
+
 QT_END_NAMESPACE
