@@ -1,12 +1,11 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
-// Qt-Security score:significant reason:default
 
 #ifndef QGTK3THEME_H
 #define QGTK3THEME_H
 
 #include <private/qtguiglobal_p.h>
-#include <private/qgenericunixthemes_p.h>
+#include <private/qgnometheme_p.h>
 #include "qgtk3storage_p.h"
 
 QT_BEGIN_NAMESPACE
@@ -32,8 +31,11 @@ public:
                            QPlatformTheme::IconOptions iconOptions = { }) const override;
 
     static const char *name;
+
 private:
-    Qt::ColorScheme m_requestedColorScheme = Qt::ColorScheme::Unknown;
+#if QT_CONFIG(dbus)
+    void updateColorScheme(Qt::ColorScheme) override;
+#endif // QT_CONFIG(dbus)
     static bool useNativeFileDialog();
     std::unique_ptr<QGtk3Storage> m_storage;
 };
