@@ -20,7 +20,7 @@
 
 QT_BEGIN_NAMESPACE
 
-static Q_LOGGING_CATEGORY(lcSqlQuery, "qt.sql.qsqlquery")
+Q_STATIC_LOGGING_CATEGORY(lcSqlQuery, "qt.sql.qsqlquery")
 
 class QSqlQueryPrivate
 {
@@ -216,11 +216,12 @@ QSqlQuery::~QSqlQuery()
         delete d;
 }
 
-#if QT_DEPRECATED_SINCE(6, 2)
+#if QT_REMOVAL_QT7_DEPRECATED_SINCE(6, 2)
 /*!
     Constructs a copy of \a other.
 
-    \deprecated QSqlQuery cannot be meaningfully copied. Prepared
+    \deprecated [6.2] QSqlQuery cannot be meaningfully copied, and
+    therefore will no longer be copiable in Qt 7. Prepared
     statements, bound values and so on will not work correctly, depending
     on your database driver (for instance, changing the copy will affect
     the original). Treat QSqlQuery as a move-only type instead.
@@ -235,7 +236,8 @@ QSqlQuery::QSqlQuery(const QSqlQuery& other)
 /*!
     Assigns \a other to this object.
 
-    \deprecated QSqlQuery cannot be meaningfully copied. Prepared
+    \deprecated [6.2] QSqlQuery cannot be meaningfully copied, and
+    therefore will no longer be copiable in Qt 7. Prepared
     statements, bound values and so on will not work correctly, depending
     on your database driver (for instance, changing the copy will affect
     the original). Treat QSqlQuery as a move-only type instead.
@@ -263,8 +265,7 @@ QSqlQuery& QSqlQuery::operator=(const QSqlQuery& other)
 /*!
     \fn void QSqlQuery::swap(QSqlQuery &other) noexcept
     \since 6.2
-    Swaps \a other to this object. This operation is very
-    fast and never fails.
+    \memberswap{query}
 */
 
 /*!
@@ -275,7 +276,7 @@ static void qInit(QSqlQuery *q, const QString& query, const QSqlDatabase &db)
     QSqlDatabase database = db;
     if (!database.isValid()) {
         database =
-                QSqlDatabase::database(QLatin1StringView(QSqlDatabase::defaultConnection), false);
+                QSqlDatabase::database(QSqlDatabase::defaultConnectionName(), false);
     }
     if (database.isValid())
         *q = QSqlQuery(database.driver()->createResult());
