@@ -749,7 +749,12 @@ void QDialog::closeEvent(QCloseEvent *e)
 
 void QDialog::setVisible(bool visible)
 {
-    QWidget::setVisible(visible);
+    Q_D(QDialog);
+
+    if (testAttribute(Qt::WA_WState_ExplicitShowHide) && testAttribute(Qt::WA_WState_Hidden) != visible)
+        return;
+
+    d->setVisible(visible);
 }
 
 void QDialogPrivate::setVisible(bool visible)
