@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qapplication.h"
 #include "qboxlayout.h"
@@ -124,6 +88,7 @@ public:
 
     void effectiveMargins(int *left, int *top, int *right, int *bottom) const;
     QLayoutItem* replaceAt(int index, QLayoutItem*) override;
+    int validateIndex(int index) const;
 };
 
 QBoxLayoutPrivate::~QBoxLayoutPrivate()
@@ -268,7 +233,7 @@ void QBoxLayoutPrivate::setupGeom()
 
     hasHfw = false;
 
-    int n = list.count();
+    int n = list.size();
     geomArray.clear();
     QList<QLayoutStruct> a(n);
 
@@ -400,7 +365,7 @@ void QBoxLayoutPrivate::setupGeom()
 void QBoxLayoutPrivate::calcHfw(int w)
 {
     QList<QLayoutStruct> &a = geomArray;
-    int n = a.count();
+    int n = a.size();
     int h = 0;
     int mh = 0;
 
@@ -443,6 +408,15 @@ QLayoutItem* QBoxLayoutPrivate::replaceAt(int index, QLayoutItem *item)
     return r;
 }
 
+int QBoxLayoutPrivate::validateIndex(int index) const
+{
+    if (index < 0)
+        return list.size(); // append
+
+    if (index > list.size())
+        qWarning("QBoxLayout::insert: index %d out of range (max: %d)", index, int(list.size()));
+    return index <= list.size() ? index : list.size();
+}
 
 /*!
     \class QBoxLayout
@@ -707,7 +681,7 @@ void QBoxLayout::invalidate()
 int QBoxLayout::count() const
 {
     Q_D(const QBoxLayout);
-    return d->list.count();
+    return d->list.size();
 }
 
 /*!
@@ -716,7 +690,7 @@ int QBoxLayout::count() const
 QLayoutItem *QBoxLayout::itemAt(int index) const
 {
     Q_D(const QBoxLayout);
-    return index >= 0 && index < d->list.count() ? d->list.at(index)->item : nullptr;
+    return index >= 0 && index < d->list.size() ? d->list.at(index)->item : nullptr;
 }
 
 /*!
@@ -725,7 +699,7 @@ QLayoutItem *QBoxLayout::itemAt(int index) const
 QLayoutItem *QBoxLayout::takeAt(int index)
 {
     Q_D(QBoxLayout);
-    if (index < 0 || index >= d->list.count())
+    if (index < 0 || index >= d->list.size())
         return nullptr;
     QBoxLayoutItem *b = d->list.takeAt(index);
     QLayoutItem *item = b->item;
@@ -776,7 +750,7 @@ void QBoxLayout::setGeometry(const QRect &r)
         QList<QLayoutStruct> a = d->geomArray;
         int pos = horz(d->dir) ? s.x() : s.y();
         int space = horz(d->dir) ? s.width() : s.height();
-        int n = a.count();
+        int n = a.size();
         if (d->hasHfw && !horz(d->dir)) {
             for (int i = 0; i < n; i++) {
                 QBoxLayoutItem *box = d->list.at(i);
@@ -838,8 +812,12 @@ void QBoxLayout::addItem(QLayoutItem *item)
 }
 
 /*!
-    Inserts \a item into this box layout at position \a index. If \a
-    index is negative, the item is added at the end.
+    Inserts \a item into this box layout at position \a index.
+    Index must be either negative or within the range 0 to count(),
+    inclusive. If \a index is negative or count(), the item is
+    added at the end.
+
+    The ownership of \a item is passed to this layout.
 
     \sa addItem(), insertWidget(), insertLayout(), insertStretch(),
         insertSpacing()
@@ -847,9 +825,7 @@ void QBoxLayout::addItem(QLayoutItem *item)
 void QBoxLayout::insertItem(int index, QLayoutItem *item)
 {
     Q_D(QBoxLayout);
-    if (index < 0)                                // append
-        index = d->list.count();
-
+    index = d->validateIndex(index);
     QBoxLayoutItem *it = new QBoxLayoutItem(item);
     d->list.insert(index, it);
     invalidate();
@@ -867,9 +843,7 @@ void QBoxLayout::insertItem(int index, QLayoutItem *item)
 void QBoxLayout::insertSpacing(int index, int size)
 {
     Q_D(QBoxLayout);
-    if (index < 0)                                // append
-        index = d->list.count();
-
+    index = d->validateIndex(index);
     QLayoutItem *b;
     if (horz(d->dir))
         b = QLayoutPrivate::createSpacerItem(this, size, 0, QSizePolicy::Fixed, QSizePolicy::Minimum);
@@ -892,9 +866,7 @@ void QBoxLayout::insertSpacing(int index, int size)
 void QBoxLayout::insertStretch(int index, int stretch)
 {
     Q_D(QBoxLayout);
-    if (index < 0)                                // append
-        index = d->list.count();
-
+    index = d->validateIndex(index);
     QLayoutItem *b;
     if (horz(d->dir))
         b = QLayoutPrivate::createSpacerItem(this, 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
@@ -914,14 +886,14 @@ void QBoxLayout::insertStretch(int index, int stretch)
     size and stretch factor. If \a index is negative the
     space is added at the end.
 
+    The ownership of \a spacerItem is passed to this layout.
+
     \sa addSpacerItem(), insertStretch(), insertSpacing()
 */
 void QBoxLayout::insertSpacerItem(int index, QSpacerItem *spacerItem)
 {
     Q_D(QBoxLayout);
-    if (index < 0)                                // append
-        index = d->list.count();
-
+    index = d->validateIndex(index);
     QBoxLayoutItem *it = new QBoxLayoutItem(spacerItem);
     it->magic = true;
     d->list.insert(index, it);
@@ -943,8 +915,7 @@ void QBoxLayout::insertLayout(int index, QLayout *layout, int stretch)
         return;
     if (!adoptLayout(layout))
         return;
-    if (index < 0)                                // append
-        index = d->list.count();
+    index = d->validateIndex(index);
     QBoxLayoutItem *it = new QBoxLayoutItem(layout, stretch);
     d->list.insert(index, it);
     invalidate();
@@ -968,6 +939,8 @@ void QBoxLayout::insertLayout(int index, QLayout *layout, int stretch)
     The alignment is specified by \a alignment. The default alignment
     is 0, which means that the widget fills the entire cell.
 
+    \a widget becomes a child of the QLayout::parentWidget().
+
     \sa addWidget(), insertItem()
 */
 void QBoxLayout::insertWidget(int index, QWidget *widget, int stretch,
@@ -977,8 +950,7 @@ void QBoxLayout::insertWidget(int index, QWidget *widget, int stretch,
     if (!d->checkWidget(widget))
          return;
     addChildWidget(widget);
-    if (index < 0)                                // append
-        index = d->list.count();
+    index = d->validateIndex(index);
     QWidgetItem *b = QLayoutPrivate::createWidgetItem(this, widget);
     b->setAlignment(alignment);
 
@@ -1015,6 +987,8 @@ void QBoxLayout::addStretch(int stretch)
 
     Adds \a spacerItem to the end of this box layout.
 
+    The ownership of \a spacerItem is passed to this layout.
+
     \sa addSpacing(), addStretch()
 */
 void QBoxLayout::addSpacerItem(QSpacerItem *spacerItem)
@@ -1039,6 +1013,8 @@ void QBoxLayout::addSpacerItem(QSpacerItem *spacerItem)
     The alignment is specified by \a alignment. The default
     alignment is 0, which means that the widget fills the entire cell.
 
+    \a widget becomes a child of the QLayout::parentWidget().
+
     \sa insertWidget(), addItem(), addLayout(), addStretch(),
         addSpacing(), addStrut()
 */
@@ -1051,6 +1027,8 @@ void QBoxLayout::addWidget(QWidget *widget, int stretch, Qt::Alignment alignment
     Adds \a layout to the end of the box, with serial stretch factor
     \a stretch.
 
+    \a layout becomes a child of the box layout.
+
     \sa insertLayout(), addItem(), addWidget()
 */
 void QBoxLayout::addLayout(QLayout *layout, int stretch)
