@@ -19,10 +19,7 @@
 
     Unlike the window managers for top-level windows, all window flags
     (Qt::WindowFlags) are supported by QMdiArea as long as the flags
-    are supported by the current widget style. If a specific flag is
-    not supported by the style (e.g., the
-    \l{Qt::}{WindowShadeButtonHint}), you can still shade the window
-    with showShaded().
+    are supported by the current widget style.
 
     Subwindows in QMdiArea are instances of QMdiSubWindow. They
     are added to an MDI area with addSubWindow(). It is common to pass
@@ -220,19 +217,7 @@ static inline QMdiArea *mdiAreaParent(QWidget *widget)
 }
 
 #if QT_CONFIG(tabwidget)
-static inline QTabBar::Shape tabBarShapeFrom(QTabWidget::TabShape shape, QTabWidget::TabPosition position)
-{
-    const bool rounded = (shape == QTabWidget::Rounded);
-    if (position == QTabWidget::North)
-        return rounded ? QTabBar::RoundedNorth : QTabBar::TriangularNorth;
-    if (position == QTabWidget::South)
-        return rounded ? QTabBar::RoundedSouth : QTabBar::TriangularSouth;
-    if (position == QTabWidget::East)
-        return rounded ? QTabBar::RoundedEast : QTabBar::TriangularEast;
-    if (position == QTabWidget::West)
-        return rounded ? QTabBar::RoundedWest : QTabBar::TriangularWest;
-    return QTabBar::RoundedNorth;
-}
+QTabBar::Shape _q_tb_tabBarShapeFrom(QTabWidget::TabShape shape, QTabWidget::TabPosition position);
 #endif // QT_CONFIG(tabwidget)
 
 static inline QString tabTextFor(QMdiSubWindow *subWindow)
@@ -434,8 +419,8 @@ QList<QRect> MinOverlapPlacer::getCandidatePlacements(const QSize &size, const Q
     ylist.erase(std::unique(ylist.begin(), ylist.end()), ylist.end());
 
     result.reserve(ylist.size() * xlist.size());
-    for (int y : qAsConst(ylist))
-        for (int x : qAsConst(xlist))
+    for (int y : std::as_const(ylist))
+        for (int x : std::as_const(xlist))
             result << QRect(QPoint(x, y), size);
     return result;
 }
@@ -662,9 +647,7 @@ QMdiAreaPrivate::QMdiAreaPrivate()
       indexToNextWindow(-1),
       indexToPreviousWindow(-1),
       indexToHighlighted(-1),
-      indexToLastActiveTab(-1),
-      resizeTimerId(-1),
-      tabToPreviousTimerId(-1)
+      indexToLastActiveTab(-1)
 {
 }
 
@@ -683,7 +666,11 @@ void QMdiAreaPrivate::_q_deactivateAllWindows(QMdiSubWindow *aboutToActivate)
         aboutToBecomeActive = aboutToActivate;
     Q_ASSERT(aboutToBecomeActive);
 
-    foreach (QMdiSubWindow *child, childWindows) {
+    // Take a copy because child->showNormal() could indirectly call
+    // QCoreApplication::sendEvent(), which could call unknown code that e.g.
+    // recurses into the class modifying childWindows.
+    const auto subWindows = childWindows;
+    for (QMdiSubWindow *child : subWindows) {
         if (!sanityCheck(child, "QMdiArea::deactivateAllWindows") || aboutToBecomeActive == child)
             continue;
         // We don't want to handle signals caused by child->showNormal().
@@ -750,7 +737,7 @@ void QMdiAreaPrivate::_q_currentTabChanged(int index)
 
     // If the previous active sub-window was hidden, disable the tab.
     if (indexToLastActiveTab >= 0 && indexToLastActiveTab < tabBar->count()
-        && indexToLastActiveTab < childWindows.count()) {
+        && indexToLastActiveTab < childWindows.size()) {
         QMdiSubWindow *lastActive = childWindows.at(indexToLastActiveTab);
         if (lastActive && lastActive->isHidden())
             tabBar->setTabEnabled(indexToLastActiveTab, false);
@@ -782,6 +769,17 @@ void QMdiAreaPrivate::_q_moveTab(int from, int to)
     Q_UNUSED(to);
 #else
     childWindows.move(from, to);
+
+    // Put the active window in front to update activation order.
+    const int indexToActiveWindow = childWindows.indexOf(active);
+    if (indexToActiveWindow != -1) {
+        const int index = indicesToActivatedChildren.indexOf(indexToActiveWindow);
+        Q_ASSERT(index != -1);
+        if (index != 0) { // if it's not in front
+            indicesToActivatedChildren.move(index, 0);
+            internalRaise(active);
+        }
+    }
 #endif // QT_CONFIG(tabbar)
 }
 
@@ -824,7 +822,7 @@ void QMdiAreaPrivate::appendChild(QMdiSubWindow *child)
     if (tabBar) {
         tabBar->addTab(child->windowIcon(), tabTextFor(child));
         updateTabBarGeometry();
-        if (childWindows.count() == 1 && !(options & QMdiArea::DontMaximizeSubWindowOnActivation))
+        if (childWindows.size() == 1 && !(options & QMdiArea::DontMaximizeSubWindowOnActivation))
             showActiveWindowMaximized = true;
     }
 #endif
@@ -858,7 +856,7 @@ void QMdiAreaPrivate::place(Placer *placer, QMdiSubWindow *child)
     QList<QRect> rects;
     rects.reserve(childWindows.size());
     QRect parentRect = q->rect();
-    foreach (QMdiSubWindow *window, childWindows) {
+    for (QMdiSubWindow *window : std::as_const(childWindows)) {
         if (!sanityCheck(window, "QMdiArea::place") || window == child || !window->isVisibleTo(q)
                 || !window->testAttribute(Qt::WA_Moved)) {
             continue;
@@ -900,7 +898,7 @@ void QMdiAreaPrivate::rearrange(Rearranger *rearranger)
     const bool reverseList = rearranger->type() == Rearranger::RegularTiler;
     const QList<QMdiSubWindow *> subWindows = subWindowList(activationOrder, reverseList);
     QSize minSubWindowSize;
-    foreach (QMdiSubWindow *child, subWindows) {
+    for (QMdiSubWindow *child : subWindows) {
         if (!sanityCheck(child, "QMdiArea::rearrange") || !child->isVisible())
             continue;
         if (rearranger->type() == Rearranger::IconTiler) {
@@ -919,7 +917,7 @@ void QMdiAreaPrivate::rearrange(Rearranger *rearranger)
 
     QRect domain = viewport->rect();
     if (rearranger->type() == Rearranger::RegularTiler && !widgets.isEmpty())
-        domain = resizeToMinimumTileSize(minSubWindowSize, widgets.count());
+        domain = resizeToMinimumTileSize(minSubWindowSize, widgets.size());
 
     rearranger->rearrange(widgets, domain);
 
@@ -990,7 +988,7 @@ void QMdiAreaPrivate::activateHighlightedWindow()
         return;
 
     Q_ASSERT(indexToHighlighted < childWindows.size());
-    if (tabToPreviousTimerId != -1)
+    if (tabToPreviousTimer.isActive())
         activateWindow(nextVisibleSubWindow(-1, QMdiArea::ActivationHistoryOrder));
     else
         activateWindow(childWindows.at(indexToHighlighted));
@@ -1083,6 +1081,13 @@ void QMdiAreaPrivate::updateActiveWindow(int removedIndex, bool activeRemoved)
 {
     Q_ASSERT(indicesToActivatedChildren.size() == childWindows.size());
 
+    // Update indices list first so that we don't rely
+    for (int i = 0; i < indicesToActivatedChildren.size(); ++i) {
+        int &index = indicesToActivatedChildren[i];
+        if (index > removedIndex)
+            --index;
+    }
+
 #if QT_CONFIG(tabbar)
     if (tabBar && removedIndex >= 0) {
         const QSignalBlocker blocker(tabBar);
@@ -1109,13 +1114,6 @@ void QMdiAreaPrivate::updateActiveWindow(int removedIndex, bool activeRemoved)
             --indexToHighlighted;
     }
 
-    // Update indices list
-    for (int i = 0; i < indicesToActivatedChildren.size(); ++i) {
-        int *index = &indicesToActivatedChildren[i];
-        if (*index > removedIndex)
-            --*index;
-    }
-
     if (!activeRemoved)
         return;
 
@@ -1172,6 +1170,8 @@ void QMdiAreaPrivate::updateScrollBars()
     const int startX = q->isLeftToRight() ? childrenRect.left() : viewportRect.right()
                                                                   - childrenRect.right();
 
+    const auto singleStep = defaultSingleStep();
+
     // Horizontal scroll bar.
     if (isSubWindowsTiled && hbar->value() != 0)
         hbar->setValue(0);
@@ -1179,7 +1179,7 @@ void QMdiAreaPrivate::updateScrollBars()
     hbar->setRange(qMin(0, xOffset),
                    qMax(0, xOffset + childrenRect.width() - viewportRect.width()));
     hbar->setPageStep(childrenRect.width());
-    hbar->setSingleStep(childrenRect.width() / 20);
+    hbar->setSingleStep(childrenRect.width() / singleStep);
 
     // Vertical scroll bar.
     if (isSubWindowsTiled && vbar->value() != 0)
@@ -1188,7 +1188,7 @@ void QMdiAreaPrivate::updateScrollBars()
     vbar->setRange(qMin(0, yOffset),
                    qMax(0, yOffset + childrenRect.height() - viewportRect.height()));
     vbar->setPageStep(childrenRect.height());
-    vbar->setSingleStep(childrenRect.height() / 20);
+    vbar->setSingleStep(childrenRect.height() / singleStep);
 }
 
 /*!
@@ -1296,7 +1296,7 @@ bool QMdiAreaPrivate::scrollBarsEnabled() const
 */
 bool QMdiAreaPrivate::lastWindowAboutToBeDestroyed() const
 {
-    if (childWindows.count() != 1)
+    if (childWindows.size() != 1)
         return false;
 
     QMdiSubWindow *last = childWindows.at(0);
@@ -1314,7 +1314,7 @@ bool QMdiAreaPrivate::lastWindowAboutToBeDestroyed() const
 */
 void QMdiAreaPrivate::setChildActivationEnabled(bool enable, bool onlyNextActivationEvent) const
 {
-    foreach (QMdiSubWindow *subWindow, childWindows) {
+    for (QMdiSubWindow *subWindow : childWindows) {
         if (!subWindow || !subWindow->isVisible())
             continue;
         if (onlyNextActivationEvent)
@@ -1336,7 +1336,11 @@ void QMdiAreaPrivate::scrollBarPolicyChanged(Qt::Orientation orientation, Qt::Sc
     const QMdiSubWindow::SubWindowOption option = orientation == Qt::Horizontal ?
         QMdiSubWindow::AllowOutsideAreaHorizontally : QMdiSubWindow::AllowOutsideAreaVertically;
     const bool enable = policy != Qt::ScrollBarAlwaysOff;
-    foreach (QMdiSubWindow *child, childWindows) {
+    // Take a copy because child->setOption() may indirectly call QCoreApplication::sendEvent(),
+    // the latter could call unknown code that could e.g. recurse into the class
+    // modifying childWindows.
+    const auto subWindows = childWindows;
+    for (QMdiSubWindow *child : subWindows) {
         if (!sanityCheck(child, "QMdiArea::scrollBarPolicyChanged"))
             continue;
         child->setOption(option, enable);
@@ -1352,7 +1356,7 @@ QMdiAreaPrivate::subWindowList(QMdiArea::WindowOrder order, bool reversed) const
         return list;
 
     if (order == QMdiArea::CreationOrder) {
-        foreach (QMdiSubWindow *child, childWindows) {
+        for (QMdiSubWindow *child : childWindows) {
             if (!child)
                 continue;
             if (!reversed)
@@ -1372,7 +1376,7 @@ QMdiAreaPrivate::subWindowList(QMdiArea::WindowOrder order, bool reversed) const
         }
     } else { // ActivationHistoryOrder
         Q_ASSERT(indicesToActivatedChildren.size() == childWindows.size());
-        for (int i = indicesToActivatedChildren.count() - 1; i >= 0; --i) {
+        for (int i = indicesToActivatedChildren.size() - 1; i >= 0; --i) {
             QMdiSubWindow *child = childWindows.at(indicesToActivatedChildren.at(i));
             if (!child)
                 continue;
@@ -1487,7 +1491,7 @@ void QMdiAreaPrivate::highlightNextSubWindow(int increaseFactor)
 
     // Only highlight if we're not switching back to the previously active window (Ctrl-Tab once).
 #if QT_CONFIG(rubberband)
-    if (tabToPreviousTimerId == -1)
+    if (!tabToPreviousTimer.isActive())
         showRubberBandFor(highlight);
 #endif
 
@@ -1533,12 +1537,17 @@ void QMdiAreaPrivate::setViewMode(QMdiArea::ViewMode mode)
         tabBar->setTabsClosable(tabsClosable);
         tabBar->setMovable(tabsMovable);
 #if QT_CONFIG(tabwidget)
-        tabBar->setShape(tabBarShapeFrom(tabShape, tabPosition));
+        tabBar->setShape(_q_tb_tabBarShapeFrom(tabShape, tabPosition));
 #endif
 
         isSubWindowsTiled = false;
 
-        foreach (QMdiSubWindow *subWindow, childWindows)
+        // Take a copy as tabBar->addTab() will (indirectly) create a connection between
+        // the tab close button clicked() signal and the _q_closeTab() slot, which may
+        // indirectly call QCoreApplication::sendEvent(), the latter could result in
+        // invoking unknown code that could e.g. recurse into the class modifying childWindows.
+        const auto subWindows = childWindows;
+        for (QMdiSubWindow *subWindow : subWindows)
             tabBar->addTab(subWindow->windowIcon(), tabTextFor(subWindow));
 
         QMdiSubWindow *current = q->currentSubWindow();
@@ -1597,7 +1606,7 @@ void QMdiAreaPrivate::updateTabBarGeometry()
 
     Q_Q(QMdiArea);
 #if QT_CONFIG(tabwidget)
-    Q_ASSERT(tabBarShapeFrom(tabShape, tabPosition) == tabBar->shape());
+    Q_ASSERT(_q_tb_tabBarShapeFrom(tabShape, tabPosition) == tabBar->shape());
 #endif
     const QSize tabBarSizeHint = tabBar->sizeHint();
 
@@ -1654,7 +1663,7 @@ void QMdiAreaPrivate::refreshTabBar()
     tabBar->setTabsClosable(tabsClosable);
     tabBar->setMovable(tabsMovable);
 #if QT_CONFIG(tabwidget)
-    tabBar->setShape(tabBarShapeFrom(tabShape, tabPosition));
+    tabBar->setShape(_q_tb_tabBarShapeFrom(tabShape, tabPosition));
 #endif
     updateTabBarGeometry();
 }
@@ -1761,7 +1770,7 @@ QMdiSubWindow *QMdiArea::currentSubWindow() const
     if (d->isActivated && !window()->isMinimized())
         return nullptr;
 
-    Q_ASSERT(d->indicesToActivatedChildren.count() > 0);
+    Q_ASSERT(d->indicesToActivatedChildren.size() > 0);
     int index = d->indicesToActivatedChildren.at(0);
     Q_ASSERT(index >= 0 && index < d->childWindows.size());
     QMdiSubWindow *current = d->childWindows.at(index);
@@ -1860,7 +1869,11 @@ void QMdiArea::closeAllSubWindows()
         return;
 
     d->isSubWindowsTiled = false;
-    foreach (QMdiSubWindow *child, d->childWindows) {
+    // Take a copy because the child->close() call below may end up indirectly calling
+    // QCoreApplication::send{Spontaneous}Event(), which may call unknown code that
+    // could e.g. recurse into the class modifying d->childWindows.
+    const auto subWindows = d->childWindows;
+    for (QMdiSubWindow *child : subWindows) {
         if (!sanityCheck(child, "QMdiArea::closeAllSubWindows"))
             continue;
         child->close();
@@ -1999,7 +2012,11 @@ void QMdiArea::removeSubWindow(QWidget *widget)
     }
 
     bool found = false;
-    foreach (QMdiSubWindow *child, d->childWindows) {
+    // Take a copy because child->setWidget(nullptr) will indirectly
+    // QCoreApplication::sendEvent(); the latter could call unknown code that could
+    // e.g. recurse into the class modifying d->childWindows.
+    const auto subWindows = d->childWindows;
+    for (QMdiSubWindow *child : subWindows) {
         if (!sanityCheck(child, "QMdiArea::removeSubWindow"))
             continue;
         if (child->widget() == widget) {
@@ -2280,7 +2297,11 @@ void QMdiArea::resizeEvent(QResizeEvent *resizeEvent)
 
     // Resize maximized views.
     bool hasMaximizedSubWindow = false;
-    foreach (QMdiSubWindow *child, d->childWindows) {
+    // Take a copy because child->resize() may call QCoreApplication::sendEvent()
+    // which may invoke unknown code, that could e.g. recurse into the class
+    // modifying d->childWindows.
+    const auto subWindows = d->childWindows;
+    for (QMdiSubWindow *child : subWindows) {
         if (sanityCheck(child, "QMdiArea::resizeEvent") && child->isMaximized()
                 && child->size() != resizeEvent->size()) {
             auto realSize = resizeEvent->size();
@@ -2313,13 +2334,11 @@ void QMdiArea::resizeEvent(QResizeEvent *resizeEvent)
 void QMdiArea::timerEvent(QTimerEvent *timerEvent)
 {
     Q_D(QMdiArea);
-    if (timerEvent->timerId() == d->resizeTimerId) {
-        killTimer(d->resizeTimerId);
-        d->resizeTimerId = -1;
+    if (timerEvent->id() == d->resizeTimer.id()) {
+        d->resizeTimer.stop();
         d->arrangeMinimizedSubWindows();
-    } else if (timerEvent->timerId() == d->tabToPreviousTimerId) {
-        killTimer(d->tabToPreviousTimerId);
-        d->tabToPreviousTimerId = -1;
+    } else if (timerEvent->id() == d->tabToPreviousTimer.id()) {
+        d->tabToPreviousTimer.stop();
         if (d->indexToHighlighted < 0)
             return;
 #if QT_CONFIG(rubberband)
@@ -2339,7 +2358,9 @@ void QMdiArea::showEvent(QShowEvent *showEvent)
     Q_D(QMdiArea);
     if (!d->pendingRearrangements.isEmpty()) {
         bool skipPlacement = false;
-        foreach (Rearranger *rearranger, d->pendingRearrangements) {
+        // Take a copy because d->rearrange() may modify d->pendingRearrangements
+        const auto pendingRearrange = d->pendingRearrangements;
+        for (Rearranger *rearranger : pendingRearrange) {
             // If this is the case, we don't have to lay out pending child windows
             // since the rearranger will find a placement for them.
             if (rearranger->type() != Rearranger::IconTiler && !skipPlacement)
@@ -2353,9 +2374,19 @@ void QMdiArea::showEvent(QShowEvent *showEvent)
     }
 
     if (!d->pendingPlacements.isEmpty()) {
-        foreach (QMdiSubWindow *window, d->pendingPlacements) {
+        // Nothing obvious in the loop body changes the container (in this code path)
+        // during iteration, this is calling into a non-const method that does change
+        // the container when called from other places. So take a copy anyway for good
+        // measure.
+        const auto copy = d->pendingPlacements;
+        for (QMdiSubWindow *window : copy) {
             if (!window)
                 continue;
+            if (d->viewMode == TabbedView && window->d_func()->isActive && !d->active) {
+                d->showActiveWindowMaximized = true;
+                d->emitWindowActivated(window); // Also maximizes the window
+                continue;
+            }
             if (!window->testAttribute(Qt::WA_Resized)) {
                 QSize newSize(window->sizeHint().boundedTo(viewport()->size()));
                 window->resize(newSize.expandedTo(qSmartMinSize(window)));
@@ -2489,12 +2520,16 @@ bool QMdiArea::event(QEvent *event)
             d->isSubWindowsTiled = true;
         }
         break;
-    case QEvent::WindowIconChange:
-        foreach (QMdiSubWindow *window, d->childWindows) {
+    case QEvent::WindowIconChange: {
+        // Take a copy because QCoreApplication::sendEvent() may call unknown code,
+        // that may cause recursing into the class
+        const auto subWindows = d->childWindows;
+        for (QMdiSubWindow *window : subWindows) {
             if (sanityCheck(window, "QMdiArea::WindowIconChange"))
                 QCoreApplication::sendEvent(window, event);
         }
         break;
+    }
     case QEvent::Hide:
         d->setActive(d->active, false, false);
         d->setChildActivationEnabled(false);
@@ -2649,7 +2684,10 @@ void QMdiArea::setupViewport(QWidget *viewport)
     Q_D(QMdiArea);
     if (viewport)
         viewport->setAttribute(Qt::WA_OpaquePaintEvent, d->background.isOpaque());
-    foreach (QMdiSubWindow *child, d->childWindows) {
+    // Take a copy because the child->setParent() call below may call QCoreApplication::sendEvent()
+    // which may call unknown code that could e.g. recurse into the class modifying d->childWindows.
+    const auto subWindows = d->childWindows;
+    for (QMdiSubWindow *child : subWindows) {
         if (!sanityCheck(child, "QMdiArea::setupViewport"))
             continue;
         child->setParent(viewport, child->windowFlags());
