@@ -1,34 +1,9 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtCore/QCoreApplication>
 #include <QtNetwork/QtNetwork>
-#include <QtTest/QtTest>
+#include <QTest>
 
 #include "../../../network-settings.h"
 
@@ -53,6 +28,12 @@ private slots:
     void readLine2_data();
     void readLine2();
 
+    void readLineInto_Checks_data();
+    void readLineInto_Checks();
+
+    void readLineInto();
+    void readLineInto_qspan();
+
     void readAllKeepPosition();
     void writeInTextMode();
     void skip_data();
@@ -70,7 +51,7 @@ private:
 
 void tst_QIODevice::initTestCase()
 {
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#ifdef Q_OS_ANDROID
     QVERIFY(QFileInfo(QStringLiteral("./tst_qiodevice.cpp")).exists()
             || QFile::copy(QStringLiteral(":/tst_qiodevice.cpp"), QStringLiteral("./tst_qiodevice.cpp")));
 #endif
@@ -104,15 +85,20 @@ void tst_QIODevice::getSetCheck()
 //----------------------------------------------------------------------------------
 void tst_QIODevice::constructing_QTcpSocket()
 {
+#ifdef QT_TEST_SERVER
+    if (!QtNetworkSettings::verifyConnection(QtNetworkSettings::imapServerName(), 143))
+        QSKIP("No network test server available");
+#else
     if (!QtNetworkSettings::verifyTestNetworkSettings())
         QSKIP("No network test server available");
+#endif
 
     QTcpSocket socket;
     QIODevice *device = &socket;
 
     QVERIFY(!device->isOpen());
 
-    socket.connectToHost(QtNetworkSettings::serverName(), 143);
+    socket.connectToHost(QtNetworkSettings::imapServerName(), 143);
     QVERIFY(socket.waitForConnected(30000));
     QVERIFY(device->isOpen());
     QCOMPARE(device->readChannelCount(), 1);
@@ -130,7 +116,7 @@ void tst_QIODevice::constructing_QTcpSocket()
     socket.close();
     QCOMPARE(socket.readChannelCount(), 0);
     QCOMPARE(socket.writeChannelCount(), 0);
-    socket.connectToHost(QtNetworkSettings::serverName(), 143);
+    socket.connectToHost(QtNetworkSettings::imapServerName(), 143);
     QVERIFY(socket.waitForConnected(30000));
     QVERIFY(device->isOpen());
 
@@ -191,16 +177,16 @@ void tst_QIODevice::constructing_QFile()
 void tst_QIODevice::read_QByteArray()
 {
     QFile f(QFINDTESTDATA("tst_qiodevice.cpp"));
-    f.open(QIODevice::ReadOnly);
+    QVERIFY(f.open(QIODevice::ReadOnly));
 
     QByteArray b = f.read(10);
-    QCOMPARE(b.length(), 10);
+    QCOMPARE(b.size(), 10);
 
     b = f.read(256);
-    QCOMPARE(b.length(), 256);
+    QCOMPARE(b.size(), 256);
 
     b = f.read(0);
-    QCOMPARE(b.length(), 0);
+    QCOMPARE(b.size(), 0);
 }
 
 //--------------------------------------------------------------------
@@ -270,9 +256,17 @@ void tst_QIODevice::unget()
             result = QByteArray("ZXCV");
             lineResult = "ZXCV";
         } else {
-            if (!QtNetworkSettings::verifyTestNetworkSettings())
-                QSKIP("No network test server available");
-            socket.connectToHost(QtNetworkSettings::serverName(), 80);
+#ifdef QT_TEST_SERVER
+            const bool hasNetworkServer =
+                    QtNetworkSettings::verifyConnection(QtNetworkSettings::httpServerName(), 80);
+#else
+            const bool hasNetworkServer = QtNetworkSettings::verifyTestNetworkSettings();
+#endif
+            if (!hasNetworkServer) {
+                qInfo("No network test server: skipping QTcpSocket part of test.");
+                continue;
+            }
+            socket.connectToHost(QtNetworkSettings::httpServerName(), 80);
             socket.write("GET / HTTP/1.0\r\n\r\n");
             QVERIFY(socket.waitForReadyRead());
             dev = &socket;
@@ -407,6 +401,9 @@ void tst_QIODevice::readLine()
     QVERIFY(buffer.open(QIODevice::ReadWrite));
     QVERIFY(buffer.canReadLine());
 
+    QTest::ignoreMessage(QtWarningMsg, "QIODevice::readLine (QBuffer): Called with maxSize < 2");
+    QCOMPARE(buffer.readLine(nullptr, 0), qint64(-1));
+
     int linelen = data.indexOf('\n') + 1;
     QByteArray line;
     line.reserve(linelen + 100);
@@ -539,6 +536,168 @@ void tst_QIODevice::readLine2()
     }
 }
 
+void tst_QIODevice::readLineInto_Checks_data()
+{
+    QTest::addColumn<bool>("open");
+    QTest::addColumn<QIODevice::OpenModeFlag>("openModeFlag");
+    QTest::addColumn<QString>("warningMessage");
+
+    QTest::newRow("Device not open") << false << QIODevice::ReadOnly
+                                     << "QIODevice::readLineInto (QBuffer): device not open";
+    QTest::newRow("Write only") << true << QIODevice::WriteOnly
+                                << "QIODevice::readLineInto (QBuffer): WriteOnly device";
+    QTest::newRow("Incorrect maxSize") << true << QIODevice::ReadOnly
+                                       << "QIODevice::readLineInto (QBuffer): Called with maxSize "
+                                          "< 2";
+}
+
+void tst_QIODevice::readLineInto_Checks()
+{
+    QFETCH(bool, open);
+    QFETCH(QIODevice::OpenModeFlag, openModeFlag);
+    QFETCH(QString, warningMessage);
+
+    QByteArray data("Try to read this.");
+    QBuffer buffer(&data);
+
+    QByteArray l1 = "Not Empty";
+    QVERIFY(!l1.isEmpty());
+    qsizetype cap_before = l1.capacity();
+
+    if (open) {
+        QVERIFY(buffer.open(openModeFlag));
+        buffer.seek(0);
+    }
+    qint64 pos_before = buffer.pos();
+
+    QTest::ignoreMessage(QtWarningMsg, warningMessage.toLatin1());
+    QCOMPARE(buffer.readLineInto(&l1, 1), false);
+    QVERIFY(l1.isEmpty()); // Make sure readLineInto() makes l1 empty in case an error occurred.
+
+    QVERIFY(l1.capacity() >= cap_before); // Capacity should not be reduced.
+    QCOMPARE(buffer.pos(), pos_before);
+}
+
+void tst_QIODevice::readLineInto()
+{
+    QByteArray data ("First line.\r\n");
+    data.append(QByteArray(100, 'x'));
+    data.append("\r\n");
+    data.append(QByteArray(32769, 'y'));
+    data.append("\r\n");
+    data.append(QByteArray(16388, 'z'));
+    data.append("\r\nThe end.");
+
+    QBuffer buffer(&data);
+    QVERIFY(buffer.open(QIODevice::ReadOnly));
+    QVERIFY(buffer.canReadLine());
+    buffer.seek(0);
+    QByteArray l1;
+
+    qsizetype cap_before = l1.capacity();
+    qint64 pos_before = buffer.pos();
+    QCOMPARE(buffer.readLineInto(&l1, 0), true);
+    QCOMPARE(l1, "First line.\r\n");
+    QCOMPARE_GT(l1.capacity(), cap_before);
+    QVERIFY(buffer.pos() > pos_before);
+
+    cap_before = l1.capacity();
+    pos_before = buffer.pos();
+    QCOMPARE(buffer.readLineInto(&l1), true);
+    QCOMPARE(l1.size(), 100 + 2);
+    QCOMPARE_GE(l1.capacity(), cap_before);
+    QCOMPARE(buffer.pos(), pos_before + 102);
+
+    pos_before = buffer.pos();
+    QCOMPARE(buffer.readLineInto(nullptr), true); // Read: 32769 'y' + '\r' + '\n'
+                                                  // but don't store it.
+    QCOMPARE(buffer.pos(), pos_before + 32769 + 2);
+
+    pos_before = buffer.pos();
+    QCOMPARE(buffer.readLineInto(nullptr, 16388 + 2), true);
+    QCOMPARE(buffer.pos(), pos_before + 16388 + 2);
+
+    pos_before = buffer.pos();
+    QByteArray *l = nullptr;
+    QCOMPARE(buffer.readLineInto(l), true); // Read "The end." but don't store it.
+    QVERIFY(buffer.pos() > pos_before);
+
+    cap_before = l1.capacity();
+    pos_before = buffer.pos();
+    QCOMPARE(buffer.readLineInto(&l1), false); // End of buffer.
+    QCOMPARE_EQ(l1.capacity(), cap_before);
+    QCOMPARE(buffer.pos(), pos_before);
+}
+
+void tst_QIODevice::readLineInto_qspan()
+{
+    QByteArray data ("1st Line\r\nL2\r\nRead the rest");
+    QBuffer buffer(&data);
+
+    {
+        QVERIFY(buffer.open(QIODevice::ReadOnly));
+        QVERIFY(buffer.canReadLine());
+        buffer.seek(0);
+
+        QSpan<char> span; // zero-sized span
+        QTest::ignoreMessage(QtWarningMsg,
+                             "QIODevice::readLineInto (QBuffer): Called with maxSize < 1");
+        QCOMPARE(buffer.readLineInto(span), "");
+
+        char buffer1[1024];
+        QCOMPARE(buffer.readLineInto(buffer1), "1st Line\r\n");
+
+        uchar buffer2[4]; // length of the buffer is equal to the size of the line
+        QCOMPARE(buffer.readLineInto(buffer2), "L2\r\n");
+
+        std::byte buffer3[5]; // length of the buffer is less than the size of the line
+        QCOMPARE(buffer.readLineInto(buffer3), "Read ");
+        QCOMPARE(buffer.readLineInto(buffer1), "the rest"); // read the rest
+
+        QCOMPARE(buffer.readLineInto(span), ""); // No warning even thought maxSize < 1 because we
+                                                 // are at the end
+    }
+
+    {
+        QVERIFY(buffer.open(QIODevice::ReadOnly | QIODevice::Text)); // "\r\n" is translated to '\n'
+        QVERIFY(buffer.canReadLine());
+        buffer.seek(0);
+
+        QSpan<char> span; // zero-sized span
+        QTest::ignoreMessage(QtWarningMsg,
+                             "QIODevice::readLineInto (QBuffer): Called with maxSize < 1");
+        QCOMPARE(buffer.readLineInto(span), "");
+
+        char buffer1[1024];
+        QCOMPARE(buffer.readLineInto(buffer1), "1st Line\n");
+
+        uchar buffer2[3]; // length of the buffer is equal to the size of the line
+        QCOMPARE(buffer.readLineInto(buffer2), "L2\n");
+
+        std::byte buffer3[5]; // length of the buffer is less than the size of the line
+        QCOMPARE(buffer.readLineInto(buffer3), "Read ");
+        QCOMPARE(buffer.readLineInto(buffer1), "the rest"); // read the rest
+
+        QCOMPARE(buffer.readLineInto(span), ""); // No warning even thought maxSize < 1 because we
+                                                 // are at the end
+    }
+
+    {
+        // This test checks the behavior when !keepDataInBuffer and !buffer.isEmpty().
+        // 'buffer' was always empty in the previous tests and calling ungetChar() changes that.
+        QByteArray data2 ("Q");
+        QBuffer buffer2(&data2);
+        QVERIFY(buffer2.open(QIODevice::ReadOnly));
+        buffer2.seek(0);
+        buffer2.read(1);
+        buffer2.ungetChar('t'); // Make the buffer size equal to 1
+
+        char buf[1];
+        QCOMPARE(buffer2.readLineInto(buf), "t");
+        QCOMPARE(buffer2.readLineInto(buf), ""); // no more data to read
+    }
+}
+
 class SequentialReadBuffer : public QIODevice
 {
 public:
@@ -555,7 +714,8 @@ protected:
     qint64 readData(char *data, qint64 maxSize) override
     {
         maxSize = qMin(maxSize, qint64(buf->size() - offset));
-        memcpy(data, buf->constData() + offset, maxSize);
+        if (maxSize > 0)
+            memcpy(data, buf->constData() + offset, maxSize);
         offset += maxSize;
         return maxSize;
     }
@@ -598,13 +758,15 @@ protected:
     qint64 readData(char *data, qint64 maxSize) override
     {
         maxSize = qMin(maxSize, qint64(buf.size() - pos()));
-        memcpy(data, buf.constData() + pos(), maxSize);
+        if (maxSize > 0)
+            memcpy(data, buf.constData() + pos(), maxSize);
         return maxSize;
     }
     qint64 writeData(const char *data, qint64 maxSize) override
     {
         maxSize = qMin(maxSize, qint64(buf.size() - pos()));
-        memcpy(buf.data() + pos(), data, maxSize);
+        if (maxSize > 0)
+            memcpy(buf.data() + pos(), data, maxSize);
         return maxSize;
     }
 
@@ -646,11 +808,12 @@ void tst_QIODevice::skip_data()
     do {
         QByteArray devName(sequential ? "sequential" : "random-access");
 
-        QTest::newRow(qPrintable(devName + "-small_data")) << true  << QByteArray("abcdefghij")
+        QTest::newRow(qPrintable(devName + "-small_data")) << sequential
+                                                           << QByteArray("abcdefghij")
                                                            << 3 << 6 << 6 << 'j';
-        QTest::newRow(qPrintable(devName + "-big_data")) << true  << bigData
+        QTest::newRow(qPrintable(devName + "-big_data")) << sequential << bigData
                                                          << 1 << 10000 << 10000 << 'x';
-        QTest::newRow(qPrintable(devName + "-beyond_the_end")) << true  << bigData
+        QTest::newRow(qPrintable(devName + "-beyond_the_end")) << sequential << bigData
                                                                << 1 << 20000 << 19999 << '\0';
 
         sequential = !sequential;
