@@ -489,7 +489,7 @@ void tst_Android::orientationChange()
     };
 
     auto requestOrientation = [nativeOrientation, context](Qt::ScreenOrientation expected) {
-        context.callMethod<void>("setRequestedOrientation", nativeOrientation(expected));
+        context.callMethod("setRequestedOrientation", nativeOrientation(expected));
     };
 
     auto restoreOrientation = qScopeGuard([&] {
