@@ -1,31 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2021 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// Copyright (C) 2021 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QSignalSpy>
@@ -37,10 +12,18 @@
 #include <QScopeGuard>
 #include "theplugin/plugininterface.h"
 
+#include <QtCore/private/qsimd_p.h>
+
 #if defined(QT_BUILD_INTERNAL) && defined(Q_OF_MACH_O)
 #  include <QtCore/private/qmachparser_p.h>
 #endif
 
+#ifdef Q_OS_ANDROID
+#include <private/qjnihelpers_p.h>
+#endif
+
+using namespace Qt::StringLiterals;
+
 // Helper macros to let us know if some suffixes are valid
 #define bundle_VALID    false
 #define dylib_VALID     false
@@ -56,10 +39,10 @@
 # define bundle_VALID   true
 # define dylib_VALID    true
 # define so_VALID       true
-# ifdef QT_NO_DEBUG
-#  define SUFFIX         ".dylib"
-# else
+# if QT_CONFIG(debug) && !QT_CONFIG(framework)
 #  define SUFFIX         "_debug.dylib"
+# else
+#  define SUFFIX         ".dylib"
 # endif
 # define PREFIX         "lib"
 
@@ -153,7 +136,7 @@ static std::unique_ptr<QTemporaryFile> patchElf(const QString &source, ElfPatche
         const char *basename = QTest::currentDataTag();
         if (!basename)
             basename = QTest::currentTestFunction();
-        tmplib.reset(new QTemporaryFile(basename + QString(".XXXXXX" SUFFIX)));
+        tmplib.reset(new QTemporaryFile(QDir::currentPath() + u'/' + basename + u".XXXXXX" SUFFIX ""_s));
         QVERIFY2(tmplib->open(), qPrintable(tmplib->errorString()));
 
         // sanity-check
@@ -222,6 +205,7 @@ private slots:
     void loadCorruptElfOldPlugin();
 #  endif
 #endif
+    void archSpecificVersion();
     void loadMachO_data();
     void loadMachO();
     void relativePath();
@@ -231,6 +215,7 @@ private slots:
     void preloadedPlugin_data();
     void preloadedPlugin();
     void staticPlugins();
+    void reregisteredStaticPlugins();
 };
 
 Q_IMPORT_PLUGIN(StaticPlugin)
@@ -302,7 +287,9 @@ void tst_QPluginLoader::errorString()
     QVERIFY(!unloaded);
     }
 
-#if !defined(Q_OS_WIN) && !defined(Q_OS_MAC) && !defined(Q_OS_HPUX)
+// A bug in QNX causes the test to crash on exit after attempting to load
+// a shared library with undefined symbols (tracked as QTBUG-114682).
+#if defined(Q_OF_ELF) && !defined(Q_OS_QNX)
     {
     QPluginLoader loader( sys_qualifiedLibraryName("almostplugin"));     //a plugin with unresolved symbols
     loader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
@@ -362,16 +349,34 @@ void tst_QPluginLoader::loadHints()
     QCOMPARE(loader.loadHints(), QLibrary::PreventUnloadHint);   //Do not crash
     loader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
     QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
+    // We can clear load hints when file name is not set.
+    loader.setLoadHints(QLibrary::LoadHints{});
+    QCOMPARE(loader.loadHints(), QLibrary::LoadHints{});
+    // Set the hints again
+    loader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
+    QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
     loader.setFileName( sys_qualifiedLibraryName("theplugin"));     //a plugin
     QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
 
+    QPluginLoader loader4;
+    QCOMPARE(loader4.loadHints(), QLibrary::PreventUnloadHint);
+    loader4.setLoadHints(QLibrary::LoadHints{});
+    QCOMPARE(loader4.loadHints(), QLibrary::LoadHints{});
+    loader4.setFileName(sys_qualifiedLibraryName("theplugin"));
+    // Hints are merged with hints from the previous loader.
+    QCOMPARE(loader4.loadHints(), QLibrary::ResolveAllSymbolsHint);
+    // We cannot clear load hints after associating the loader with a file.
+    loader.setLoadHints(QLibrary::LoadHints{});
+    QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
+
     QPluginLoader loader2;
     QCOMPARE(loader2.loadHints(), QLibrary::PreventUnloadHint);
     loader2.setFileName(sys_qualifiedLibraryName("theplugin"));
-    QCOMPARE(loader2.loadHints(), QLibrary::PreventUnloadHint);
+    // Hints are merged with hints from previous loaders.
+    QCOMPARE(loader2.loadHints(), QLibrary::PreventUnloadHint | QLibrary::ResolveAllSymbolsHint);
 
     QPluginLoader loader3(sys_qualifiedLibraryName("theplugin"));
-    QCOMPARE(loader3.loadHints(), QLibrary::PreventUnloadHint);
+    QCOMPARE(loader3.loadHints(), QLibrary::PreventUnloadHint | QLibrary::ResolveAllSymbolsHint);
 }
 
 void tst_QPluginLoader::deleteinstanceOnUnload()
@@ -401,14 +406,14 @@ void tst_QPluginLoader::deleteinstanceOnUnload()
         QVERIFY(spy2.isValid());
         if (pass == 0) {
             QCOMPARE(loader2.unload(), false);  // refcount not reached 0, not really unloaded
-            QCOMPARE(spy1.count(), 0);
-            QCOMPARE(spy2.count(), 0);
+            QCOMPARE(spy1.size(), 0);
+            QCOMPARE(spy2.size(), 0);
         }
         QCOMPARE(instance1->pluginName(), QLatin1String("Plugin ok"));
         QCOMPARE(instance2->pluginName(), QLatin1String("Plugin ok"));
         QVERIFY(loader1.unload());   // refcount reached 0, did really unload
-        QCOMPARE(spy1.count(), 1);
-        QCOMPARE(spy2.count(), 1);
+        QCOMPARE(spy1.size(), 1);
+        QCOMPARE(spy2.size(), 1);
     }
 }
 
@@ -476,7 +481,7 @@ static void loadCorruptElfCommonRows()
         memcpy(h, &o, sizeof(o));
     });
     newRow("invalid-word-size", "file is for a different word size", [](H h) {
-        h->e_ident[EI_CLASS] = ELFCLASSNONE;;
+        h->e_ident[EI_CLASS] = ELFCLASSNONE;
     });
     newRow("unknown-word-size", "file is for a different word size", [](H h) {
         h->e_ident[EI_CLASS] |= 0x40;
@@ -723,6 +728,12 @@ static void loadCorruptElf_helper(const QString &origLibrary)
     QFETCH(QString, snippet);
     QFETCH(ElfPatcher, patcher);
 
+#ifdef Q_OS_ANDROID
+    // patchElf() tries to map with private mode
+    if (QtAndroidPrivate::isUncompressedNativeLibs())
+        QSKIP("Mapping in-APK libraries with private mode is not supported on Android");
+#endif
+
     std::unique_ptr<QTemporaryFile> tmplib = patchElf(origLibrary, patcher);
 
     QPluginLoader lib(tmplib->fileName());
@@ -850,6 +861,28 @@ void tst_QPluginLoader::loadCorruptElfOldPlugin()
 #  endif // Qt 7
 #endif // Q_OF_ELF
 
+void tst_QPluginLoader::archSpecificVersion()
+{
+#if !defined(QT_SHARED)
+    QSKIP("This test requires Qt to create shared libraries.");
+#endif
+    QPluginLoader loader(sys_qualifiedLibraryName("theplugin"));
+    QVERIFY2(loader.load(), qPrintable(loader.errorString()));
+
+    QString expectedArch;
+#if defined(Q_PROCESSOR_X86_64) && defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
+    // On Unix systems (other than Darwin, which has fat binaries),
+    // QPluginLoader will load a separate file for x86-64-v3 systems.
+    if (qCpuHasFeature(ArchHaswell))
+        expectedArch = "x86-64-v3";
+#endif
+
+    PluginInterface* theplugin = qobject_cast<PluginInterface*>(loader.instance());
+    QVERIFY(theplugin);
+    QCOMPARE(theplugin->architectureName(), expectedArch);
+    QVERIFY(loader.unload());
+}
+
 void tst_QPluginLoader::loadMachO_data()
 {
 #if defined(QT_BUILD_INTERNAL) && defined(Q_OF_MACH_O)
@@ -862,26 +895,23 @@ void tst_QPluginLoader::loadMachO_data()
 
 #  ifdef Q_PROCESSOR_X86_64
     QTest::newRow("machtest/good.x86_64.dylib") << true;
-    QTest::newRow("machtest/good.i386.dylib") << false;
+    QTest::newRow("machtest/good.arm64.dylib") << false;
     QTest::newRow("machtest/good.fat.no-x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.no-i386.dylib") << true;
-#  elif defined(Q_PROCESSOR_X86_32)
-    QTest::newRow("machtest/good.i386.dylib") << true;
+    QTest::newRow("machtest/good.fat.no-arm64.dylib") << true;
+#  elif defined(Q_PROCESSOR_ARM)
+    QTest::newRow("machtest/good.arm64.dylib") << true;
     QTest::newRow("machtest/good.x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.no-i386.dylib") << false;
+    QTest::newRow("machtest/good.fat.no-arm64.dylib") << false;
     QTest::newRow("machtest/good.fat.no-x86_64.dylib") << true;
 #  endif
-#  ifndef Q_PROCESSOR_POWER_64
-    QTest::newRow("machtest/good.ppc64.dylib") << false;
-#  endif
 
     QTest::newRow("machtest/good.fat.all.dylib") << true;
     QTest::newRow("machtest/good.fat.stub-x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.stub-i386.dylib") << false;
+    QTest::newRow("machtest/good.fat.stub-arm64.dylib") << false;
 
     QDir d(QFINDTESTDATA("machtest"));
-    QStringList badlist = d.entryList(QStringList() << "bad*.dylib");
-    foreach (const QString &bad, badlist)
+    const QStringList badlist = d.entryList(QStringList() << "bad*.dylib");
+    for (const QString &bad : badlist)
         QTest::newRow(qPrintable("machtest/" + bad)) << false;
 #endif
 }
@@ -905,12 +935,7 @@ void tst_QPluginLoader::loadMachO()
     }
 
     QVERIFY(r.pos > 0);
-    QVERIFY(size_t(r.length) >= sizeof(void*));
     QVERIFY(r.pos + r.length < data.size());
-    QCOMPARE(r.pos & (sizeof(void*) - 1), 0UL);
-
-    void *value = *(void**)(data.constData() + r.pos);
-    QCOMPARE(value, sizeof(void*) > 4 ? (void*)(0xc0ffeec0ffeeL) : (void*)0xc0ffee);
 
     // now that we know it's valid, let's try to make it invalid
     ulong offeredlen = r.pos;
@@ -994,7 +1019,7 @@ void tst_QPluginLoader::reloadPlugin()
     QSignalSpy spy(loader.instance(), &QObject::destroyed);
     QVERIFY(spy.isValid());
     QVERIFY(loader.unload());   // refcount reached 0, did really unload
-    QCOMPARE(spy.count(), 1);
+    QCOMPARE(spy.size(), 1);
 
     // reload plugin
     QVERIFY(loader.load());
@@ -1009,6 +1034,10 @@ void tst_QPluginLoader::reloadPlugin()
 
 void tst_QPluginLoader::loadSectionTableStrippedElf()
 {
+#ifdef Q_OS_ANDROID
+    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 24)
+        QSKIP("Android 7+ (API 24+) linker doesn't allow missing or bad section header");
+#endif
 #if !defined(QT_SHARED)
     QSKIP("This test requires a shared build of Qt, as QPluginLoader::setFileName is a no-op in static builds");
 #elif !defined(Q_OF_ELF)
@@ -1101,19 +1130,18 @@ void tst_QPluginLoader::staticPlugins()
     const QObjectList instances = QPluginLoader::staticInstances();
     QVERIFY(instances.size());
 
-    bool found = false;
-    for (QObject *obj : instances) {
-        found = obj->metaObject()->className() == QLatin1String("StaticPlugin");
-        if (found)
-            break;
-    }
-    QVERIFY(found);
+    // ensure the our plugin only shows up once
+    int foundCount = std::count_if(instances.begin(), instances.end(), [](QObject *obj) {
+            return obj->metaObject()->className() == QLatin1String("StaticPlugin");
+    });
+    QCOMPARE(foundCount, 1);
 
     const auto plugins = QPluginLoader::staticPlugins();
     QCOMPARE(plugins.size(), instances.size());
 
     // find the metadata
     QJsonObject metaData;
+    bool found = false;
     for (const auto &p : plugins) {
         metaData = p.metaData();
         found = metaData.value("className").toString() == QLatin1String("StaticPlugin");
@@ -1129,6 +1157,18 @@ void tst_QPluginLoader::staticPlugins()
     QCOMPARE(metaData.value("URI").toString(), "qt.test.pluginloader.staticplugin");
 }
 
+void tst_QPluginLoader::reregisteredStaticPlugins()
+{
+    // the Q_IMPORT_PLUGIN macro will have already done this
+    qRegisterStaticPluginFunction(QT_MANGLE_NAMESPACE(qt_static_plugin_StaticPlugin)());
+    staticPlugins();
+    if (QTest::currentTestFailed())
+        return;
+
+    qRegisterStaticPluginFunction(QT_MANGLE_NAMESPACE(qt_static_plugin_StaticPlugin)());
+    staticPlugins();
+}
+
 
 QTEST_MAIN(tst_QPluginLoader)
 #include "tst_qpluginloader.moc"
