@@ -19,8 +19,19 @@
 using namespace QtPrivate::DateTimeConstants;
 using namespace Qt::StringLiterals;
 
-#if defined(Q_OS_WIN) && !QT_CONFIG(icu)
-#  define USING_WIN_TZ
+#undef USING_MS_TZDB
+#undef USING_WIN_TZ
+#ifdef Q_OS_WIN
+#  if QT_CONFIG(timezone_tzdb)
+#    define USING_MS_TZDB
+#  elif !QT_CONFIG(icu)
+#    define USING_WIN_TZ
+#  endif
+#endif
+
+#undef GLIBC_TZDB_MISPARSE
+#if QT_CONFIG(timezone_tzdb) && defined(__GLIBCXX__) // && _GLIBCXX_RELEASE <= 14
+#  define GLIBC_TZDB_MISPARSE // QTBUG-127598
 #endif
 
 class tst_QDate : public QObject
@@ -192,7 +203,8 @@ void tst_QDate::isValid_data()
     QTest::newRow("jd latest formula")   << 1400000 << 12 << 31 << qint64(513060925) << true;
 }
 
-#if __cpp_lib_chrono >= 201907L
+// INTEGRITY incident-85878 (timezone and clock_cast are not supported)
+#if __cpp_lib_chrono >= 201907L && !defined(Q_OS_INTEGRITY)
 // QDate has a bigger range than year_month_date. The tests use this bigger
 // range. However building a year_month_time with "out of range" data has
 // unspecified results, so don't do that. See [time.cal.year],
@@ -237,7 +249,8 @@ void tst_QDate::isValid()
         QCOMPARE(d.year(), year);
         QCOMPARE(d.month(), month);
         QCOMPARE(d.day(), day);
-#if __cpp_lib_chrono >= 201907L
+// INTEGRITY incident-85878 (timezone and clock_cast are not supported)
+#if __cpp_lib_chrono >= 201907L && !defined(Q_OS_INTEGRITY)
         std::optional<std::chrono::year_month_day> ymd = convertToStdYearMonthDay(year, month, day);
         if (ymd) {
             QDate d = *ymd;
@@ -507,6 +520,7 @@ void tst_QDate::weekNumber_invalid()
 enum BackendKludge { IgnoreStart = 1, IgnoreEnd = 2, };
 Q_DECLARE_FLAGS(BackendKludges, BackendKludge)
 Q_DECLARE_OPERATORS_FOR_FLAGS(BackendKludges)
+#undef KLUDGING
 
 void tst_QDate::startOfDay_endOfDay_data()
 {
@@ -522,13 +536,26 @@ void tst_QDate::startOfDay_endOfDay_data()
     const QTime early(0, 0), late(23, 59, 59, 999), invalid(QDateTime().time());
     constexpr BackendKludges Clean = {};
     constexpr BackendKludges IgnoreBoth = IgnoreStart | IgnoreEnd;
+    // Use IgnoreBoth directly for the one transition Android lacks; the other
+    // two that need kludges also fail this one.
+#ifdef Q_OS_ANDROID
+#define KLUDGING
+#endif
 #ifdef USING_WIN_TZ
     constexpr BackendKludges MsNoStart = IgnoreStart;
     constexpr BackendKludges MsNoBoth = IgnoreBoth;
+#define KLUDGING
 #else
     constexpr BackendKludges MsNoStart = Clean;
     constexpr BackendKludges MsNoBoth = Clean;
-    // And use IgnoreBoth directly for the one transition Android lacks.
+#endif
+#if QT_CONFIG(timezone) && QT_CONFIG(timezone_tzdb) && defined(__GLIBCXX__)
+    // The IANA-DB parser in libstdc++ (at least up to _GLIBCXX_RELEASE == 14) gets
+    // a lot of zone-transitions wrong in C++20's tzdb :-(
+    constexpr BackendKludges GlibCxxNoBoth = IgnoreBoth;
+#define KLUDGING
+#else
+    constexpr BackendKludges GlibCxxNoBoth = Clean;
 #endif
     const QTimeZone UTC(QTimeZone::UTC);
 
@@ -554,9 +581,6 @@ void tst_QDate::startOfDay_endOfDay_data()
         const QTime end;
         const BackendKludges msOpt;
     } transitions[] = {
-        // The western Mexico time-zones skipped the first hour of 1970.
-        { "BajaMexico", "America/Hermosillo", QDate(1970, 1, 1), QTime(1, 0), late, MsNoStart },
-
         // Compare tst_QDateTime::fromStringDateFormat(ISO 24:00 in DST).
         { "Brazil", "America/Sao_Paulo", QDate(2008, 10, 19), QTime(1, 0), late, Clean },
 
@@ -567,7 +591,8 @@ void tst_QDate::startOfDay_endOfDay_data()
         // Two Pacific zones skipped days to get on the west of the
         // International Date Line; those days have neither start nor end.
         { "Kiritimati", "Pacific/Kiritimati", QDate(1994, 12, 31), invalid, invalid, IgnoreBoth },
-        { "Samoa", "Pacific/Apia", QDate(2011, 12, 30), invalid, invalid, MsNoBoth },
+        { "Samoa", "Pacific/Apia", QDate(2011, 12, 30), invalid, invalid,
+          MsNoBoth | GlibCxxNoBoth },
 
         // TODO: find other zones with transitions at/crossing midnight.
     };
@@ -610,7 +635,7 @@ void tst_QDate::startOfDay_endOfDay()
     QFETCH(const QTimeZone, zone);
     QFETCH(const QTime, start);
     QFETCH(const QTime, end);
-#if defined(USING_WIN_TZ) || defined(Q_OS_ANDROID) // Coping with backend limitations.
+#ifdef KLUDGING // Cope with backend limitations:
     QFETCH(const BackendKludges, kludge);
 #define UNLESSKLUDGE(flag) if (!kludge.testFlag(flag))
 #else
@@ -627,6 +652,9 @@ void tst_QDate::startOfDay_endOfDay()
     if (start.isValid()) {
         QVERIFY(front.isValid());
         QCOMPARE(front.date(), date);
+#ifdef USING_MS_TZDB
+        QEXPECT_FAIL("Brazil", "MS misreads the IANA DB", Continue);
+#endif
         UNLESSKLUDGE(IgnoreStart) QCOMPARE(front.time(), start);
     } else UNLESSKLUDGE(IgnoreStart) {
         auto report = qScopeGuard([front]() { qDebug() << "Start of day:" << front; });
@@ -644,6 +672,7 @@ void tst_QDate::startOfDay_endOfDay()
     }
 #undef UNLESSKLUDGE
 }
+#undef KLUDGING
 
 void tst_QDate::startOfDay_endOfDay_fixed_data()
 {
@@ -698,7 +727,7 @@ void tst_QDate::startOfDay_endOfDay_fixed()
     // Minimal testing for LocalTime and TimeZone
     QCOMPARE(date.startOfDay().date(), date);
     QCOMPARE(date.endOfDay().date(), date);
-#if QT_CONFIG(timezone)
+#if QT_CONFIG(timezone) && !defined(GLIBC_TZDB_MISPARSE)
     const QTimeZone cet("Europe/Oslo");
     if (cet.isValid()) {
         QCOMPARE(date.startOfDay(cet).date(), date);
@@ -719,10 +748,13 @@ void tst_QDate::startOfDay_endOfDay_bounds()
     // First, check these *are* the start and end of QDateTime's range:
     QVERIFY(first.isValid());
     QVERIFY(last.isValid());
-    QVERIFY(first < epoch);
-    QVERIFY(last > epoch);
-    QVERIFY(!first.addMSecs(-1).isValid() || first.addMSecs(-1) > first);
-    QVERIFY(!last.addMSecs(1).isValid() || last.addMSecs(1) < last);
+    QCOMPARE_LT(first, epoch);
+    QCOMPARE_GT(last, epoch);
+    // Step off end of range => either invalid or at opposite end:
+    if (first.addMSecs(-1).isValid())
+        QCOMPARE_GT(first.addMSecs(-1), first);
+    if (last.addMSecs(1).isValid())
+        QCOMPARE_LT(last.addMSecs(1), last);
 
     // Now test start/end methods with them:
     QCOMPARE(first.date().endOfDay(UTC).time(), QTime(23, 59, 59, 999));
@@ -820,8 +852,8 @@ void tst_QDate::addDays()
     QCOMPARE( dt2.year(), expectedYear );
     QCOMPARE( dt2.month(), expectedMonth );
     QCOMPARE( dt2.day(), expectedDay );
-
-#if __cpp_lib_chrono >= 201907L
+// INTEGRITY incident-85878 (timezone and clock_cast are not supported)
+#if __cpp_lib_chrono >= 201907L && !defined(Q_OS_INTEGRITY)
     QDate dt3 = dt.addDuration( std::chrono::days( amountToAdd ) );
 
     QCOMPARE( dt3.year(), expectedYear );
@@ -997,7 +1029,8 @@ void tst_QDate::daysTo()
 void tst_QDate::orderingCompiles()
 {
     QTestPrivate::testAllComparisonOperatorsCompile<QDate>();
-#if __cpp_lib_chrono >= 201907L
+// INTEGRITY incident-85878 (timezone and clock_cast are not supported)
+#if __cpp_lib_chrono >= 201907L && !defined(Q_OS_INTEGRITY)
     QTestPrivate::testAllComparisonOperatorsCompile<QDate, std::chrono::year_month_day>();
     QTestPrivate::testAllComparisonOperatorsCompile<QDate, std::chrono::year_month_day_last>();
     QTestPrivate::testAllComparisonOperatorsCompile<QDate, std::chrono::year_month_weekday>();
@@ -1080,7 +1113,8 @@ void tst_QDate::ordering()
 
 void tst_QDate::ordering_chrono_types()
 {
-#if __cpp_lib_chrono >= 201907L
+// INTEGRITY incident-85878 (timezone and clock_cast are not supported)
+#if __cpp_lib_chrono >= 201907L && !defined(Q_OS_INTEGRITY)
     using namespace std::chrono;
     QDate friday(2001, 11, 30); // the 5th Friday of November 2001
     // std::chrono::year_month_day
@@ -1127,38 +1161,38 @@ void tst_QDate::operator_insert_extract_data()
     QTest::addColumn<QDate>("date");
     QTest::addColumn<QDataStream::Version>("dataStreamVersion");
 
-    QMap<QDataStream::Version, QString> versionsToTest;
-    versionsToTest.insert(QDataStream::Qt_1_0, QString::fromLatin1("Qt_1_0"));
-    versionsToTest.insert(QDataStream::Qt_2_0, QString::fromLatin1("Qt_2_0"));
-    versionsToTest.insert(QDataStream::Qt_2_1, QString::fromLatin1("Qt_2_1"));
-    versionsToTest.insert(QDataStream::Qt_3_0, QString::fromLatin1("Qt_3_0"));
-    versionsToTest.insert(QDataStream::Qt_3_1, QString::fromLatin1("Qt_3_1"));
-    versionsToTest.insert(QDataStream::Qt_3_3, QString::fromLatin1("Qt_3_3"));
-    versionsToTest.insert(QDataStream::Qt_4_0, QString::fromLatin1("Qt_4_0"));
-    versionsToTest.insert(QDataStream::Qt_4_1, QString::fromLatin1("Qt_4_1"));
-    versionsToTest.insert(QDataStream::Qt_4_2, QString::fromLatin1("Qt_4_2"));
-    versionsToTest.insert(QDataStream::Qt_4_3, QString::fromLatin1("Qt_4_3"));
-    versionsToTest.insert(QDataStream::Qt_4_4, QString::fromLatin1("Qt_4_4"));
-    versionsToTest.insert(QDataStream::Qt_4_5, QString::fromLatin1("Qt_4_5"));
-    versionsToTest.insert(QDataStream::Qt_4_6, QString::fromLatin1("Qt_4_6"));
-    versionsToTest.insert(QDataStream::Qt_4_7, QString::fromLatin1("Qt_4_7"));
-    versionsToTest.insert(QDataStream::Qt_4_8, QString::fromLatin1("Qt_4_8"));
-    versionsToTest.insert(QDataStream::Qt_4_9, QString::fromLatin1("Qt_4_9"));
-    versionsToTest.insert(QDataStream::Qt_5_0, QString::fromLatin1("Qt_5_0"));
+    QMap<QDataStream::Version, QByteArray> versionsToTest;
+    versionsToTest.insert(QDataStream::Qt_1_0, "Qt_1_0"_ba);
+    versionsToTest.insert(QDataStream::Qt_2_0, "Qt_2_0"_ba);
+    versionsToTest.insert(QDataStream::Qt_2_1, "Qt_2_1"_ba);
+    versionsToTest.insert(QDataStream::Qt_3_0, "Qt_3_0"_ba);
+    versionsToTest.insert(QDataStream::Qt_3_1, "Qt_3_1"_ba);
+    versionsToTest.insert(QDataStream::Qt_3_3, "Qt_3_3"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_0, "Qt_4_0"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_1, "Qt_4_1"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_2, "Qt_4_2"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_3, "Qt_4_3"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_4, "Qt_4_4"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_5, "Qt_4_5"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_6, "Qt_4_6"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_7, "Qt_4_7"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_8, "Qt_4_8"_ba);
+    versionsToTest.insert(QDataStream::Qt_4_9, "Qt_4_9"_ba);
+    versionsToTest.insert(QDataStream::Qt_5_0, "Qt_5_0"_ba);
 
-    for (QMap<QDataStream::Version, QString>::ConstIterator it = versionsToTest.constBegin();
-            it != versionsToTest.constEnd(); ++it) {
-        const QString &version(it.value());
-        QTest::newRow(("(invalid) " + version).toLocal8Bit().constData()) << invalidDate() << it.key();
-        QTest::newRow(("(1, 1, 1) " + version).toLocal8Bit().constData()) << QDate(1, 1, 1) << it.key();
-        QTest::newRow(("(-1, 1, 1) " + version).toLocal8Bit().constData()) << QDate(-1, 1, 1) << it.key();
-        QTest::newRow(("(1995, 5, 20) " + version).toLocal8Bit().constData()) << QDate(1995, 5, 20) << it.key();
+    for (auto it = versionsToTest.constBegin(); it != versionsToTest.constEnd(); ++it) {
+        const QByteArray &version(it.value());
+        const char *const tag = version.constData();
+        QTest::addRow("(invalid) %s", tag) << invalidDate() << it.key();
+        QTest::addRow("(1, 1, 1) %s", tag) << QDate(1, 1, 1) << it.key();
+        QTest::addRow("(-1, 1, 1) %s", tag) << QDate(-1, 1, 1) << it.key();
+        QTest::addRow("(1995, 5, 20) %s", tag) << QDate(1995, 5, 20) << it.key();
 
         // Test minimums for quint32/qint64.
         if (it.key() >= QDataStream::Qt_5_0)
-            QTest::newRow(("(-4714, 11, 24) " + version).toLocal8Bit().constData()) << QDate(-4714, 11, 24) << it.key();
+            QTest::addRow("(-4714, 11, 24) %s", tag) << QDate(-4714, 11, 24) << it.key();
         else
-            QTest::newRow(("(-4713, 1, 2) " + version).toLocal8Bit().constData()) << QDate(-4713, 1, 2) << it.key();
+            QTest::addRow("(-4713, 1, 2) %s", tag) << QDate(-4713, 1, 2) << it.key();
     }
 }
 
@@ -1179,119 +1213,120 @@ void tst_QDate::operator_insert_extract()
     QCOMPARE(deserialised, date);
 }
 
-#if QT_CONFIG(datetimeparser)
+#if QT_CONFIG(datestring)
 void tst_QDate::fromStringDateFormat_data()
 {
     QTest::addColumn<QString>("dateStr");
     QTest::addColumn<Qt::DateFormat>("dateFormat");
     QTest::addColumn<QDate>("expectedDate");
 
-    QTest::newRow("text0") << QString("Sat May 20 1995") << Qt::TextDate << QDate(1995, 5, 20);
-    QTest::newRow("text1") << QString("Tue Dec 17 2002") << Qt::TextDate << QDate(2002, 12, 17);
+    QTest::newRow("text0") << u"Sat May 20 1995"_s << Qt::TextDate << QDate(1995, 5, 20);
+    QTest::newRow("text1") << u"Tue Dec 17 2002"_s << Qt::TextDate << QDate(2002, 12, 17);
     QTest::newRow("text2") << QDate(1999, 11, 14).toString(Qt::TextDate) << Qt::TextDate << QDate(1999, 11, 14);
-    QTest::newRow("text3") << QString("xxx Jan 1 0999") << Qt::TextDate << QDate(999, 1, 1);
-    QTest::newRow("text3b") << QString("xxx Jan 1 999") << Qt::TextDate << QDate(999, 1, 1);
-    QTest::newRow("text4") << QString("xxx Jan 1 12345") << Qt::TextDate << QDate(12345, 1, 1);
-    QTest::newRow("text5") << QString("xxx Jan 1 -0001") << Qt::TextDate << QDate(-1, 1, 1);
-    QTest::newRow("text6") << QString("xxx Jan 1 -4712") << Qt::TextDate << QDate(-4712, 1, 1);
-    QTest::newRow("text7") << QString("xxx Nov 25 -4713") << Qt::TextDate << QDate(-4713, 11, 25);
+    QTest::newRow("text3") << u"xxx Jan 1 0999"_s << Qt::TextDate << QDate(999, 1, 1);
+    QTest::newRow("text3b") << u"xxx Jan 1 999"_s << Qt::TextDate << QDate(999, 1, 1);
+    QTest::newRow("text4") << u"xxx Jan 1 12345"_s << Qt::TextDate << QDate(12345, 1, 1);
+    QTest::newRow("text5") << u"xxx Jan 1 -0001"_s << Qt::TextDate << QDate(-1, 1, 1);
+    QTest::newRow("text6") << u"xxx Jan 1 -4712"_s << Qt::TextDate << QDate(-4712, 1, 1);
+    QTest::newRow("text7") << u"xxx Nov 25 -4713"_s << Qt::TextDate << QDate(-4713, 11, 25);
     QTest::newRow("text, empty") << QString() << Qt::TextDate << QDate();
-    QTest::newRow("text, 3 part") << QString("part1 part2 part3") << Qt::TextDate << QDate();
-    QTest::newRow("text, invalid month name") << QString("Wed BabytownFrolics 8 2012") << Qt::TextDate << QDate();
-    QTest::newRow("text, invalid day") << QString("Wed May Wilhelm 2012") << Qt::TextDate << QDate();
-    QTest::newRow("text, invalid year") << QString("Wed May 8 Cats") << Qt::TextDate << QDate();
+    QTest::newRow("text, 3 part") << u"part1 part2 part3"_s << Qt::TextDate << QDate();
+    QTest::newRow("text, invalid month name") << u"Wed BabytownFrolics 8 2012"_s << Qt::TextDate << QDate();
+    QTest::newRow("text, invalid day") << u"Wed May Wilhelm 2012"_s << Qt::TextDate << QDate();
+    QTest::newRow("text, invalid year") << u"Wed May 8 Cats"_s << Qt::TextDate << QDate();
 
-    QTest::newRow("iso0") << QString("1995-05-20") << Qt::ISODate << QDate(1995, 5, 20);
-    QTest::newRow("iso1") << QString("2002-12-17") << Qt::ISODate << QDate(2002, 12, 17);
+    QTest::newRow("iso0") << u"1995-05-20"_s << Qt::ISODate << QDate(1995, 5, 20);
+    QTest::newRow("iso1") << u"2002-12-17"_s << Qt::ISODate << QDate(2002, 12, 17);
     QTest::newRow("iso2") << QDate(1999, 11, 14).toString(Qt::ISODate) << Qt::ISODate << QDate(1999, 11, 14);
-    QTest::newRow("iso3") << QString("0999-01-01") << Qt::ISODate << QDate(999, 1, 1);
-    QTest::newRow("iso3b") << QString("0999-01-01") << Qt::ISODate << QDate(999, 1, 1);
-    QTest::newRow("iso4") << QString("2000101101")      << Qt::ISODate << QDate();
-    QTest::newRow("iso5") << QString("2000/01/01")      << Qt::ISODate << QDate(2000, 1, 1);
-    QTest::newRow("iso6") << QString("2000-01-01 blah") << Qt::ISODate << QDate(2000, 1, 1);
-    QTest::newRow("iso7") << QString("2000-01-011blah") << Qt::ISODate << QDate();
-    QTest::newRow("iso8") << QString("2000-01-01blah")  << Qt::ISODate << QDate(2000, 1, 1);
-    QTest::newRow("iso9") << QString("-001-01-01")      << Qt::ISODate << QDate();
-    QTest::newRow("iso10") << QString("99999-01-01")    << Qt::ISODate << QDate();
+    QTest::newRow("iso3") << u"0999-01-01"_s << Qt::ISODate << QDate(999, 1, 1);
+    QTest::newRow("iso3b") << u"0999-01-01"_s << Qt::ISODate << QDate(999, 1, 1);
+    QTest::newRow("iso4") << u"2000101101"_s      << Qt::ISODate << QDate();
+    QTest::newRow("iso5") << u"2000/01/01"_s      << Qt::ISODate << QDate(2000, 1, 1);
+    QTest::newRow("iso6") << u"2000-01-01 blah"_s << Qt::ISODate << QDate(2000, 1, 1);
+    QTest::newRow("iso7") << u"2000-01-011blah"_s << Qt::ISODate << QDate();
+    QTest::newRow("iso8") << u"2000-01-01blah"_s  << Qt::ISODate << QDate(2000, 1, 1);
+    QTest::newRow("iso9") << u"-001-01-01"_s      << Qt::ISODate << QDate();
+    QTest::newRow("iso10") << u"99999-01-01"_s    << Qt::ISODate << QDate();
+    QTest::newRow("iso-yr-0") << u"0000-01-01"_s  << Qt::ISODate << QDate();
 
     // Test Qt::RFC2822Date format (RFC 2822).
-    QTest::newRow("RFC 2822") << QString::fromLatin1("13 Feb 1987 13:24:51 +0100")
+    QTest::newRow("RFC 2822") << u"13 Feb 1987 13:24:51 +0100"_s
         << Qt::RFC2822Date << QDate(1987, 2, 13);
     QTest::newRow("RFC 2822 after space")
-        << QString::fromLatin1(" 13 Feb 1987 13:24:51 +0100")
+        << u" 13 Feb 1987 13:24:51 +0100"_s
         << Qt::RFC2822Date << QDate(1987, 2, 13);
-    QTest::newRow("RFC 2822 with day") << QString::fromLatin1("Thu, 01 Jan 1970 00:12:34 +0000")
+    QTest::newRow("RFC 2822 with day") << u"Thu, 01 Jan 1970 00:12:34 +0000"_s
         << Qt::RFC2822Date << epochDate();
     QTest::newRow("RFC 2822 with day after space")
-        << QString::fromLatin1(" Thu, 01 Jan 1970 00:12:34 +0000")
+        << u" Thu, 01 Jan 1970 00:12:34 +0000"_s
         << Qt::RFC2822Date << epochDate();
     // No timezone
-    QTest::newRow("RFC 2822 no timezone") << QString::fromLatin1("01 Jan 1970 00:12:34")
+    QTest::newRow("RFC 2822 no timezone") << u"01 Jan 1970 00:12:34"_s
         << Qt::RFC2822Date << epochDate();
     // No time specified
-    QTest::newRow("RFC 2822 date only") << QString::fromLatin1("01 Nov 2002")
+    QTest::newRow("RFC 2822 date only") << u"01 Nov 2002"_s
         << Qt::RFC2822Date << QDate(2002, 11, 1);
-    QTest::newRow("RFC 2822 with day date only") << QString::fromLatin1("Fri, 01 Nov 2002")
+    QTest::newRow("RFC 2822 with day date only") << u"Fri, 01 Nov 2002"_s
         << Qt::RFC2822Date << QDate(2002, 11, 1);
     QTest::newRow("RFC 2822 malformed time")
-        << QString::fromLatin1("01 Nov 2002 0:") << Qt::RFC2822Date << QDate();
+        << u"01 Nov 2002 0:"_s << Qt::RFC2822Date << QDate();
     // Test invalid month, day, year
-    QTest::newRow("RFC 2822 invalid month name") << QString::fromLatin1("13 Fev 1987 13:24:51 +0100")
+    QTest::newRow("RFC 2822 invalid month name") << u"13 Fev 1987 13:24:51 +0100"_s
         << Qt::RFC2822Date << QDate();
-    QTest::newRow("RFC 2822 invalid day") << QString::fromLatin1("36 Fev 1987 13:24:51 +0100")
+    QTest::newRow("RFC 2822 invalid day") << u"36 Fev 1987 13:24:51 +0100"_s
         << Qt::RFC2822Date << QDate();
-    QTest::newRow("RFC 2822 invalid year") << QString::fromLatin1("13 Fev 0000 13:24:51 +0100")
+    QTest::newRow("RFC 2822 invalid year") << u"13 Fev 0000 13:24:51 +0100"_s
         << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 2822 invalid character at end")
-        << QString::fromLatin1("01 Jan 2012 08:00:00 +0100!") << Qt::RFC2822Date << QDate();
+        << u"01 Jan 2012 08:00:00 +0100!"_s << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 2822 invalid character at front")
-        << QString::fromLatin1("!01 Jan 2012 08:00:00 +0100") << Qt::RFC2822Date << QDate();
+        << u"!01 Jan 2012 08:00:00 +0100"_s << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 2822 invalid character both ends")
-        << QString::fromLatin1("!01 Jan 2012 08:00:00 +0100!") << Qt::RFC2822Date << QDate();
+        << u"!01 Jan 2012 08:00:00 +0100!"_s << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 2822 invalid character at front, 2 at back")
-        << QString::fromLatin1("!01 Jan 2012 08:00:00 +0100..") << Qt::RFC2822Date << QDate();
+        << u"!01 Jan 2012 08:00:00 +0100.."_s << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 2822 invalid character 2 at front")
-        << QString::fromLatin1("!!01 Jan 2012 08:00:00 +0100") << Qt::RFC2822Date << QDate();
+        << u"!!01 Jan 2012 08:00:00 +0100"_s << Qt::RFC2822Date << QDate();
     // The common date text used by the "invalid character" tests, just to be
     // sure *it's* not what's invalid:
     QTest::newRow("RFC 2822 (not invalid)")
-        << QString::fromLatin1("01 Jan 2012 08:00:00 +0100")
+        << u"01 Jan 2012 08:00:00 +0100"_s
         << Qt::RFC2822Date << QDate(2012, 1, 1);
 
     // Test Qt::RFC2822Date format (RFC 850 and 1036, permissive).
-    QTest::newRow("RFC 850 and 1036") << QString::fromLatin1("Fri Feb 13 13:24:51 1987 +0100")
+    QTest::newRow("RFC 850 and 1036") << u"Fri Feb 13 13:24:51 1987 +0100"_s
         << Qt::RFC2822Date << QDate(1987, 2, 13);
     QTest::newRow("RFC 850 and 1036 after space")
-        << QString::fromLatin1(" Fri Feb 13 13:24:51 1987 +0100")
+        << u" Fri Feb 13 13:24:51 1987 +0100"_s
         << Qt::RFC2822Date << QDate(1987, 2, 13);
     // No timezone
-    QTest::newRow("RFC 850 and 1036 no timezone") << QString::fromLatin1("Thu Jan 01 00:12:34 1970")
+    QTest::newRow("RFC 850 and 1036 no timezone") << u"Thu Jan 01 00:12:34 1970"_s
         << Qt::RFC2822Date << epochDate();
     // No time specified
-    QTest::newRow("RFC 850 and 1036 date only") << QString::fromLatin1("Fri Nov 01 2002")
+    QTest::newRow("RFC 850 and 1036 date only") << u"Fri Nov 01 2002"_s
         << Qt::RFC2822Date << QDate(2002, 11, 1);
     // Test invalid characters.
     QTest::newRow("RFC 850 and 1036 invalid character at end")
-        << QString::fromLatin1("Sun Jan 01 08:00:00 2012 +0100!")
+        << u"Sun Jan 01 08:00:00 2012 +0100!"_s
         << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 850 and 1036 invalid character at front")
-        << QString::fromLatin1("!Sun Jan 01 08:00:00 2012 +0100")
+        << u"!Sun Jan 01 08:00:00 2012 +0100"_s
         << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 850 and 1036 invalid character both ends")
-        << QString::fromLatin1("!Sun Jan 01 08:00:00 2012 +0100!")
+        << u"!Sun Jan 01 08:00:00 2012 +0100!"_s
         << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 850 and 1036 invalid character at front, 2 at back")
-        << QString::fromLatin1("!Sun Jan 01 08:00:00 2012 +0100..")
+        << u"!Sun Jan 01 08:00:00 2012 +0100.."_s
         << Qt::RFC2822Date << QDate();
     QTest::newRow("RFC 850 and 1036 invalid character 2 at front")
-        << QString::fromLatin1("!!Sun Jan 01 08:00:00 2012 +0100")
+        << u"!!Sun Jan 01 08:00:00 2012 +0100"_s
         << Qt::RFC2822Date << QDate();
     // Again, check the text in the "invalid character" tests isn't the source of invalidity:
     QTest::newRow("RFC 850 and 1036 (not invalid)")
-        << QString::fromLatin1("Sun Jan 01 08:00:00 2012 +0100")
+        << u"Sun Jan 01 08:00:00 2012 +0100"_s
         << Qt::RFC2822Date << QDate(2012, 1, 1);
 
-    QTest::newRow("RFC empty") << QString::fromLatin1("") << Qt::RFC2822Date << QDate();
+    QTest::newRow("RFC empty") << u""_s << Qt::RFC2822Date << QDate();
 }
 
 void tst_QDate::fromStringDateFormat()
@@ -1303,6 +1338,7 @@ void tst_QDate::fromStringDateFormat()
     QCOMPARE(QDate::fromString(dateStr, dateFormat), expectedDate);
 }
 
+# if QT_CONFIG(datetimeparser)
 void tst_QDate::fromStringFormat_data()
 {
     QTest::addColumn<QString>("string");
@@ -1334,7 +1370,7 @@ void tst_QDate::fromStringFormat_data()
     QTest::newRow("year-match-1999") << u"1999:99"_s << u"yyyy:yy"_s << 1900 << QDate(1999, 1, 1);
     QTest::newRow("year-match-2099") << u"2099:99"_s << u"yyyy:yy"_s << 1900 << QDate(2099, 1, 1);
     QTest::newRow("year-match-2001") << u"2001:01"_s << u"yyyy:yy"_s << 1900 << QDate(2001, 1, 1);
-    QTest::newRow("year-match-1999") << u"99"_s << u"yy"_s << 1900 << QDate(1999, 1, 1);
+    QTest::newRow("just-yy-1999") << u"99"_s << u"yy"_s << 1900 << QDate(1999, 1, 1);
     QTest::newRow("just-yy-1901") << u"01"_s << u"yy"_s << 1900 << QDate(1901, 1, 1);
     QTest::newRow("just-yy-2001") << u"01"_s << u"yy"_s << 1970 << QDate(2001, 1, 1);
 
@@ -1390,15 +1426,15 @@ void tst_QDate::fromStringFormat_data()
 
     // Test unicode handling.
     QTest::newRow("Unicode in format string")
-            << QString(u8"2020🤣09🤣21") << QString(u8"yyyy🤣MM🤣dd") << 1900 << QDate(2020, 9, 21);
+            << u"2020🤣09🤣21"_s << u"yyyy🤣MM🤣dd"_s << 1900 << QDate(2020, 9, 21);
     QTest::newRow("Unicode-in-format-string-quoted-emoji")
-            << QString(u8"🤣🤣2020👍09🤣21") << QString(u8"'🤣🤣'yyyy👍MM🤣dd") << 1900
+            << u"🤣🤣2020👍09🤣21"_s << u"'🤣🤣'yyyy👍MM🤣dd"_s << 1900
             << QDate(2020, 9, 21);
     QTest::newRow("Unicode-in-quoted-dd-format-string")
-            << QString(u8"🤣🤣2020👍09🤣21dd") << QString(u8"🤣🤣yyyy👍MM🤣dd'dd'") << 1900
+            << u"🤣🤣2020👍09🤣21dd"_s << u"🤣🤣yyyy👍MM🤣dd'dd'"_s << 1900
             << QDate(2020, 9, 21);
     QTest::newRow("Unicode-in-all-formats-quoted-string")
-            << QString(u8"🤣🤣yyyy2020👍MM09🤣21dd") << QString(u8"🤣🤣'yyyy'yyyy👍'MM'MM🤣dd'dd'")
+            << u"🤣🤣yyyy2020👍MM09🤣21dd"_s << u"🤣🤣'yyyy'yyyy👍'MM'MM🤣dd'dd'"_s
             << 1900 << QDate(2020, 9, 21);
 
     // QTBUG-84334
@@ -1417,6 +1453,13 @@ void tst_QDate::fromStringFormat_data()
             << u"05--2006-21"_s << u"MM-yyyy-dd"_s << 1900 << QDate(-2006, 5, 21);
     QTest::newRow("-ve year: back, dash")
             << u"05-21--2006"_s << u"MM-dd-yyyy"_s << 1900 << QDate(-2006, 5, 21);
+    // zero year number is invalid
+    QTest::newRow("year-zero-front")
+            << u"0000-05-21"_s << u"yyyy-MM-dd"_s << 1900 << QDate();
+    QTest::newRow("year-zero-mid")
+            << u"05-0000-21"_s << u"MM-yyyy-dd"_s << 1900 << QDate();
+    QTest::newRow("year-zero-back")
+            << u"05-21-0000"_s << u"MM-dd-yyyy"_s << 1900 << QDate();
     // negative three digit year numbers should be rejected:
     QTest::newRow("-ve 3digit year: front")
             << u"-206-05-21"_s << u"yyyy-MM-dd"_s << 1900 << QDate();
@@ -1463,6 +1506,8 @@ void tst_QDate::fromStringFormat_data()
             << u"05-00206-21"_s << u"MM-yyyy-dd"_s << 1900 << QDate();
     QTest::newRow("5digit year, back")
             << u"05-21-00206"_s << u"MM-dd-yyyy"_s << 1900 << QDate();
+    QTest::newRow("non-leap-feb-29") // QTBUG-132115: should fail but not assert
+            << u"290215"_s << u"ddMMyy"_s << 1900 << QDate();
 
     QTest::newRow("dash separator, no year at end")
             << u"05-21-"_s << u"dd-MM-yyyy"_s << 1900 << QDate();
@@ -1490,18 +1535,17 @@ void tst_QDate::fromStringFormat()
 }
 #endif // datetimeparser
 
-#if QT_CONFIG(datestring)
 void tst_QDate::toStringFormat_data()
 {
     QTest::addColumn<QDate>("t");
     QTest::addColumn<QString>("format");
     QTest::addColumn<QString>("str");
 
-    QTest::newRow( "data0" ) << QDate(1995,5,20) << QString("d-M-yy") << QString("20-5-95");
-    QTest::newRow( "data1" ) << QDate(2002,12,17) << QString("dd-MM-yyyy") << QString("17-12-2002");
-    QTest::newRow( "data2" ) << QDate(1995,5,20) << QString("M-yy") << QString("5-95");
-    QTest::newRow( "data3" ) << QDate(2002,12,17) << QString("dd") << QString("17");
-    QTest::newRow( "data4" ) << QDate() << QString("dd-mm-yyyy") << QString();
+    QTest::newRow( "data0" ) << QDate(1995,5,20) << u"d-M-yy"_s << u"20-5-95"_s;
+    QTest::newRow( "data1" ) << QDate(2002,12,17) << u"dd-MM-yyyy"_s << u"17-12-2002"_s;
+    QTest::newRow( "data2" ) << QDate(1995,5,20) << u"M-yy"_s << u"5-95"_s;
+    QTest::newRow( "data3" ) << QDate(2002,12,17) << u"dd"_s << u"17"_s;
+    QTest::newRow( "data4" ) << QDate() << u"dd-mm-yyyy"_s << QString();
 }
 
 void tst_QDate::toStringFormat()
@@ -1519,14 +1563,14 @@ void tst_QDate::toStringDateFormat_data()
     QTest::addColumn<Qt::DateFormat>("format");
     QTest::addColumn<QString>("expectedStr");
 
-    QTest::newRow("data0") << QDate(1,1,1) << Qt::ISODate << QString("0001-01-01");
-    QTest::newRow("data1") << QDate(11,1,1) << Qt::ISODate << QString("0011-01-01");
-    QTest::newRow("data2") << QDate(111,1,1) << Qt::ISODate << QString("0111-01-01");
-    QTest::newRow("data3") << QDate(1974,12,1) << Qt::ISODate << QString("1974-12-01");
+    QTest::newRow("data0") << QDate(1,1,1) << Qt::ISODate << u"0001-01-01"_s;
+    QTest::newRow("data1") << QDate(11,1,1) << Qt::ISODate << u"0011-01-01"_s;
+    QTest::newRow("data2") << QDate(111,1,1) << Qt::ISODate << u"0111-01-01"_s;
+    QTest::newRow("data3") << QDate(1974,12,1) << Qt::ISODate << u"1974-12-01"_s;
     QTest::newRow("year < 0") << QDate(-1,1,1) << Qt::ISODate << QString();
     QTest::newRow("year > 9999") << QDate(10000, 1, 1) << Qt::ISODate << QString();
-    QTest::newRow("RFC2822Date") << QDate(1974,12,1) << Qt::RFC2822Date << QString("01 Dec 1974");
-    QTest::newRow("ISODateWithMs") << QDate(1974,12,1) << Qt::ISODateWithMs << QString("1974-12-01");
+    QTest::newRow("RFC2822Date") << QDate(1974,12,1) << Qt::RFC2822Date << u"01 Dec 1974"_s;
+    QTest::newRow("ISODateWithMs") << QDate(1974,12,1) << Qt::ISODateWithMs << u"1974-12-01"_s;
 }
 
 void tst_QDate::toStringDateFormat()
@@ -1626,10 +1670,10 @@ void tst_QDate::printNegativeYear_data() const
 {
     QTest::addColumn<int>("year");
     QTest::addColumn<QString>("expect");
-    QTest::newRow("millennium") << -1000 << QStringLiteral("-1000");
-    QTest::newRow("century") << -500 << QStringLiteral("-0500");
-    QTest::newRow("decade") << -20 << QStringLiteral("-0020");
-    QTest::newRow("year") << -7 << QStringLiteral("-0007");
+    QTest::newRow("millennium") << -1000 << u"-1000"_s;
+    QTest::newRow("century") << -500 << u"-0500"_s;
+    QTest::newRow("decade") << -20 << u"-0020"_s;
+    QTest::newRow("year") << -7 << u"-0007"_s;
 }
 
 void tst_QDate::printNegativeYear() const
@@ -1641,7 +1685,7 @@ void tst_QDate::printNegativeYear() const
     QDate date(year, 3, 4);
     QVERIFY(date.isValid());
     QCOMPARE(date.year(), year);
-    QCOMPARE(date.toString(QLatin1String("yyyy")), expect);
+    QCOMPARE(date.toString(u"yyyy"), expect);
 }
 
 #if QT_CONFIG(datestring)
@@ -1713,8 +1757,8 @@ void tst_QDate::roundtrip() const
         QCOMPARE(loopDate.toJulianDay(), testDate.toJulianDay());
         loopDate = loopDate.addDays(1);
     }
-
-#if __cpp_lib_chrono >= 201907L
+// INTEGRITY incident-85878 (timezone and clock_cast are not supported)
+#if __cpp_lib_chrono >= 201907L && !defined(Q_OS_INTEGRITY)
     // Test roundtrip for from/to std::chrono conversions.
     // Compile-time test, to verify it's all constexpr.
     using namespace std::chrono;
