@@ -56,6 +56,7 @@ private slots:
     void restoreDockWidget();
     void restoreStateWhileStillFloating();
     void setWindowTitle();
+    void windowIcon();
 
     // task specific tests:
     void task165177_deleteFocusWidget();
@@ -915,7 +916,8 @@ void tst_QDockWidget::titleBarDoubleClick()
     win.show();
     dock.setFloating(true);
 
-    QEvent e(QEvent::NonClientAreaMouseButtonDblClick);
+    QMouseEvent e(QEvent::NonClientAreaMouseButtonDblClick, {}, {}, {}, {}, {}, {}, {},
+                  QPointingDevice::primaryPointingDevice());
     QApplication::sendEvent(&dock, &e);
     QVERIFY(dock.isFloating());
     QCOMPARE(win.dockWidgetArea(&dock), Qt::NoDockWidgetArea);
@@ -1248,6 +1250,28 @@ void tst_QDockWidget::setWindowTitle()
     QCOMPARE(dock2.windowTitle(), closedDock2);
 }
 
+void tst_QDockWidget::windowIcon()
+{
+    QPixmap pm(5, 5);
+    pm.fill(Qt::red);
+    const QIcon icon(pm);
+    pm.fill(Qt::green);
+    const QIcon appIcon(pm);
+    qApp->setWindowIcon(appIcon);
+    QMainWindow mainWindow;
+    mainWindow.setCentralWidget(new QWidget);
+    auto *d1 = new QDockWidget;
+    auto *d2 = new QDockWidget;
+    d2->setWindowIcon(icon);
+    mainWindow.addDockWidget(Qt::TopDockWidgetArea, d1);
+    mainWindow.addDockWidget(Qt::TopDockWidgetArea, d2);
+    mainWindow.tabifyDockWidget(d1, d2);
+    auto *bar = mainWindow.findChild<QTabBar *>();
+    Q_ASSERT(bar->count() == 2);
+    QCOMPARE(bar->tabIcon(0), QIcon());
+    QCOMPARE(bar->tabIcon(1), icon);
+}
+
 // helpers for dockPermissions, hideAndShow, closeAndDelete
 #ifdef QT_BUILD_INTERNAL
 void tst_QDockWidget::createTestWidgets(QMainWindow* &mainWindow, QPointer<QWidget> &cent, QPointer<QDockWidget> &d1, QPointer<QDockWidget> &d2) const
