@@ -1,38 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
-
-
-#include <QtTest/QtTest>
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "qmdisubwindow.h"
 #include "private/qmdisubwindow_p.h"
 #include "qmdiarea.h"
 
+#include <QTest>
 #include <QLayout>
 #include <QLineEdit>
 #include <QMainWindow>
@@ -47,9 +20,12 @@
 #include <QStyleOptionTitleBar>
 #include <QPushButton>
 #include <QScreen>
+#include <QScrollBar>
 #include <QSizeGrip>
+#include <QSignalSpy>
+#include <QList>
 
-#include <QVector>
+#include <QtWidgets/private/qapplication_p.h>
 
 QT_BEGIN_NAMESPACE
 extern bool qt_tab_all_widgets();
@@ -152,6 +128,22 @@ Q_DECLARE_METATYPE(Qt::WindowType);
 Q_DECLARE_METATYPE(Qt::WindowFlags);
 Q_DECLARE_METATYPE(QMdiSubWindow*);
 
+class TestPushButton : public QPushButton
+{
+public:
+    TestPushButton(const QString &title, QWidget *parent = nullptr)
+    : QPushButton(title, parent)
+    {
+    }
+
+protected:
+    // don't rely on style-specific button behavior in test
+    bool hitButton(const QPoint &point) const override
+    {
+        return rect().contains(point);
+    }
+};
+
 class tst_QMdiSubWindow : public QObject
 {
     Q_OBJECT
@@ -207,10 +199,14 @@ private slots:
     void styleChange();
     void testFullScreenState();
     void testRemoveBaseWidget();
+    void testRespectMinimumSize();
 };
 
 void tst_QMdiSubWindow::initTestCase()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: Almost all of these fail. Figure out why.");
+
     qRegisterMetaType<Qt::WindowStates>("Qt::WindowStates");
     // Avoid unnecessary waits for empty top level widget lists when
     // testing menus.
@@ -236,25 +232,33 @@ void tst_QMdiSubWindow::sizeHint()
 
 void tst_QMdiSubWindow::minimumSizeHint()
 {
-    const auto globalStrut = QApplication::globalStrut();
+    class Widget : public QWidget
+    {
+    public:
+        Widget() = default;
+
+        QSize minimumSizeHint() const override
+        {
+            return QSize(100, 100);
+        }
+
+    };
     QMdiSubWindow window;
     window.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
     window.show();
 
-    QCOMPARE(window.minimumSizeHint(), globalStrut);
+    QCOMPARE(window.minimumSizeHint(), QSize(0, 0));
 
     window.setWidget(new QWidget);
-    QCOMPARE(window.minimumSizeHint(), window.layout()->minimumSize()
-                                       .expandedTo(globalStrut));
+    QCOMPARE(window.minimumSizeHint(), window.layout()->minimumSize());
 
     delete window.widget();
     delete window.layout();
-    window.setWidget(new QWidget);
-    QCOMPARE(window.minimumSizeHint(), globalStrut);
+    window.setWidget(new Widget);
+    QCOMPARE(window.minimumSizeHint(), QSize(0, 0));
 
     window.widget()->show();
-    QCOMPARE(window.minimumSizeHint(), window.widget()->minimumSizeHint()
-                                       .expandedTo(globalStrut));
+    QCOMPARE(window.minimumSizeHint(), window.widget()->minimumSizeHint());
 }
 
 void tst_QMdiSubWindow::minimumSize()
@@ -374,20 +378,19 @@ void tst_QMdiSubWindow::setWindowState()
 
 void tst_QMdiSubWindow::mainWindowSupport()
 {
-    QVector<QMdiSubWindow *> windows;
+    QList<QMdiSubWindow *> windows;
     QMdiArea *workspace = new QMdiArea;
     QMainWindow mainWindow;
     mainWindow.setCentralWidget(workspace);
     mainWindow.show();
     mainWindow.menuBar()->setVisible(true);
-    QApplication::setActiveWindow(&mainWindow);
     bool nativeMenuBar = mainWindow.menuBar()->isNativeMenuBar();
 
     // QMainWindow's window title is empty, so on a platform which does NOT have a native menubar,
     // the maximized subwindow's title is imposed onto the main window's titlebar.
     if (!nativeMenuBar) {
         QCOMPARE(mainWindow.windowTitle(), QString());
-        QMdiSubWindow *window = workspace->addSubWindow(new QPushButton(QLatin1String("Test")));
+        QMdiSubWindow *window = workspace->addSubWindow(new TestPushButton(QLatin1String("Test")));
         QString expectedTitle = QLatin1String("MainWindow's title is empty");
         window->setWindowTitle(expectedTitle);
         QCOMPARE(window->windowTitle(), expectedTitle);
@@ -469,7 +472,7 @@ void tst_QMdiSubWindow::mainWindowSupport()
 
     workspace->activateNextSubWindow();
     QCoreApplication::processEvents();
-    for (QMdiSubWindow *window : qAsConst(windows)) {
+    for (QMdiSubWindow *window : std::as_const(windows)) {
         QCOMPARE(workspace->activeSubWindow(), window);
         QVERIFY(window->isMaximized());
         QVERIFY(window->maximizedButtonsWidget());
@@ -505,8 +508,8 @@ void tst_QMdiSubWindow::emittingOfSignals()
     QMdiArea workspace;
     workspace.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
     workspace.show();
+    QVERIFY(QTest::qWaitFor([&]{ return workspace.isActiveWindow(); }));
     QCoreApplication::processEvents();
-    QApplication::setActiveWindow(&workspace);
     QMdiSubWindow *window = qobject_cast<QMdiSubWindow *>(workspace.addSubWindow(new QWidget));
     QCoreApplication::processEvents();
     window->show();
@@ -526,9 +529,9 @@ void tst_QMdiSubWindow::emittingOfSignals()
 
     int count = 0;
     if (signal == SIGNAL(aboutToActivate())) {
-        count += spy.count();
+        count += spy.size();
     } else {
-        for (int i = 0; i < spy.count(); ++i) {
+        for (int i = 0; i < spy.size(); ++i) {
             Qt::WindowStates oldState = qvariant_cast<Qt::WindowStates>(spy.at(i).at(0));
             Qt::WindowStates newState = qvariant_cast<Qt::WindowStates>(spy.at(i).at(1));
             if (watchedState != Qt::WindowNoState) {
@@ -542,9 +545,6 @@ void tst_QMdiSubWindow::emittingOfSignals()
             }
         }
     }
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("windowMaximized", "Broken on WinRT - QTBUG-68297", Abort);
-#endif
     QCOMPARE(count, 1);
 
     window->setParent(nullptr);
@@ -554,7 +554,7 @@ void tst_QMdiSubWindow::emittingOfSignals()
 
     spy.clear();
     triggerSignal(window, &workspace, signal);
-    QCOMPARE(spy.count(), 0);
+    QCOMPARE(spy.size(), 0);
 
     delete window;
     window = nullptr;
@@ -571,9 +571,6 @@ void tst_QMdiSubWindow::showShaded()
     QVERIFY(QTest::qWaitForWindowExposed(&workspace));
 
     QVERIFY(!window->isShaded());
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Windows are maximized per default on WinRt ", Abort);
-#endif
     QVERIFY(!window->isMaximized());
 
     QCOMPARE(window->size(), QSize(300, 300));
@@ -667,10 +664,6 @@ void tst_QMdiSubWindow::showNormal()
     QCoreApplication::processEvents();
     window->showNormal();
     QCoreApplication::processEvents();
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("showMinimized", "Windows are maximized per default on WinRt ", Abort);
-    QEXPECT_FAIL("showMaximized", "Windows are maximized per default on WinRt ", Abort);
-#endif
     QCOMPARE(window->geometry(), originalGeometry);
 }
 
@@ -754,7 +747,7 @@ void tst_QMdiSubWindow::setOpaqueResizeAndMove()
 
     // we need to wait for the resizeTimer to make sure updateDirtyRegions is called
     auto priv = static_cast<QMdiSubWindowPrivate*>(qt_widget_private(window));
-    QTRY_COMPARE(priv->resizeTimerId, -1);
+    QTRY_COMPARE(priv->resizeTimer.id(), Qt::TimerId::Invalid);
 
     // Enter resize mode.
     int offset = window->style()->pixelMetric(QStyle::PM_MdiSubWindowFrameWidth) / 2;
@@ -781,9 +774,6 @@ void tst_QMdiSubWindow::setOpaqueResizeAndMove()
 
     // Leave resize mode
     sendMouseRelease(mouseReceiver, mousePosition);
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Fails on WinRT - QTBUG-68297", Abort);
-#endif
     QCOMPARE(resizeSpy.count(), expectedGeometryCount);
     QCOMPARE(window->size(), windowSize + QSize(geometryCount, geometryCount));
     }
@@ -950,9 +940,6 @@ void tst_QMdiSubWindow::mouseDoubleClick()
     workspace.show();
     window->show();
 
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Windows are maximized per default on WinRt ", Abort);
-#endif
     QVERIFY(!window->isMaximized());
     QVERIFY(!window->isShaded());
 
@@ -966,6 +953,8 @@ void tst_QMdiSubWindow::mouseDoubleClick()
     if (!window->style()->styleHint(QStyle::SH_TitleBar_NoBorder, &options, window))
         height += window->isMinimized() ? 8 : 4;
     QPoint mousePosition(window->width() / 2, height - 1);
+    if (window->style()->inherits("QWindows11Style"))
+        mousePosition = QPoint(8, height - 1);
     sendMouseMove(window, mousePosition, Qt::NoButton);
 
     // Without Qt::WindowShadeButtonHint flag set
@@ -993,8 +982,10 @@ void tst_QMdiSubWindow::mouseDoubleClick()
 
     window->showMinimized();
     QVERIFY(window->isMinimized());
+    //Process QEvent::WindowStateChange
+    QCoreApplication::processEvents();
     sendMouseDoubleClick(window, mousePosition);
-    QVERIFY(!window->isMinimized());
+    QTRY_VERIFY(!window->isMinimized());
     QCOMPARE(window->geometry(), originalGeometry);
 }
 
@@ -1026,9 +1017,6 @@ void tst_QMdiSubWindow::setSystemMenu()
     QVERIFY(!QApplication::activePopupWidget());
     subWindow->showSystemMenu();
     QTRY_COMPARE(QApplication::activePopupWidget(), qobject_cast<QWidget *>(systemMenu));
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Broken on WinRT - QTBUG-68297", Abort);
-#endif
     QTRY_COMPARE(systemMenu->mapToGlobal(QPoint(0, 0)),
                  (globalPopupPos = subWindow->mapToGlobal(subWindow->contentsRect().topLeft())) );
 
@@ -1047,7 +1035,7 @@ void tst_QMdiSubWindow::setSystemMenu()
     subWindow->setSystemMenu(systemMenu);
     QCOMPARE(subWindow->systemMenu(), qobject_cast<QMenu *>(systemMenu));
     QCOMPARE(subWindow->systemMenu()->parentWidget(), static_cast<QWidget *>(subWindow));
-    QCOMPARE(subWindow->systemMenu()->actions().count(), 1);
+    QCOMPARE(subWindow->systemMenu()->actions().size(), 1);
 
     // Show the new system menu
     QVERIFY(!QApplication::activePopupWidget());
@@ -1151,7 +1139,7 @@ void tst_QMdiSubWindow::restoreFocus()
     topArea.show();
     box->show();
 
-    QApplication::setActiveWindow(&topArea);
+    QApplicationPrivate::setActiveWindow(&topArea);
     QMdiSubWindow *expectedFocusWindow = nestedWorkspace->subWindowList().last();
     QVERIFY(expectedFocusWindow);
     QVERIFY(expectedFocusWindow->widget());
@@ -1243,7 +1231,6 @@ void tst_QMdiSubWindow::restoreFocusOverCreation()
     subWidget1->m_lineEdit2->setFocus();
     subWindow1->show();
     mdiArea.show();
-    QApplication::setActiveWindow(&mdiArea);
     QVERIFY(QTest::qWaitForWindowActive(&mdiArea));
     QCOMPARE(QApplication::focusWidget(), subWidget1->m_lineEdit2);
 
@@ -1252,9 +1239,6 @@ void tst_QMdiSubWindow::restoreFocusOverCreation()
     QTRY_COMPARE(QApplication::focusWidget(), subWidget2->m_lineEdit1);
 
     mdiArea.setActiveSubWindow(subWindow1);
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Broken on WinRt - QTBUG-68297", Abort);
-#endif
     QTRY_COMPARE(QApplication::focusWidget(), subWidget1->m_lineEdit2);
 }
 
@@ -1274,9 +1258,9 @@ void tst_QMdiSubWindow::changeFocusWithTab()
     mdiArea.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
     mdiArea.addSubWindow(widget);
     mdiArea.show();
-    QCOMPARE(mdiArea.subWindowList().count(), 1);
+    QCOMPARE(mdiArea.subWindowList().size(), 1);
 
-    QApplication::setActiveWindow(&mdiArea);
+    QApplicationPrivate::setActiveWindow(&mdiArea);
     QCOMPARE(QApplication::focusWidget(), static_cast<QWidget *>(firstLineEdit));
 
     // Next
@@ -1295,7 +1279,7 @@ void tst_QMdiSubWindow::changeFocusWithTab()
     QTest::keyPress(widget, Qt::Key_Backtab);
     QCOMPARE(QApplication::focusWidget(), static_cast<QWidget *>(firstLineEdit));
 
-    QMdiSubWindow *window = mdiArea.addSubWindow(new QPushButton);
+    QMdiSubWindow *window = mdiArea.addSubWindow(new TestPushButton(QLatin1String("test")));
     window->show();
     QCOMPARE(mdiArea.activeSubWindow(), window);
 
@@ -1356,7 +1340,7 @@ void tst_QMdiSubWindow::closeEvent()
 
     QVERIFY(window->close());
     QCOMPARE(closeSpy.count(), 3);
-    QCOMPARE(mdiArea.subWindowList().count(), 0);
+    QCOMPARE(mdiArea.subWindowList().size(), 0);
 }
 
 // There exists more tests in QMdiArea which covers window title support
@@ -1391,7 +1375,7 @@ void tst_QMdiSubWindow::setWindowTitle()
     // other widgets which are not real top-level widgets).
     QCOMPARE(window->windowTitle(), expectedWindowTitle);
 
-    textEdit->setWindowModified(true);;
+    textEdit->setWindowModified(true);
     expectedWindowTitle = QLatin1String("Override child title");
     window->setWindowTitle(expectedWindowTitle);
     QVERIFY(window->isWindowModified());
@@ -1467,9 +1451,6 @@ void tst_QMdiSubWindow::resizeEvents()
     QCOMPARE(window->widget()->windowState(), windowState);
 
     // Make sure we got as many resize events as expected.
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("maximized", "Broken on WinRT - QTBUG-68297", Abort);
-#endif
     QCOMPARE(windowResizeEventSpy.count(), expectedWindowResizeEvents);
     QCOMPARE(widgetResizeEventSpy.count(), expectedWidgetResizeEvents);
     windowResizeEventSpy.clear();
@@ -1479,10 +1460,6 @@ void tst_QMdiSubWindow::resizeEvents()
     window->showNormal();
 
     // Check that the window state is correct.
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("minimized", "Broken on WinRT - QTBUG-68297", Abort);
-    QEXPECT_FAIL("shaded", "Broken on WinRT - QTBUG-68297", Abort);
-#endif
     QCOMPARE(window->windowState(), Qt::WindowNoState | Qt::WindowActive);
     QCOMPARE(window->widget()->windowState(), Qt::WindowNoState);
 
@@ -1571,9 +1548,6 @@ void tst_QMdiSubWindow::hideAndShow()
 
 #if !defined (Q_OS_DARWIN)
     QVERIFY(menuBar->cornerWidget(Qt::TopRightCorner));
-#if defined Q_OS_QNX
-    QEXPECT_FAIL("", "QTBUG-38231", Abort);
-#endif
     QVERIFY(subWindow->maximizedButtonsWidget());
     QVERIFY(subWindow->maximizedSystemMenuIconWidget());
     QCOMPARE(menuBar->cornerWidget(Qt::TopRightCorner), subWindow->maximizedButtonsWidget());
@@ -1749,9 +1723,6 @@ void tst_QMdiSubWindow::fixedMinMaxSize()
     QCOMPARE(subWindow->maximumSize(), maximumSize);
     mdiArea.addSubWindow(subWindow);
     subWindow->show();
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Windows are maximized per default on WinRt ", Abort);
-#endif
     QCOMPARE(subWindow->size(), minimumSize);
 
     // Calculate the size of a minimized sub window.
@@ -1807,9 +1778,6 @@ void tst_QMdiSubWindow::replaceMenuBarWhileMaximized()
 
     QCoreApplication::processEvents();
 
-#if defined Q_OS_QNX
-    QEXPECT_FAIL("", "QTBUG-38231", Abort);
-#endif
     QVERIFY(subWindow->maximizedButtonsWidget());
     QVERIFY(subWindow->maximizedSystemMenuIconWidget());
     QCOMPARE(menuBar1->cornerWidget(Qt::TopLeftCorner), subWindow->maximizedSystemMenuIconWidget());
@@ -1901,10 +1869,9 @@ void tst_QMdiSubWindow::closeOnDoubleClick()
 
 void tst_QMdiSubWindow::setFont()
 {
-    QSKIP("This test function is unstable in CI, please see QTBUG-22544");
     QMdiArea mdiArea;
     mdiArea.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
-    QMdiSubWindow *subWindow = mdiArea.addSubWindow(new QPushButton(QLatin1String("test")));
+    QMdiSubWindow *subWindow = mdiArea.addSubWindow(new TestPushButton(QLatin1String("test")));
     subWindow->resize(300, 100);
     subWindow->setWindowTitle(QLatin1String("Window title"));
     mdiArea.show();
@@ -1987,7 +1954,6 @@ void tst_QMdiSubWindow::task_182852()
     mainWindow.setCentralWidget(workspace);
     mainWindow.show();
     mainWindow.menuBar()->setVisible(true);
-    QApplication::setActiveWindow(&mainWindow);
     if (mainWindow.menuBar()->isNativeMenuBar())
         return; // The main window's title is not overwritten if we have a native menubar (macOS, Unity etc.)
 
@@ -2054,19 +2020,19 @@ void tst_QMdiSubWindow::task_233197()
     QMenuBar *menuBar = mainWindow->menuBar(); // force creation of a menubar
     Q_UNUSED(menuBar);
 
-    QPushButton *focus1 = new QPushButton(QLatin1String("Focus 1"), mainWindow);
+    QPushButton *focus1 = new TestPushButton(QLatin1String("Focus 1"), mainWindow);
     QObject::connect(focus1, &QAbstractButton::clicked, subWindow1,
                      QOverload<>::of(&QWidget::setFocus));
     focus1->move(5, 30);
     focus1->show();
 
-    QPushButton *focus2 = new QPushButton(QLatin1String("Focus 2"), mainWindow);
+    QPushButton *focus2 = new TestPushButton(QLatin1String("Focus 2"), mainWindow);
     QObject::connect(focus2, &QAbstractButton::clicked, subWindow2,
                      QOverload<>::of(&QWidget::setFocus));
     focus2->move(5, 60);
     focus2->show();
 
-    QPushButton *close = new QPushButton(QLatin1String("Close"), mainWindow);
+    QPushButton *close = new TestPushButton(QLatin1String("Close"), mainWindow);
     QObject::connect(close, &QAbstractButton::clicked, mainWindow, &QWidget::close);
     close->move(5, 90);
     close->show();
@@ -2137,7 +2103,7 @@ void tst_QMdiSubWindow::styleChange()
 
     // subWindowActivated should NOT be activated by a style change,
     // even if internally QMdiSubWindow un-minimizes subwindows temporarily.
-    QCOMPARE(spy.count(), 0);
+    QCOMPARE(spy.size(), 0);
 }
 
 void tst_QMdiSubWindow::testFullScreenState()
@@ -2151,9 +2117,6 @@ void tst_QMdiSubWindow::testFullScreenState()
     subWindow->showFullScreen(); // QMdiSubWindow does not support the fullscreen state. This call
                                  // should be equivalent to setVisible(true) (and not showNormal())
     QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
-#ifdef Q_OS_WINRT
-    QEXPECT_FAIL("", "Windows are maximized per default on WinRt ", Abort);
-#endif
     QCOMPARE(subWindow->size(), QSize(300, 300));
 }
 
@@ -2179,6 +2142,36 @@ void tst_QMdiSubWindow::testRemoveBaseWidget()
     delete widget1;
 }
 
+void tst_QMdiSubWindow::testRespectMinimumSize()  // QTBUG-100494
+{
+    QMdiArea mdiArea;
+    mdiArea.resize(400, 400);
+    mdiArea.setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    mdiArea.setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+
+    auto vlay = new QVBoxLayout;
+    vlay->addWidget(new QPushButton(QLatin1String("btn1-1")));
+    vlay->addSpacerItem(new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding));
+    vlay->addWidget(new QPushButton(QLatin1String("btn1-2")));
+    auto w1 = new QWidget;
+    w1->setLayout(vlay);
+    w1->resize(300, 200);
+    w1->setMinimumSize(200, 150);
+    auto sw = new QMdiSubWindow;
+    sw->setWidget(w1);
+    sw->resize(w1->size());
+    mdiArea.addSubWindow(sw);
+    sw->showMaximized();
+
+    mdiArea.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
+    QVERIFY(!mdiArea.horizontalScrollBar()->isVisible());
+    QVERIFY(!mdiArea.verticalScrollBar()->isVisible());
+    mdiArea.resize(150, 100);
+    QTRY_VERIFY(mdiArea.horizontalScrollBar()->isVisible());
+    QTRY_VERIFY(mdiArea.verticalScrollBar()->isVisible());
+}
+
 QTEST_MAIN(tst_QMdiSubWindow)
 #include "tst_qmdisubwindow.moc"
 
